'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

// FILE: ./assets/js/utilities/UtilitarioGeral.js
function UtilitarioGeral(ZHPromise, ScreenService, templateManager, ZhUtil, ValidationEngine, restEngine, metaDataFactory) {

    var self = this;

    var capitalize = 'C',
        uppercase = 'U',
        lowercase = 'L',
        domainNames = {
        "product": "produto",
        "store": "loja",
        "filial": "filial",
        "client": "cliente"
    };

    // Domain Name
    this.getDomainName = function (domain, type) {
        switch (type) {
            case capitalize:
                return domainNames[domain].replace(/\b\w/gi, function (letter) {
                    return letter.toUpperCase();
                });

            case uppercase:
                return domainNames[domain].toUpperCase();

            case lowercase:
                return domainNames[domain].toLowerCase();

            default:
                return domainNames[domain];
        }
    };

    ValidationEngine.registerValidation("intervalNotEmpty", function (value, validationAttrs) {
        var valid = false;
        if (value) {
            if (value.period) {
                valid = true;
            } else {
                if (value.start && value.end) {
                    valid = true;
                }
            }
        }
        return {
            valid: valid,
            message: ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Required field")
        };
    });

    // Receives an Array with the name of the messages you want. You can join messages.
    this.getDefaultMessages = function (msg_array) {
        var str = '';
        msg_array.forEach(function (item) {
            str += self.defaultMessages[item] + ' ';
        });
        return str;
    };

    // Use this for messages that are repeated during the code
    this.defaultMessages = {
        "prodFilter": "O valor inicial não pode ser maior que o final!",
        "invalidWidget": "Todos os campos obrigatórios devem ser preenchidos!",
        "onlyAlphaNumeric": "Por favor use apenas letras e números!",
        "uniqueCD": "Esse código já existe!",
        "uniqueModelo": "Esse modelo já foi cadastrado!",
        "invalidCD": "O código inserido é inválido!",
        "invalidDelete": "Não foi possível deletar",
        "askForPDV": "Deseja inserir PDVs?",
        "askForContinue": "Deseja prosseguir ?",
        "falseIp": "O IP é falso.",
        "portNotAvaible": "Esta porta já está em uso!",
        "deleteError": "Não foi possível deletar",
        "deleteSuccess": "Registros deletados com sucesso",
        "uniquePrinter": "Essa impressora já está cadastrada!"
    };

    var messagesWidget = {
        "id": 0,
        "name": "messagesWidget",
        "label": "",
        "messagesDataset": "",
        "template": "widget/form.html",
        "container": {},
        "isVisible": true,
        "popupNoBlock": false,
        "fields": [],
        "actions": [],
        "events": []
    };

    var messagesFields = [{
        "label": "",
        "name": "CONTENT",
        "readOnly": true,
        "template": "field/textarea.html",
        "textRows": 14,
        "isVisible": true,
        "class": 12,
        "descriptionOrder": true,
        "showOnView": true,
        "showOnForm": true,
        "showOnList": true
    }];

    var messagesActions = [{
        "name": "cancelar",
        "label": "Cancelar",
        "idVisible": true,
        "hideIcon": true,
        "showAsAction": "back",
        "events": [{
            "name": "ActionEvent",
            "code": "ScreenService.closePopup();"
        }]
    }, {
        "name": "exportar",
        "label": "Exportar",
        "idVisible": true,
        "icon": "export",
        "showAsAction": "always",
        "events": [{
            "name": "ActionEvent",
            "code": "UtilitarioGeral.generateFileByDataSet(args.owner.widget.messagesDataset,{fileExtension: 'txt', fileName: args.owner.widget.label} )"
        }]
    }];

    this.showErrorMessages = function (widget, dataset, popupName) {
        messagesWidget.container = widget.container;
        messagesWidget.fields = messagesFields;
        messagesWidget.actions = messagesActions;

        var messagesWidgetInitialized = metaDataFactory.widgetFactory(messagesWidget);
        var content = '';
        if (popupName !== undefined) messagesWidgetInitialized.label = popupName;
        messagesWidgetInitialized.messagesDataset = dataset;
        dataset.forEach(function (message) {
            content += message + '\n';
        });
        messagesWidgetInitialized.getField('CONTENT').setValue(content);
        ScreenService.openPopup(messagesWidgetInitialized);
    };

    this.generateFileByDataSet = function (dataset, fileProperties) {
        var content = "";
        var name = '';
        var extension = 'text/plain';

        if (fileProperties !== undefined) {
            if (fileProperties.fileName !== undefined) name = fileProperties.fileName;
        }

        dataset.forEach(function (line) {
            content += line + "\r\n";
        });

        var element = document.createElement('a');
        element.setAttribute('href', 'data:' + extension + ';charset=utf-8,' + encodeURIComponent(content));
        element.setAttribute('download', name);

        element.style.display = 'none';
        document.body.appendChild(element);

        element.click();

        document.body.removeChild(element);
    };

    // Util now:

    /*
    	prodFilter: Tela de Loja, onde tem um filtro de produtos
    	isValid: Todos os campos obrigatórios devem ser preenchidos
    	onlyAlphaNumeric: Por favor use apenas letras e números!
    	uniqueCD: O código deve ser único
    	isValidCD: O código deve ser válido 
    	askForPDV: adicionar pontos de venda 
    	askForContinue: msg para continuar
    	falseIp: quando o Ip não bate com a regex...
    */

    this.modeViewOn = function (widget) {
        widget.view();
    };

    this.modeListOn = function (widget) {
        widget.list();
    };

    this.editModeOn = function (widget) {
        widget.edit();
    };

    this.cancelEditForm = function (widget) {
        widget.view();
        widget.list();
    };

    var requestModulesMetadata = function requestModulesMetadata() {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };

    this.voltarDashboard = function () {
        this.openWindow('product#dashboard');
    };

    this.openWindow = function (windowName, newWindow) {
        requestModulesMetadata().then(function (modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            if (ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            } else {
                ScreenService.openWindow(windowName);
            }
        });
    };

    this.masterDetailNewRow = function (widget) {
        self.mudarVisibilidadeWidgets(widget.widgets, false);
        widget.newRow();
    };

    this.masterDetailVoltarList = function (widget) {
        widget.list();
        ScreenService.closeSwipe();
        widget.activate();
    };

    this.masterDetailEdit = function (widget) {
        self.mudarVisibilidadeWidgets(widget.widgets, false);
        widget.backupRow = Util.clone(widget.currentRow);
        widget.edit();
    };

    this.masterDetailCancelaEdicao = function (widget) {
        self.mudarVisibilidadeWidgets(widget.widgets, true);
        if (widget.currentRow.__is_new) {
            var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
            if (indexOfCurrentRow !== -1) {
                widget.dataSource.data.pop();
            }
            widget.list();
            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backupRow);
            widget.view();
        }
    };

    this.mudarVisibilidadeWidgets = function (widgets, isVisible) {
        widgets.forEach(function (widget) {
            widget.isVisible = isVisible;
        });
    };

    this.masterDetailSave = function (widget) {
        self.mudarVisibilidadeWidgets(widget.widgets, true);
        if (widget.isValid()) {
            widget.currentRow.changed = true;
            return widget.saveRow().then(function () {
                widget.currentRow.changed = false;
                widget.currentRow.__is_new = false;
                widget.view();
            });
        } else {
            return ZHPromise.reject();
        }
    };

    this.masterDetailVoltarParent = function (widget) {
        ScreenService.closeSwipe().then(function () {
            widget.getParent().list();
            widget.getParent().activate();
        });
    };

    this.retornaTemplate = function (widget) {
        var template;

        if (widget == 'Geral') {
            template = 'widget/form-view.html';
        } else {
            template = 'widget/master_detail/grid.html';
        }
    };

    this.retornaJson = function (strHierarquia) {
        console.log(strHierarquia);
        var json = '';
        if (strHierarquia == 'Geral_Filial/Loja/PDV_' || strHierarquia == 'Filial/Loja/PDV_Geral_') {
            window.setLocalVar('RELACIONAMENTOH', 'PDV');
            json = 'param00001_geral_pdv';
        } else if (strHierarquia == 'Tipo de Recebimento_') {
            json = 'param00002_tiporece';
        } else if (strHierarquia == 'PDV_') {
            window.setLocalVar('RELACIONAMENTOH', 'PDV');
            json = 'param00001_geral_pdv';
        } else if (strHierarquia == 'Geral_Filial/Loja_' || strHierarquia == 'Filial/Loja_Geral_') {
            window.setLocalVar('RELACIONAMENTOH', 'LOJA');
            json = 'param00001_geral_pdv';
        } else if (strHierarquia == 'Geral_Filial/Loja/Tipo de Recebimento_' || strHierarquia == 'Filial/Loja/Tipo de Recebimento_Geral_') {
            window.setLocalVar('RELACIONAMENTOH', 'LOJA');
            json = 'param00005_filial_loja_tiporece';
        } else {
            json = 'param00006_tiporece_filial_loja';
        }
        return json;
    };

    this.getDataSource = function (parametro) {
        if (parametro == 'Tipo de Baixa Contas a receber') {} else if (parametro == 'Tipo de Conta Receber') {} else if (parametro == 'Cliente para geração do Contas a Receber') {} else if (parametro == 'Forma Pagamento') {}
    };

    this.returnNmparametro = function (descparametro) {
        var nmparametro = '';
        if (descparametro == 'Tipo de Baixa Contas a receber') {
            nmparametro = 'CDTPBAIXAREC';
        } else if (descparametro == 'Tipo de Conta Receber') {
            nmparametro = 'CDTPCTRECEB';
        } else if (descparametro == 'Taxa administrativa') {
            nmparametro = 'VRTAXAADMI';
        } else if (descparametro == 'Cliente para geração do Contas a Receber') {
            nmparametro = 'CDCLIENTE';
        } else if (descparametro == 'Forma Pagamento') {
            nmparametro = 'CDFORMPGTO';
        } else if (descparametro == 'Dias p/ recebimento') {
            nmparametro = 'NRDIASADMI';
        } else if (descparametro == 'Geração de movimento Contábil?') {
            nmparametro = 'IDGERAFIN';
        }
        return nmparametro;
    };

    this.checkDate = function (ini, fim) {
        var dtini = ini.split('/');
        var dtfin = fim.split('/');

        if (dtini[2] < dtfin[2]) {
            return true;
        } else if (dtini[2] == dtfin[2]) {
            if (dtini[1] < dtfin[1]) {
                return true;
            } else if (dtini[1] == dtfin[1]) {
                if (dtini[0] <= dtfin[0]) {
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    };

    this.clearField = function (field) {
        if (field.readOnly || field.aclReadOnly) {
            return;
        }
        field.widget.currentRow[field.name] = "";
        for (var prop in field.outData) {
            field.widget.currentRow[field.outData[prop]] = "";
        }
        field.widget.currentRow.changed = true;
        if (field.change) {
            field.change();
        }
    };

    String.prototype.moneyToFloat = function () {
        var treated = this.replace('R$', '');
        treated = treated.replace(' ', '');
        treated = treated.replace(',', '.');
        return parseFloat(treated);
    };

    Number.prototype.moneyToFloat = function () {
        return parseFloat(this);
    };

    Number.prototype.isInt = function () {
        return this % 1 === 0;
    };

    Number.prototype.toMoney = function () {
        var treated = "";
        var splited = this.toString().split('.');
        if (parseFloat(this).isInt()) {
            treated = this + ',00';
        } else if (parseFloat(splited[1]) < 10) {
            splited[1] += '0';
            treated = splited[0] + ',' + splited[1];
        } else {
            treated = this;
        }

        treated = treated.replace('.', ',');
        treated = 'R$ ' + treated;
        return treated;
    };

    this.reloadDataSource = function (dataOwner) {
        dataOwner.dataSource.reload();
        templateManager.updateTemplate();
    };

    this.hourToMinutes = function (time) {
        var hour = time.substring(0, 2);
        var minutes = time.substring(3, 5);
        var totalminutes = hour * 60 + parseInt(minutes);
        return totalminutes;
    };

    this.handleBack = function (activePage) {
        if (activePage == 'faturometro') {
            openWindow('dashboard');
        }
    };

    this.ativaTab = function (widget) {
        widget.activate();
    };

    this.formataVigencia = function (data) {
        var dia = data.substring(8, 10);
        var mes = data.substring(5, 7);
        var ano = data.substring(0, 4);

        return dia + '/' + mes + '/' + ano;
    };

    this.analiseFormatoData = function (data) {
        if (data.substring(2, 3) != '/') {
            return false;
        } else if (data.substring(5, 6) != '/') {
            return false;
        } else if (data.substring(8, 11) === "") {
            return false;
        } else if (data.substring(10, 12) !== "") {
            return false;
        } else {
            return true;
        }
    };

    this.verificaVigencia = function (DTINIVIGCAMP, DTFIMVIGCAMP) {
        return moment(DTINIVIGCAMP, 'DD/MM/YYYY', true).unix() > moment(DTFIMVIGCAMP, 'DD/MM/YYYY', true).unix();
    };

    this.formataDecimal = function (numero) {
        if (numero !== null || numero !== undefined) {
            numero = numero.replace(",", ".");
        }
        return numero;
    };

    this.cancelarPopUp = function (widget) {
        ScreenService.closePopup();
        widget.activate();
        templateManager.updateTemplate();
    };

    this.cancelarSwipe = function (widget) {
        ScreenService.closeSwipe();
        widget.activate();
        templateManager.updateTemplate();
    };

    this.atualizaTemplate = function (widget) {
        return widget.reload().then(function () {
            widget.activate();
            ScreenService.updateTemplate();
        });
    };

    this.atualizaTemplateFechaSwipe = function (widget) {
        ScreenService.closeSwipe();
        widget.reload();
        widget.activate();
        templateManager.updateTemplate();
    };

    this.validaNumero = function (NUMERO) {
        if (isNaN(NUMERO) || NUMERO === null || NUMERO === "") {
            return false;
        } else {
            return true;
        }
    };
    //this is a magic function made by me Gabriel s2
    //it put the filter gifted by the currentField on teh child and set the child readOnly if the current field as been erased
    //I clean the child values too
    this.setSubFieldFilter = function (currentField, childField) {
        self.clearField(childField);
        if (currentField.widget.currentRow[currentField.valueField]) {
            self.setDataSourceFilters(childField, currentField.widget.currentRow).then(function () {
                childField.readOnly = false;
                childField.reload();
            });
        } else {
            childField.readOnly = true;
        }
    };

    //this is a magic function made by me Gabriel s2
    //it put the filters on the object and replaces null whit ''
    this.setDataSourceFilters = function (object, filters) {
        var defer = ZHPromise.defer();
        object.dataSourceFilter.forEach(function (filter) {
            if (filters[filter.name] == null) {
                filters[filter.name] = '';
            }
            filter.value = filters[filter.name];
        });
        defer.resolve();
        return defer;
    };

    this.validaIpParaPopUp = function (callback, widget) {
        widget.currentRow.DSEMAILOPER = '';
        ScreenService.openPopup(widget);
    };

    this.readQRCode = function () {
        ZhNativeInterface.readCode();
    };

    this.checkEmail = function (email) {
        /* This regular expression is said to not work in some very specific cases, but it appears to be safe to use in our case. */
        var match = email.match(/^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/);
        if (match !== null) return true;else return false;
    };

    this.resetGrid = function (widget) {
        widget.dataSource.data = [];
        widget.reload();
        widget.plainTreeData = [];
        templateManager.updateTemplate();
    };

    this.resetNewRow = function (widget) {
        widget.edit();
        widget.currentRow = {};
        widget.reload();
    };

    this.resetStripeGrid = function (widget) {
        widget.dataSource.data = [];
    };

    this.goBackSwipe = function (widget) {
        widget.activate();
    };

    this.goBack = function (widget) {
        widget.activate();
    };

    this.setMaxlength = function (field, maxLength) {
        if (typeof field.value() === "number") {
            var length = field.value().toString().length;
            if (length > maxLength) {
                field.value(parseInt(field.value().toString().slice(0, -1)));
            }
        }
    };

    this.setModeView = function (widget) {
        widget.view();
    };

    this.compareDates = function (minorDate, majorDate) {
        //INFORMAR DATA NO PADRÃO BRASILEIRO
        var small = new Date(moment(minorDate, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
        var big = new Date(moment(majorDate, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();

        if (small > big) {
            return false;
        } else return true;
    };
    // Returns if there's only numbers
    this.hasOnlyNumbers = function (str) {
        return (/^[0-9]+$/g.test(str)
        );
    };

    // returns null or the formatted CD
    this.getValidCD = function (check) {
        var sizeOfColumn = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 20;

        if (check !== null) {
            var regex = /^[0-9]+$/g;
            regex.test(regex, sizeOfColumn.toString());
            return regex.test(check) ? '0'.repeat(sizeOfColumn - check.length) + check : null;
        } else {
            return null;
        }
    };

    this.isValidIP = function (end) {
        return (/(^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])((\/{1}[0-9A-Za-z]+)|()))$/g.test(end)
        );
    };

    this.openReport = function (route, reportName, params) {
        params.__report_name = reportName;
        return restEngine.post(route, {
            "requestType": "Row",
            "row": params
        }, null, null, null, null, 3000000).then(function (response) {
            if (response && response.messages) {
                var report = response.messages.pop();
                window.open(report.message);
            }
        }).catch(function (response) {
            if (response.data && response.data.error) {
                ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
            }
        });
    };

    this.callbackError = function (callback) {

        if (callback.data) {

            ScreenService.showMessage(ScreenService.i18n(callback.data.error));
        } else {

            ScreenService.showMessage(ScreenService.i18n('Timeout'));
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('UtilitarioGeral', UtilitarioGeral);
});

// FILE: ./assets/js/utilities/UtilitariosConexao.js
function UtilitariosConexao(templateManager, ScreenService) {

    var self = this;

    this.prepareServerForm = function (ipWidget) {
        var ip = getLocalVar('ipSemPorta');
        var porta = getLocalVar('porta');

        var data = [];
        data.porta = porta;
        data.ip = ip;
        ipWidget.setCurrentRow(data);
    };

    this.validateIp = function (demo) {
        var doLogin = false;
        if (demo != 'S') {
            if (getLocalVar('ip') === undefined) {
                ScreenService.showMessage("Configure o ip do servidor da aplicação.");
            } else {
                templateManager.updateURL(getLocalVar('ip'));
                doLogin = true;
            }
        } else {
            this.setDemoIp();
            doLogin = true;
        }
        return doLogin;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('UtilitariosConexao', UtilitariosConexao);
});

// FILE: ./assets/js/utilities/UtilitariosTela.js
function UtilitariosTela(ScreenService, templateManager) {
    this.editar = function (widget) {
        widget.edit();
        widget.actions[1].isVisible = false;
        widget.actions[2].isVisible = true;
        widget.template = "widget/form-view-edit.html";
        widget.formTemplate = "widget/form-view-edit.html";
        widget.oldRow = Util.clone(widget.currentRow);
    };

    this.view = function (widget) {
        widget.view();
        widget.actions[2].isVisible = false;
        widget.actions[1].isVisible = true;
        widget.activate();
        widget.template = widget.templateToView;
    };

    this.cancel = function (widget) {
        widget.view();
        widget.actions[2].isVisible = false;
        widget.actions[1].isVisible = true;
        widget.activate();
        widget.template = widget.templateToView;
        widget.currentRow = Util.clone(widget.oldRow);
    };

    this.goBack = function (widget) {
        widget.activate();
        ScreenService.openWindow(widget);
    };

    this.goBackSwipe = function (widget) {
        ScreenService.closeSwipe();
        widget.activate();
    };

    this.reconfigureTabFidelity = function (widget) {
        widget.view();
        widget.template = widget.templateToView;
        widget.actions[1].isVisible = true;
        widget.actions[2].isVisible = false;
        templateManager.updateTemplate();
    };

    //this works in forms
    this.templateToView = function (widget) {
        widget.list();
        widget.template = widget.templateToView;
        widget.getAction('EDIT_MODE').isVisible = true;
        widget.getAction('SAVE_FORM').isVisible = false;
        ScreenService.updateTemplate();
    };

    this.resetGridFilter = function (widget) {
        widget.dataSource.data = [];
    };

    this.templateToEdit = function (widget) {
        widget.edit();
        widget.template = widget.templateToForm;
        widget.getAction('EDIT_MODE').isVisible = false;
        widget.getAction('SAVE_FORM').isVisible = true;
        ScreenService.updateTemplate();
    };
    //toggling things whit checkboxes

    this.toggleSubField = function (fieldNames, widget, value) {
        if ((typeof fieldNames === 'undefined' ? 'undefined' : _typeof(fieldNames)) == 'object') {
            fieldNames.forEach(function (fieldName) {
                this.toggleSubField(fieldName, widget, value);
            }.bind(this));
        } else {
            widget.fields.forEach(function (field) {
                if (field.name == fieldNames) {
                    field.isVisible = value == 'S';
                }
            });
        }
    };

    this.toggleSubGroup = function (fieldGroupNames, widget, value) {
        if ((typeof fieldGroupNames === 'undefined' ? 'undefined' : _typeof(fieldGroupNames)) == 'object') {
            fieldGroupNames.forEach(function (fieldGroupName) {
                this.toggleSubField(fieldGroupName, widget, value);
            }.bind(this));
        } else {
            widget.fieldGroups.forEach(function (fieldGroup) {
                if (fieldGroup.name == fieldGroupNames) {
                    fieldGroup.isVisible = value == 'S';
                }
            });
        }
    };

    /*
     *
     * a função recebe o nome do campo que terá o dia adicionado para que
     * Interval funcione com o period de hoje
     *
     */

    this.addTimeToInterval = function () {
        var dateFinal = moment(templateManager.container.getWidget('caixa').currentRow.DTABERTURA.end, "DD/MM/YYYY");
        dateFinal.add(1, 'days');
        templateManager.container.getWidget('caixa').dataSourceFilter.forEach(function (part, index, dtSourceFilter) {
            if (part.name == 'DTABERTURA') dtSourceFilter[index].value[1] = dateFinal.format('DD/MM/YYYY');
        });
        templateManager.container.getWidget('caixa').reload();
        templateManager.container.getWidget('caixa').activate();
    };

    this.setIntervalToday = function (widget, fieldName) {
        widget.getField(fieldName).value({
            'start': moment().format('DD/MM/YY'),
            'period': undefined,
            'end': moment().format('DD/MM/YY')
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('UtilitariosTela', UtilitariosTela);
});

// FILE: ./assets/js/utilities/zhFilter.js
function Filter(ScreenService, templateManager, ZhFilterPreferences, restEngine, UtilitarioGeral) {
    var _this = this;

    var self = this;

    this.getAppliedFilters = function (widget) {
        return widget.appliedFilter.FILTER_ROW;
    };

    this.setDefaultFilter = function (widget, fieldDateName) {
        var row = widget.currentRow;

        row[fieldDateName] = {
            'start': moment().format('DD/MM/YYYY'),
            'period': undefined,
            'end': moment().format('DD/MM/YYYY')
        };

        self.openFilterWithRows(widget, row);
    };

    this.startFiltering = function (widget) {
        self.openFilterWithRows(widget);
    };

    this.openFilterWithRows = function (widget, rows) {
        var row = !Util.isUndefinedOrNull(rows) ? rows : rowsFiltroPadrao();

        fixWidgetSetSearchCriteria(widget);

        self.checkUnidadeDisp().then(function (condition) {
            return condition ? ZhFilterPreferences.openFilterWidget(widget, row) : UtilitarioGeral.openWindow('man#ger01742_selecionar_unidade');
        });
    };

    var fixWidgetSetSearchCriteria = function fixWidgetSetSearchCriteria(widget) {
        if (!widget.hasOwnProperty(setSearchCriteria)) {
            widget.setSearchCriteria = setSearchCriteria.bind(widget);
        }
    };

    var clearSearchCriteria = function clearSearchCriteria() {
        _this.searchCriteria = {};
        _this.searchList = "";
    };

    var searchFromSearchCriteria = Util.buildDebounceMethod(function () {
        if (!_this._noReloadFromSearchCriteria) {
            _this.searchList = _this.searchCriteria && _this.searchCriteria._ALL || undefined;
        }
        _this._noReloadFromSearchCriteria = false;
    }, 200);

    var setSearchCriteria = function setSearchCriteria(searchCriteria) {
        _this._noReloadFromSearchCriteria = true;

        clearSearchCriteria = clearSearchCriteria.bind(_this);
        clearSearchCriteria();

        if (searchCriteria) {
            searchCriteria.forEach(function (field) {
                var fieldValue = field.value;
                if (fieldValue[fieldValue.length - 1] == '%') {
                    fieldValue = fieldValue.substring(0, fieldValue.length - 1);
                }
                if (fieldValue[0] == "%") {
                    fieldValue = fieldValue.substring(1, fieldValue.length);
                }
                this.searchCriteria[field.name] = fieldValue;
            });
        }
        searchFromSearchCriteria = searchFromSearchCriteria.bind(_this);
        searchFromSearchCriteria(_this.searchCriteria);
    };

    this.checkUnidadeDisp = function (CDFILIAL, NRPERFIL) {
        var filter = [{
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORG')
        }, {
            "name": "CDFILIAL",
            "operator": "=",
            "value": CDFILIAL ? CDFILIAL : getLocalVar('CDFILIAL')
        }, {
            "name": "CDOPERADOR",
            "operator": "=",
            "value": getLocalVar('CDOPERADOR')
        }, {
            "name": "CDPERFILSELECIONADO",
            "operator": "=",
            "value": NRPERFIL ? NRPERFIL : getLocalVar('CDPERFILSELECIONADO')
        }];

        return restEngine.post('product#/checkFilialDisp', {
            "requestType": "FilterData",
            "filter": filter
        }).then(function (response) {
            return response.dataset.checkFilialDisp[0];
        }, function (error) {
            throw 'Ocorreu um erro!';
        });
    };

    var rowsFiltroPadrao = function rowsFiltroPadrao() {
        return {
            DTCREDITO: {
                start: moment().format('DD/MM/YYYY'),
                period: undefined,
                end: moment().format('DD/MM/YYYY')
            },
            DTVENDA: {
                start: moment().format('DD/MM/YYYY'),
                period: undefined,
                end: moment().format('DD/MM/YYYY')
            },
            DTRECEBIMENTO: {
                start: moment().format('DD/MM/YYYY'),
                period: undefined,
                end: moment().format('DD/MM/YYYY')
            },
            DTVENDAMESINI: moment().format('MM/YYYY'),
            DTVENDAMESFIN: moment().format('MM/YYYY'),
            PERIODO: 'M',
            STATUS: 'T',
            NRORG: getLocalVar('NRORG')
        };
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Filter', Filter);
});

// FILE: ./assets/js/repositories/AdquirentesRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var AdquirentesRepo = RepositoryFactory.factory("cnc#/AdquirentesRepo", "LOCAL");
    ContextRegister.register('AdquirentesRepo', AdquirentesRepo);
});

// FILE: ./assets/js/repositories/BandeirasRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var BandeirasRepo = RepositoryFactory.factory("cnc#/BandeirasRepo", "LOCAL");
    ContextRegister.register('BandeirasRepo', BandeirasRepo);
});

// FILE: ./assets/js/repositories/Cnc00100_AdicionaCriteriosEstabelecRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00100_AdicionaCriteriosEstabelecRepo = RepositoryFactory.factory("cnc#/Cnc00100_AdicionaCriteriosEstabelecRepo", "LOCAL");
    ContextRegister.register('Cnc00100_AdicionaCriteriosEstabelecRepo', Cnc00100_AdicionaCriteriosEstabelecRepo);
});

// FILE: ./assets/js/repositories/Cnc00100_AlteraCriterioRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00100_AlteraCriterioRepo = RepositoryFactory.factory("cnc#/Cnc00100_AlteraCriterioRepo", "LOCAL");
    ContextRegister.register('Cnc00100_AlteraCriterioRepo', Cnc00100_AlteraCriterioRepo);
});

// FILE: ./assets/js/repositories/Cnc00100_AlteraCriteriosEstabelecRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00100_AlteraCriteriosEstabelecRepo = RepositoryFactory.factory("cnc#/Cnc00100_AlteraCriteriosEstabelecRepo", "LOCAL");
    ContextRegister.register('Cnc00100_AlteraCriteriosEstabelecRepo', Cnc00100_AlteraCriteriosEstabelecRepo);
});

// FILE: ./assets/js/repositories/Cnc00100_BuscaCriterioRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00100_BuscaCriterioRepo = RepositoryFactory.factory("cnc#/Cnc00100_BuscaCriterioRepo", "LOCAL");
    ContextRegister.register('Cnc00100_BuscaCriterioRepo', Cnc00100_BuscaCriterioRepo);
});

// FILE: ./assets/js/repositories/Cnc00100_BuscaCriteriosEstabelecRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00100_BuscaCriteriosEstabelecRepo = RepositoryFactory.factory("cnc#/Cnc00100_BuscaCriteriosEstabelecRepo", "LOCAL");
    ContextRegister.register('Cnc00100_BuscaCriteriosEstabelecRepo', Cnc00100_BuscaCriteriosEstabelecRepo);
});

// FILE: ./assets/js/repositories/Cnc00100_BuscaOrdensEstRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00100_BuscaOrdensEstRepo = RepositoryFactory.factory("cnc#/Cnc00100_BuscaOrdensEstRepo", "LOCAL");
    ContextRegister.register('Cnc00100_BuscaOrdensEstRepo', Cnc00100_BuscaOrdensEstRepo);
});

// FILE: ./assets/js/repositories/Cnc00100_BuscaOrdensRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00100_BuscaOrdensRepo = RepositoryFactory.factory("cnc#/Cnc00100_BuscaOrdensRepo", "LOCAL");
    ContextRegister.register('Cnc00100_BuscaOrdensRepo', Cnc00100_BuscaOrdensRepo);
});

// FILE: ./assets/js/repositories/Cnc00100_LimpaCriteriosEstAdqRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00100_LimpaCriteriosEstAdqRepo = RepositoryFactory.factory("cnc#/Cnc00100_LimpaCriteriosEstAdqRepo", "LOCAL");
    ContextRegister.register('Cnc00100_LimpaCriteriosEstAdqRepo', Cnc00100_LimpaCriteriosEstAdqRepo);
});

// FILE: ./assets/js/repositories/Cnc00200_CadastroTecnologiaAdq.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00200_CadastroTecnologiaAdq = RepositoryFactory.factory("cnc#/Cnc00200_CadastroTecnologiaAdq", "LOCAL");
    ContextRegister.register('Cnc00200_CadastroTecnologiaAdq', Cnc00200_CadastroTecnologiaAdq);
});

// FILE: ./assets/js/repositories/Cnc00200_CadastroTecnologiaAdqEstabelec.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00200_CadastroTecnologiaAdqEstabelec = RepositoryFactory.factory("cnc#/Cnc00200_CadastroTecnologiaAdqEstabelec", "LOCAL");
    ContextRegister.register('Cnc00200_CadastroTecnologiaAdqEstabelec', Cnc00200_CadastroTecnologiaAdqEstabelec);
});

// FILE: ./assets/js/repositories/Cnc00300_AlteraPeriodoTaxaRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00300_AlteraPeriodoTaxaRepo = RepositoryFactory.factory("cnc#/Cnc00300_AlteraPeriodoTaxaRepo", "LOCAL");
    ContextRegister.register('Cnc00300_AlteraPeriodoTaxaRepo', Cnc00300_AlteraPeriodoTaxaRepo);
});

// FILE: ./assets/js/repositories/Cnc00300_DeletaPeriodoTaxaRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00300_DeletaPeriodoTaxaRepo = RepositoryFactory.factory("cnc#/Cnc00300_DeletaPeriodoTaxaRepo", "LOCAL");
    ContextRegister.register('Cnc00300_DeletaPeriodoTaxaRepo', Cnc00300_DeletaPeriodoTaxaRepo);
});

// FILE: ./assets/js/repositories/Cnc00300_InserePeriodoTaxaRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00300_InserePeriodoTaxaRepo = RepositoryFactory.factory("cnc#/Cnc00300_InserePeriodoTaxaRepo", "LOCAL");
    ContextRegister.register('Cnc00300_InserePeriodoTaxaRepo', Cnc00300_InserePeriodoTaxaRepo);
});

// FILE: ./assets/js/repositories/Cnc00300_adquirentePeriodoRow.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00300_adquirentePeriodoRow = RepositoryFactory.factory("cnc#/Cnc00300_adquirentePeriodoRow", "LOCAL");
    ContextRegister.register('Cnc00300_adquirentePeriodoRow', Cnc00300_adquirentePeriodoRow);
});

// FILE: ./assets/js/repositories/Cnc00300_adquirentePeriodos.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00300_adquirentePeriodos = RepositoryFactory.factory("cnc#/Cnc00300_adquirentePeriodos", "LOCAL");
    ContextRegister.register('Cnc00300_adquirentePeriodos', Cnc00300_adquirentePeriodos);
});

// FILE: ./assets/js/repositories/Cnc00400_VendasInexAdqui_Repository.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00400_VendasInexAdquiRepo = RepositoryFactory.factory("cnc#/Cnc00400_VendasInexAdquiRepo", "LOCAL");
    ContextRegister.register('Cnc00400_VendasInexAdquiRepo', Cnc00400_VendasInexAdquiRepo);
});

// FILE: ./assets/js/repositories/Cnc00600_ConciliatorSellsDetailsRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00600_ConciliatorSellsDetailsRepo = RepositoryFactory.factory("cnc#/Cnc00600_ConciliatorSellsDetailsRepo", "LOCAL");
    ContextRegister.register('Cnc00600_ConciliatorSellsDetailsRepo', Cnc00600_ConciliatorSellsDetailsRepo);
});

// FILE: ./assets/js/repositories/Cnc00700_GetRecebimentosBandeira.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00700_GetRecebimentosBandeira = RepositoryFactory.factory("cnc#/Cnc00700_GetRecebimentosBandeira", "LOCAL");
    ContextRegister.register('Cnc00700_GetRecebimentosBandeira', Cnc00700_GetRecebimentosBandeira);
});

// FILE: ./assets/js/repositories/Cnc00700_GetRecebimentosBandeiraDetalhes.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00700_GetRecebimentosBandeiraDetalhes = RepositoryFactory.factory("cnc#/Cnc00700_GetRecebimentosBandeiraDetalhes", "LOCAL");
    ContextRegister.register('Cnc00700_GetRecebimentosBandeiraDetalhes', Cnc00700_GetRecebimentosBandeiraDetalhes);
});

// FILE: ./assets/js/repositories/Cnc00800_GetComparacaoValoresDetalhes.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00800_GetComparacaoValoresDetalhes = RepositoryFactory.factory("cnc#/Cnc00800_GetComparacaoValoresDetalhes", "LOCAL");
    ContextRegister.register('Cnc00800_GetComparacaoValoresDetalhes', Cnc00800_GetComparacaoValoresDetalhes);
});

// FILE: ./assets/js/repositories/Cnc00900_BandeiraPorNomeRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00900_BandeiraPorNomeRepo = RepositoryFactory.factory("cnc#/Cnc00900_BandeiraPorNomeRepo", "LOCAL");
    ContextRegister.register('Cnc00900_BandeiraPorNomeRepo', Cnc00900_BandeiraPorNomeRepo);
});

// FILE: ./assets/js/repositories/Cnc00900_VendasAntecipadasDetalhesRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00900_VendasAntecipadasDetalhesRepo = RepositoryFactory.factory("cnc#/Cnc00900_VendasAntecipadasDetalhesRepo", "LOCAL");
    ContextRegister.register('Cnc00900_VendasAntecipadasDetalhesRepo', Cnc00900_VendasAntecipadasDetalhesRepo);
});

// FILE: ./assets/js/repositories/Cnc00900_VendasAntecipadasRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00900_VendasAntecipadasRepo = RepositoryFactory.factory("cnc#/Cnc00900_VendasAntecipadasRepo", "LOCAL");
    ContextRegister.register('Cnc00900_VendasAntecipadasRepo', Cnc00900_VendasAntecipadasRepo);
});

// FILE: ./assets/js/repositories/Cnc01000_CorrecaoPagamento.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc01000_CorrecaoPagamento = RepositoryFactory.factory("cnc#/Cnc01000_CorrecaoPagamento", "LOCAL");
    ContextRegister.register('Cnc01000_CorrecaoPagamento', Cnc01000_CorrecaoPagamento);
});

// FILE: ./assets/js/repositories/Cnc01000_DetalhamentoParcelaRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc01000_DetalhamentoParcelaRepo = RepositoryFactory.factory("cnc#/Cnc01000_DetalhamentoParcelaRepo", "LOCAL");
    ContextRegister.register('Cnc01000_DetalhamentoParcelaRepo', Cnc01000_DetalhamentoParcelaRepo);
});

// FILE: ./assets/js/repositories/Cnc01000_DivergenciaConciliacao.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc01000_DivergenciaConciliacao = RepositoryFactory.factory("cnc#/Cnc01000_DivergenciaConciliacao", "LOCAL");
    ContextRegister.register('Cnc01000_DivergenciaConciliacao', Cnc01000_DivergenciaConciliacao);
});

// FILE: ./assets/js/repositories/Cnc01000_DivergenciaPagamento.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc01000_DivergenciaPagamento = RepositoryFactory.factory("cnc#/Cnc01000_DivergenciaPagamento", "LOCAL");
    ContextRegister.register('Cnc01000_DivergenciaPagamento', Cnc01000_DivergenciaPagamento);
});

// FILE: ./assets/js/repositories/Cnc01100_ValoresSimProcedureRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc01100_ValoresSimProcedureRepo = RepositoryFactory.factory("cnc#/Cnc01100_ValoresSimProcedureRepo", "LOCAL");
    ContextRegister.register('Cnc01100_ValoresSimProcedureRepo', Cnc01100_ValoresSimProcedureRepo);
});

// FILE: ./assets/js/repositories/Cnc01100_ValoresSimuladosRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc01100_ValoresSimuladosRepo = RepositoryFactory.factory("cnc#/Cnc01100_ValoresSimuladosRepo", "LOCAL");
    ContextRegister.register('Cnc01100_ValoresSimuladosRepo', Cnc01100_ValoresSimuladosRepo);
});

// FILE: ./assets/js/repositories/Cnc01400_CancelamentosAjustesRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc01400_CancelamentosAjustesRepo = RepositoryFactory.factory("cnc#/Cnc01400_CancelamentosAjustesRepo", "LOCAL");
    ContextRegister.register('Cnc01400_CancelamentosAjustesRepo', Cnc01400_CancelamentosAjustesRepo);
});

// FILE: ./assets/js/repositories/Cnc01500_ajustesAdquirentesRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc01500_ajustesAdquirentesRepo = RepositoryFactory.factory("cnc#/Cnc01500_ajustesAdquirentesRepo", "LOCAL");
    ContextRegister.register('Cnc01500_ajustesAdquirentesRepo', Cnc01500_ajustesAdquirentesRepo);
});

// FILE: ./assets/js/repositories/Cnc01700_integracaoFinanceiroAdq.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var cnc01700_integracaoFinanceiroAdq = RepositoryFactory.factory("cnc#/cnc01700_integracaoFinanceiroAdq", "LOCAL");
    ContextRegister.register('cnc01700_integracaoFinanceiroAdq', cnc01700_integracaoFinanceiroAdq);
});

// FILE: ./assets/js/repositories/Cnc02500_SerieEstabelecimento.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var cnc02500_serie_estabelecimento = RepositoryFactory.factory("cnc#/cnc02500_serie_estabelecimento", "LOCAL");
    ContextRegister.register('cnc02500_serie_estabelecimento', cnc02500_serie_estabelecimento);
});

// FILE: ./assets/js/repositories/Cnc_GetEstabelecAdqRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc_GetEstabelecAdqRepo = RepositoryFactory.factory("cnc#/Cnc_GetEstabelecAdqRepo", "LOCAL");
    ContextRegister.register('Cnc_GetEstabelecAdqRepo', Cnc_GetEstabelecAdqRepo);
});

// FILE: ./assets/js/repositories/Cnc_GetEstabelecRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc_GetEstabelecRepo = RepositoryFactory.factory("cnc#/Cnc_GetEstabelecRepo", "LOCAL");
    ContextRegister.register('Cnc_GetEstabelecRepo', Cnc_GetEstabelecRepo);
});

// FILE: ./assets/js/repositories/GetFiliaisAdqRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var GetFiliaisAdqRepo = RepositoryFactory.factory("cnc#/GetFiliaisAdqRepo", "LOCAL");
    ContextRegister.register('GetFiliaisAdqRepo', GetFiliaisAdqRepo);
});

// FILE: ./assets/js/repositories/GetFiliaisRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var GetFiliaisRepo = RepositoryFactory.factory("cnc#/GetFiliaisRepo", "LOCAL");
    ContextRegister.register('GetFiliaisRepo', GetFiliaisRepo);
});

// FILE: ./assets/js/repositories/GetVendasConciliadores.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var Cnc00600_GetVendasConciliadores = RepositoryFactory.factory("cnc#/Cnc00600_GetVendasConciliadores", "LOCAL");
    ContextRegister.register('Cnc00600_GetVendasConciliadores', Cnc00600_GetVendasConciliadores);
});

// FILE: ./assets/js/repositories/ProdutoAdquirenteRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var ProdutoAdquirenteRepo = RepositoryFactory.factory("cnc#/ProdutoAdquirenteRepo", "LOCAL");
    ContextRegister.register('ProdutoAdquirenteRepo', ProdutoAdquirenteRepo);
});

// FILE: ./assets/js/repositories/ProdutosGerRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var ProdutosGerRepo = RepositoryFactory.factory("cnc#/ProdutosGerRepo", "LOCAL");
    ContextRegister.register('ProdutosGerRepo', ProdutosGerRepo);
});

// FILE: ./assets/js/repositories/ProdutosRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var ProdutosRepo = RepositoryFactory.factory("cnc#/ProdutosRepo", "LOCAL");
    ContextRegister.register('ProdutosRepo', ProdutosRepo);
});

// FILE: ./assets/js/repositories/TipoReceRepo.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var TipoReceRepo = RepositoryFactory.factory("cnc#/TipoReceRepo", "LOCAL");
    ContextRegister.register('TipoReceRepo', TipoReceRepo);
});

// FILE: ./assets/js/repositories/cnc00200_adquirentes.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var cnc00200_adquirentes = RepositoryFactory.factory("cnc#/cnc00200_adquirentes", "LOCAL");
    ContextRegister.register('cnc00200_adquirentes', cnc00200_adquirentes);
});

// FILE: ./assets/js/repositories/cnc01000_vendasconciliadas_tree.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var cnc01000_vendasconciliadas_tree = RepositoryFactory.factory("cnc#/cnc01000_vendasconciliadas_tree", "LOCAL");
    ContextRegister.register('cnc01000_vendasconciliadas_tree', cnc01000_vendasconciliadas_tree);
});

// FILE: ./assets/js/repositories/cnc01900_relTaxaVendaCheck.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var cnc01900_relTaxaVendaCheck = RepositoryFactory.factory("cnc#/cnc01900_relTaxaVendaCheck", "LOCAL");
    ContextRegister.register('cnc01900_relTaxaVendaCheck', cnc01900_relTaxaVendaCheck);
});

// FILE: ./assets/js/repositories/cnc02000_relVendaInexisAdqCheck.js
Configuration(function (RepositoryFactory, ContextRegister) {
    var cnc02000_relVendaInexisAdqCheck = RepositoryFactory.factory("cnc#/cnc02000_relVendaInexisAdqCheck", "LOCAL");
    ContextRegister.register('cnc02000_relVendaInexisAdqCheck', cnc02000_relVendaInexisAdqCheck);
});

// FILE: ./assets/js/services/Cnc00600_ConciliatorSellsService.js
function Cnc00600_ConciliatorSellsService(Query, Cnc00600_ConciliatorSellsDetailsRepo) {

    this.getConciliatorSellsDetails = function (NRADQUIRENTE, NMPRODUTOADQ, NRESTABELEC, DTVENDAINI, DTVENDAFIN) {
        var query = Query.build().where('NRESTABELEC').equals(NRESTABELEC).where('NRADQUIRENTE').equals(NRADQUIRENTE).where('NMPRODUTOADQ').equals(NMPRODUTOADQ).where('DTVENDAINI').equals(DTVENDAINI).where('DTVENDAFIN').equals(DTVENDAFIN);
        return Cnc00600_ConciliatorSellsDetailsRepo.download(query);
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00600_ConciliatorSellsService', Cnc00600_ConciliatorSellsService);
});

// FILE: ./assets/js/services/Cnc01100_SimulacaoAntecipada_Service.js
function Cnc01100_SimulacaoAntecipada_Service(Query, Cnc01100_ValoresSimProcedureRepo) {

    this.getAnticipatedParametrizedSimulation = function (NRESTABELEC, NRSEQADQ, NRSEQPRODADQ) {
        var query = Query.build().where('NRESTABELEC').equals(NRESTABELEC).where('NRADQUIRENTE').equals(NRSEQADQ).where('NRPRODUTOADQ').equals(NRSEQPRODADQ);
        return Cnc01100_ValoresSimProcedureRepo.download(query);
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01100_SimulacaoAntecipada_Service', Cnc01100_SimulacaoAntecipada_Service);
});

// FILE: ./assets/js/controllers/Cnc00100_CriteriosConsController.js
function Cnc00100_CriteriosConsController(ScreenService, templateManager, ZHPromise, restEngine) {
    var self = this;

    this.openPopup00100 = function (widget) {

        ScreenService.openPopup(widget);
        widget.getField('NRADQUIRENTE').value('');
        widget.getField('NRESTABELEC').value('');
        widget.fields[2].value('N');
        widget.fields[3].value('N');
        widget.fields[4].value('N');
        widget.fields[5].value('N');
        widget.fields[6].value('N');
        widget.fields[7].value('N');
        widget.fields[8].value('N');
        widget.fields[9].value('N');
        widget.fields[10].value('N');
        widget.fields[11].value('N');
        widget.fields[12].value('N');
        widget.fields[13].value('N');
        widget.fields[14].value('N');
        widget.dataSource.data = [];
        widget.template = "widget/form.html";
        widget.formTemplate = "widget/form.html";
    };

    this.setFieldValue00100 = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NRADQUIRENTE':
                    field.setValue('');
                    break;
                case 'NRESTABELEC':
                    field.setValue('');
                    break;
                case 'NRPRODUTOADQ':
                    field.setValue('');
                    break;
                default:
                    field.setValue('');
            }
        }
    };

    this.loadCriterioOrigemDetalhes = function (widget) {
        var NRADQUIRENTE = widget.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        var NRVALIDACAOADQ = widget.container.getWidget('tipoCriterio').currentRow.NRVALIDACAOADQ;
        widget.dataSourceFilter = [{
            "name": "NRADQUIRENTE",
            "operator": "=",
            "value": NRADQUIRENTE
        }, {
            "name": "NRVALIDACAOADQ",
            "operator": "=",
            "value": NRVALIDACAOADQ
        }, {
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }];
        widget.dataSource.load(widget.dataSourceFilter).then(function (dado) {
            widget.currentRow = dado.dataset.Cnc00100_BuscaCriterioRepo[0];
            ScreenService.openSwipe(widget);
            widget.view();
            widget.activate();
            widget.reload();
        });
    };

    this.loadCriterioOrigemEstDetalhes = function (widget) {
        var NRADQUIRENTE = widget.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        var NRESTABELEC = widget.container.getWidget('criterioEstabelec').currentRow.NRESTABELEC;
        var NRVALIDACAOADQEST = widget.container.getWidget('criterioOrdemEstabelec').currentRow.NRVALIDACAOADQEST;
        widget.dataSourceFilter = [{
            "name": "NRADQUIRENTE",
            "operator": "=",
            "value": NRADQUIRENTE
        }, {
            "name": "NRESTABELEC",
            "operator": "=",
            "value": NRESTABELEC
        }, {
            "name": "NRVALIDACAOADQEST",
            "operator": "=",
            "value": NRVALIDACAOADQEST
        }, {
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }];
        widget.dataSource.load(widget.dataSourceFilter).then(function (dado) {
            widget.currentRow = dado.dataset.Cnc00100_BuscaCriteriosEstabelecRepo[0];
            ScreenService.openSwipe(widget);
            widget.view();
            widget.activate();
            widget.reload();
        });
    };

    this.openEditCriterio = function (params) {
        params.edit();
        params.template = params.formTemplate;
        ScreenService.updateTemplate();
    };

    this.cancelEditCriterioGeral = function (widget) {
        self.loadCriterioOrigemDetalhes(widget);
        widget.template = 'widget/form-view.html';
    };

    this.cancelEditCriterioEst = function (widget) {
        self.loadCriterioOrigemEstDetalhes(widget);
        widget.template = 'widget/form-view.html';
    };

    this.setFieldValue = function (field) {
        var values = field.widget.currentRow;
        field.value({ LABELESTABELEC: values.LABELESTABELEC, NRESTABELEC: values.NRESTABELEC });
    };

    this.getAcquirers = function (swipe) {
        swipe.dataSource.load().then(function (dado) {
            swipe.container.getWidget('tipoCriterio').dataSource.data = dado.dataset.AdquirentesRepo;
            swipe.container.getWidget('adquirente').activate();
            swipe.container.getWidget('adquirente').dataSource.reload();
        });
    };

    this.goBackSwipeGeral = function (widget) {
        self.loadAcquirersPorAdquirente(templateManager.container.getWidget('generalParams'));
        ScreenService.closeSwipe();
        widget.activate();
    };

    this.goBackSwipeEst = function (widget) {
        self.loadAcquirersPorAdquirenteEst(templateManager.container.getWidget('generalParams'));
        ScreenService.closeSwipe();
        widget.activate();
    };

    this.goBackEst = function (widget) {
        ScreenService.closeSwipe();
        widget.activate();
        widget.reload();
    };

    this.setAcquire = function (swipe) {

        var NRADQUIRENTE = swipe.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        swipe.container.getWidget('criterioEstabelec').activate();
        swipe.container.getWidget('criterioEstabelec').dataSourceFilter[0].value = NRADQUIRENTE;
        swipe.container.getWidget('criterioEstabelec').fields[3].reload();
        self.cancelEdit(swipe.container.getWidget('tipoCriterio'));
    };

    this.loadAcquirersPorAdquirente = function (swipe) {
        var NRADQUIRENTE = swipe.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        var criterios = swipe.container.getWidget('tipoCriterio');
        var criteriosEst = swipe.container.getWidget('criterioEstabelec');
        var generalP = swipe.container.getWidget('generalParams');
        criteriosEst.dataSourceFilter = [{
            "name": "NRADQUIRENTE",
            "operator": "=",
            "value": NRADQUIRENTE
        }, {
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }];
        criterios.dataSourceFilter = [{
            "name": "NRADQUIRENTE",
            "operator": "=",
            "value": NRADQUIRENTE
        }, {
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }];
        criterios.dataSource.load(criterios.dataSourceFilter).then(function (dado) {
            swipe.container.getWidget('tipoCriterio').dataSource.data = dado.dataset.Cnc00100_BuscaOrdensRepo;
            swipe.container.getWidget('tipoCriterio').activate();
            ScreenService.openSwipe(generalP);
        });
    };

    this.loadAcquirersPorAdquirenteEst = function (swipe) {
        var NRADQUIRENTE = swipe.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        var NRESTABELEC = swipe.container.getWidget('criterioEstabelec').currentRow.NRESTABELEC;
        var criteriosEst = swipe.container.getWidget('criterioOrdemEstabelec');
        criteriosEst.dataSourceFilter = [{
            "name": "NRADQUIRENTE",
            "operator": "=",
            "value": NRADQUIRENTE
        }, {
            "name": "NRESTABELEC",
            "operator": "=",
            "value": NRESTABELEC
        }, {
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }];
        criteriosEst.dataSource.load(criteriosEst.dataSourceFilter).then(function (dado) {
            swipe.container.getWidget('criterioOrdemEstabelec').dataSource.data = dado.dataset.Cnc00100_BuscaOrdensEstRepo;
            swipe.container.getWidget('criterioOrdemEstabelec').activate();
            ScreenService.openSwipe(criteriosEst);
        });
    };

    this.alterarOrdemGeral = function (standardWidget) {

        var NRADQUIRENTE = standardWidget.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        var dataWidget = templateManager.container.getWidget('tipoCriterio');

        var row = standardWidget.currentRow;
        row.NRADQUIRENTE = NRADQUIRENTE;
        var DADOSBANCO = row.DADOSBANCO;

        if (row.NRDIASTOL == null) row.NRDIASTOL = 0;
        if (row.NRORDEM == null) {
            ScreenService.showMessage("Número de Ordem deve ser informado.");
        } else if (row.NRORDEM <= 0) {
            ScreenService.showMessage("O Número de Ordem deve ser maior que 0.");
        } else if (row.NRDIASTOL > 10) {
            ScreenService.showMessage("A tolerância deve ser igual ou menor a 10 dias.");
        } else if (row.NRDIASTOL < 0) {
            ScreenService.showMessage("A tolerância deve ser maior ou igual que 0 dias.");
        } else {
            if (DADOSBANCO === "S") {
                row.IDBANCO = "S", row.IDAGENCIA = "S", row.IDCONTCORR = "S";
            } else {
                row.IDBANCO = "N", row.IDAGENCIA = "N", row.IDCONTCORR = "N";
            }
            dataWidget.currentRow = row;
            if (standardWidget.name == 'addOrdemGeral') {
                row.__is_new = true;
                row.isUp = "N";
                dataWidget.saveRow().then(function () {
                    standardWidget.template = 'widget/form-view.html';
                    ScreenService.successNotification("Salvo com sucesso!");
                    dataWidget.actions[0].isVisible = true;
                    ScreenService.closeSwipe();
                    dataWidget.reload();
                    dataWidget.activate();
                }).catch(function (error) {
                    switch (error.data.error) {
                        case "ORDERSTD":
                            ScreenService.showMessage("Ordem de regra já cadastrada para a Adquirente informada.", "ERROR");
                            ScreenService.openSwipe(standardWidget);
                            standardWidget.edit();
                            standardWidget.activate();
                            dataWidget.reload();
                            break;
                        default:
                            ScreenService.showMessage(error.data.error, "ERROR");
                            break;
                    }
                });
            } else {
                row.__is_new = false;
                row.isUp = "S";
                dataWidget.saveRow().then(function () {
                    standardWidget.template = 'widget/form-view.html';
                    ScreenService.successNotification("Salvo com sucesso!");
                    dataWidget.actions[0].isVisible = true;
                    standardWidget.view();
                    ScreenService.closeSwipe();
                    dataWidget.reload();
                    dataWidget.activate();
                }).catch(function (error) {
                    switch (error.data.error) {
                        case "ORDERSTD":
                            ScreenService.showMessage("Ordem de regra já cadastrada para a Adquirente informada.", "ERROR");
                            ScreenService.openSwipe(standardWidget);
                            standardWidget.edit();
                            standardWidget.activate();
                            dataWidget.reload();
                            break;
                        default:
                            ScreenService.showMessage(error.data.error, "ERROR");
                            break;
                    }
                });
            }
        }
    };

    this.alterarOrdemEst = function (standardWidget) {

        var NRADQUIRENTE = standardWidget.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        var NRESTABELEC = standardWidget.container.getWidget('criterioEstabelec').currentRow.NRESTABELEC;
        var dataWidget = templateManager.container.getWidget('criterioOrdemEstabelec');

        var row = standardWidget.currentRow;
        row.NRADQUIRENTE = NRADQUIRENTE;
        row.NRESTABELEC = NRESTABELEC;
        var DADOSBANCO = row.DADOSBANCO;

        if (row.NRDIASTOL == null) row.NRDIASTOL = 0;
        if (row.NRORDEM == null) {
            ScreenService.showMessage("Número de Ordem deve ser informado.");
        } else if (row.NRORDEM <= 0) {
            ScreenService.showMessage("O Número de Ordem deve ser maior que 0.");
        } else if (row.NRDIASTOL > 10) {
            ScreenService.showMessage("A tolerância deve ser igual ou menor a 10 dias.");
        } else if (row.NRDIASTOL < 0) {
            ScreenService.showMessage("A tolerância deve ser maior ou igual que 0 dias.");
        } else {
            if (DADOSBANCO === "S") {
                row.IDBANCO = "S", row.IDAGENCIA = "S", row.IDCONTCORR = "S";
            } else {
                row.IDBANCO = "N", row.IDAGENCIA = "N", row.IDCONTCORR = "N";
            }
            dataWidget.currentRow = row;
            if (standardWidget.name == 'addOrdemEst') {
                row.__is_new = true;
                row.isUp = "N";
                dataWidget.saveRow().then(function () {
                    standardWidget.template = 'widget/form-view.html';
                    ScreenService.successNotification("Salvo com sucesso!");
                    dataWidget.actions[0].isVisible = true;
                    ScreenService.closeSwipe();
                    dataWidget.reload();
                    dataWidget.activate();
                }).catch(function (error) {
                    switch (error.data.error) {
                        case "ORDERSTDEST":
                            ScreenService.showMessage("Ordem de regra já cadastrada para a Adquirente e Estabelecimentos informados.", "ERROR");
                            ScreenService.openSwipe(standardWidget);
                            standardWidget.edit();
                            standardWidget.activate();
                            dataWidget.reload();
                            break;
                        default:
                            ScreenService.showMessage(error.data.error, "ERROR");
                            break;
                    }
                });
            } else {
                row.__is_new = false;
                row.isUp = "S";
                dataWidget.saveRow().then(function () {
                    standardWidget.template = 'widget/form-view.html';
                    ScreenService.successNotification("Salvo com sucesso!");
                    dataWidget.actions[0].isVisible = true;
                    standardWidget.view();
                    ScreenService.closeSwipe();
                    dataWidget.reload();
                    dataWidget.activate();
                }).catch(function (error) {
                    switch (error.data.error) {
                        case "ORDERSTDEST":
                            ScreenService.showMessage("Ordem de regra já cadastrada para a Adquirente e Estabelecimentos informados.", "ERROR");
                            ScreenService.openSwipe(standardWidget);
                            standardWidget.edit();
                            standardWidget.activate();
                            dataWidget.reload();
                            break;
                        default:
                            ScreenService.showMessage(error.data.error, "ERROR");
                            break;
                    }
                });
            }
        }
    };

    this.removerOrdemGeral = function (widget) {
        var NRADQUIRENTE = widget.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        var widgetPrt = widget.container.getWidget('tipoCriterio');
        ScreenService.confirmMessage('Deseja deletar a Ordem de Critérios ?', 'question', function () {
            widget.dataSource.remove([widget.currentRow]).then(function (message) {
                widgetPrt.dataSourceFilter = [{
                    "name": "NRADQUIRENTE",
                    "operator": "=",
                    "value": NRADQUIRENTE
                }, {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('NRORGTRAB')
                }];
                widgetPrt.dataSource.load(widgetPrt.dataSourceFilter).then(function (dado) {
                    ScreenService.successNotification('Deletado com sucesso!');
                    ScreenService.closeSwipe();
                    widgetPrt.dataSource.data = dado.dataset.Cnc00100_BuscaOrdensRepo;
                    widgetPrt.activate();
                });
            });
        }, function () {});
    };

    this.removerOrdemEst = function (widget) {
        var NRADQUIRENTE = widget.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        var NRESTABELEC = widget.container.getWidget('criterioEstabelec').currentRow.NRESTABELEC;
        var widgetPrt = widget.container.getWidget('criterioOrdemEstabelec');
        widget.currentRow.NRESTABELEC = NRESTABELEC;
        ScreenService.confirmMessage('Deseja deletar a Ordem de Critérios ?', 'question', function () {
            widget.dataSource.remove([widget.currentRow]).then(function (message) {
                widgetPrt.dataSourceFilter = [{
                    "name": "NRADQUIRENTE",
                    "operator": "=",
                    "value": NRADQUIRENTE
                }, {
                    "name": "NRESTABELEC",
                    "operator": "=",
                    "value": NRESTABELEC
                }, {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('NRORGTRAB')
                }];
                widgetPrt.dataSource.load(widgetPrt.dataSourceFilter).then(function (dado) {
                    ScreenService.successNotification('Deletado com sucesso!');
                    ScreenService.closeSwipe();
                    widgetPrt.dataSource.data = dado.dataset.Cnc00100_BuscaOrdensEstRepo;
                    widgetPrt.activate();
                });
            });
        }, function () {});
    };

    this.openSwipeAddCriterio = function (widget) {
        widget.currentRow = {};
        widget.fields[0].value(null);
        widget.fields[1].value(0);
        widget.fields[2].value('N');
        widget.fields[3].value('N');
        widget.fields[4].value('N');
        widget.fields[5].value('N');
        widget.fields[6].value('N');
        widget.fields[7].value('N');
        widget.fields[8].value('N');
        widget.fields[9].value('N');
        widget.fields[10].value('N');
        widget.fields[11].value('N');
        widget.fields[12].value('N');
        ScreenService.openSwipe(widget);
        widget.edit();
    };

    this.PositionChange = function (TextField) {
        TextField.readOnly = !TextField.readOnly;
    };

    this.openStandards = function (sonWidget) {
        var NRADQUIRENTE = sonWidget.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        sonWidget.dataSource.load(sonWidget.dataSourceFilter).then(function (dado) {
            var params = dado.dataset.Cnc00100_BuscaOrdensRepo[0];
            sonWidget.fields[0].value(params.NRORDEM);
            sonWidget.fields[1].value(params.NRDIASTOL);
            sonWidget.fields[2].value(params.IDNSU);
            sonWidget.fields[3].value(params.IDNSUPOS);
            sonWidget.fields[4].value(params.IDTOLERANCIA);
            sonWidget.fields[5].value(params.IDAUTORIZACAO);
            sonWidget.fields[6].value(params.DADOSBANCO);
            sonWidget.fields[7].value(params.IDPRODUTOADQ);
            sonWidget.fields[8].value(params.IDADQUIRENTE);
            sonWidget.fields[9].value(params.IDBANDEIRA);
            sonWidget.fields[10].value(params.IDEXIGEVRIGUAL);
            sonWidget.fields[11].value(params.IDAJUSTEAUT);
            sonWidget.fields[12].value(params.IDDTVENCIMENTO);
            sonWidget.fields[13].value(params.IDCONSTXADMANT);
        });
    };

    this.salvarCadastro00100 = function (widget) {

        var standardWidget = widget.container.getWidget('tipoCriterio');
        var aqcuireWidget = widget.container.getWidget('adquirente');
        var standardEstWidget = widget.container.getWidget('criterioOrdemEstabelec');
        var NRESTABELEC = widget.getField('NRESTABELEC').value();
        var NRADQUIRENTE = widget.getField('NRADQUIRENTE').value();

        var row = widget.currentRow;
        var DADOSBANCO = row.DADOSBANCO;
        row.IsCadAut = true;

        if (widget.isValid()) {
            if (row.NRDIASTOL == null) row.NRDIASTOL = 0;
            if (row.NRORDEM == null) {
                ScreenService.showMessage("Número de Ordem deve ser informado.");
            } else if (row.NRORDEM <= 0) {
                ScreenService.showMessage("O Número de Ordem deve ser maior que 0.");
            } else if (row.NRDIASTOL > 10) {
                ScreenService.showMessage("A tolerância deve ser igual ou menor a 10 dias.");
            } else if (row.NRDIASTOL < 0) {
                ScreenService.showMessage("A tolerância deve ser maior ou igual que 0 dias.");
            } else {
                if (DADOSBANCO === "S") {
                    row.IDBANCO = "S", row.IDAGENCIA = "S", row.IDCONTCORR = "S";
                } else {
                    row.IDBANCO = "N", row.IDAGENCIA = "N", row.IDCONTCORR = "N";
                }
                if (NRADQUIRENTE == "") {
                    ScreenService.showMessage("Pelo menos uma Adquirente ou Estabelecimento deverá ser informado.");
                } else if (NRESTABELEC == "") {
                    ScreenService.confirmMessage('Ao realizar a operação parametrizações existentes serão sobrepostas. Deseja continuar?', 'question', function () {
                        standardWidget.currentRow = row;
                        var promisesArr = [];
                        var filters = [];
                        NRADQUIRENTE.forEach(function (rowAdquirente, index) {
                            standardWidget.currentRow.NRADQUIRENTE = rowAdquirente;
                            var filter = [{
                                "name": "NRADQUIRENTE",
                                "operator": "=",
                                "value": rowAdquirente
                            }, {
                                "name": "NRORDEM",
                                "operator": "=",
                                "value": standardWidget.currentRow.NRORDEM
                            }, {
                                "name": "NRORG",
                                "operator": "=",
                                "value": getLocalVar('NRORGTRAB')
                            }];
                            filters.push(filter);
                        });
                        ZHPromise.all(filters.map(function (ff) {
                            return restEngine.post('cnc#/Cnc00100_BuscaOrdensCadRepo', {
                                "requestType": "FilterData",
                                "filter": ff
                            });
                        })).then(function (dado) {
                            dado.forEach(function (dates) {
                                if (!dates.dataset.Cnc00100_BuscaOrdensCadRepo[0]) {
                                    standardWidget.currentRow.__is_new = true;
                                    standardWidget.currentRow.NRADQUIRENTE = dates.dataset.Cnc00100_BuscaOrdensCadRepo.NRADQUIRENTE;
                                    promisesArr = [standardWidget.saveRow()];
                                } else {
                                    standardWidget.currentRow = standardWidget.container.getWidget('cadastroAutomatico').currentRow;
                                    standardWidget.currentRow.__is_new = false;
                                    standardWidget.currentRow.NRADQUIRENTE = dates.dataset.Cnc00100_BuscaOrdensCadRepo[0].NRADQUIRENTE;
                                    standardWidget.currentRow.NRVALIDACAOADQ = dates.dataset.Cnc00100_BuscaOrdensCadRepo[0].NRVALIDACAOADQ;
                                    promisesArr = [standardWidget.saveRow()];
                                }
                            });
                            ZHPromise.all(promisesArr).then(function () {
                                ScreenService.successNotification("Salvo com sucesso!");
                                ScreenService.closePopup();
                                standardWidget.reload();
                                aqcuireWidget.activate();
                            });
                        });
                    }), function () {};
                } else {
                    ScreenService.confirmMessage('Ao realizar a operação, parametrizações existentes podem ser sobrepostas. Deseja continuar?', 'question', function () {
                        standardEstWidget.currentRow = row;
                        var promisesArr = [];
                        var filters = [];
                        NRADQUIRENTE.forEach(function (rowAdquirente, index) {
                            standardEstWidget.currentRow.NRADQUIRENTE = rowAdquirente;
                            NRESTABELEC.forEach(function (rowEstabelec, index) {
                                standardEstWidget.currentRow.NRESTABELEC = rowEstabelec;
                                var filter = [{
                                    "name": "NRADQUIRENTE",
                                    "operator": "=",
                                    "value": rowAdquirente
                                }, {
                                    "name": "NRESTABELEC",
                                    "operator": "=",
                                    "value": rowEstabelec
                                }, {
                                    "name": "NRORDEM",
                                    "operator": "=",
                                    "value": standardEstWidget.currentRow.NRORDEM
                                }, {
                                    "name": "NRORG",
                                    "operator": "=",
                                    "value": getLocalVar('NRORGTRAB')
                                }];
                                filters.push(filter);
                            });
                        });
                        ZHPromise.all(filters.map(function (ff) {
                            return restEngine.post('cnc#/Cnc00100_BuscaOrdensEstCadRepo', {
                                "requestType": "FilterData",
                                "filter": ff
                            });
                        })).then(function (dado) {
                            dado.forEach(function (dates) {
                                if (!dates.dataset.Cnc00100_BuscaOrdensEstCadRepo[0]) {
                                    standardEstWidget.currentRow.__is_new = true;
                                    standardEstWidget.currentRow.NRADQUIRENTE = dates.dataset.Cnc00100_BuscaOrdensEstCadRepo.NRADQUIRENTE;
                                    standardEstWidget.currentRow.NRESTABELEC = dates.dataset.Cnc00100_BuscaOrdensEstCadRepo.NRESTABELEC;
                                    promisesArr = [standardEstWidget.saveRow()];
                                } else {
                                    standardEstWidget.currentRow = standardEstWidget.container.getWidget('cadastroAutomatico').currentRow;
                                    standardEstWidget.currentRow.__is_new = false;
                                    standardEstWidget.currentRow.NRADQUIRENTE = dates.dataset.Cnc00100_BuscaOrdensEstCadRepo[0].NRADQUIRENTE;
                                    standardEstWidget.currentRow.NRESTABELEC = dates.dataset.Cnc00100_BuscaOrdensEstCadRepo[0].NRESTABELEC;
                                    standardEstWidget.currentRow.NRVALIDACAOADQEST = dates.dataset.Cnc00100_BuscaOrdensEstCadRepo[0].NRVALIDACAOADQEST;
                                    promisesArr = [standardEstWidget.saveRow()];
                                }
                            });
                            ZHPromise.all(promisesArr).then(function () {
                                ScreenService.successNotification("Salvo com sucesso!");
                                ScreenService.closePopup();
                                standardWidget.reload();
                                aqcuireWidget.activate();
                            });
                        });
                    }), function () {};
                }
            }
        }
    };

    this.clearEstabelecStandarts = function (widget) {
        var adquirente = widget.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
        var swipe = templateManager.container.getWidget('criterioEstabelec');
        var standarts = widget.getCheckedRows();
        standarts.forEach(function (row, index) {
            standarts[index].NRADQUIRENTE = adquirente;
        });

        ScreenService.confirmMessage('Você realmente deseja limpar os critérios do(s) estabelecimento(s) selecionados?', 'question', function (_) {
            ZHPromise.all(standarts.map(function (row) {
                return widget.dataSource.remove([row]);
            })).then(function (_) {
                widget.reload();
                var NRADQUIRENTE = swipe.container.getWidget('adquirente').currentRow.NRADQUIRENTE;
                swipe.activate();
                swipe.dataSourceFilter[0].value = NRADQUIRENTE;
                swipe.fields[3].reload();
                ScreenService.showMessage("Limpo com sucesso!");
            });
        });
    };

    this.voltar = function (widget) {
        ScreenService.closeSwipe();
        widget.activate();
        ScreenService.updateTemplate();
    };

    this.cancelEdit = function (widget) {
        self.openStandards(templateManager.container.getWidget('tipoCriterio'));
        widget.template = 'widget/form-view.html';
        templateManager.container.getWidget('tipoCriterio').actions[0].isVisible = true;
        widget.view();
    };

    this.justInteger = function (selector, retryCount) {
        if (!retryCount || retryCount > 5) retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function () {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonInteger;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justInteger(selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonInteger(e) {
        var integerNumbers = [48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var numberCode = e.which;
        if (integerNumbers.indexOf(numberCode) < 0) {
            preventEvent(e);
        }
    };

    function preventEvent(e) {
        e.preventDefault();
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00100_CriteriosConsController', Cnc00100_CriteriosConsController);
});

// FILE: ./assets/js/controllers/Cnc00200_AdquirentesController.js
function Cnc00200_AdquirentesController(ScreenService, restEngine, templateManager) {
    var _this2 = this;

    this.getWidget = function (args) {
        return args.owner;
    };

    this.enableField = function (field) {
        field.readOnly = false;
        return field;
    };

    this.disableField = function (field) {
        field.readOnly = true;
        return field;
    };

    this.applyMaskToField = function (field) {
        return function (maskString) {
            return field.mask.params.mask = maskString;
        };
    };

    this.applyMaskToString = function (string) {
        return function (expression) {
            return function (format) {
                return string.replace(expression, format);
            };
        };
    };

    this.completeFieldFilterParam = function (field) {
        return function (paramName) {
            return function (paramValue) {
                field.dataSourceFilter.filter(function (param) {
                    return param.name === paramName;
                }).shift().value = paramValue;

                return field;
            };
        };
    };

    this.adquirentesActionEventEdit = function (args) {
        var widget = this.getWidget(args);

        widget.edit();
        widget.backUpRow = Util.clone(widget.currentRow);
    };

    this.adquirentesActionEventCancelEdit = function (args) {
        var widget = this.getWidget(args);

        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.view();
        }
    };

    this.adquirentesActionEventSave = function (args) {
        var widget = this.getWidget(args);

        var taxaAdm = widget.currentRow.VRTAXAADM;
        var tolerancia = widget.currentRow.VRTOLERANCIA;
        var idvisivel = widget.currentRow.IDVISIVEL;

        if (typeof tolerancia == 'string') tolerancia = this.parseCurrencyMaskToDouble(tolerancia);

        widget.currentRow.VRTAXAADM = taxaAdm;
        widget.currentRow.VRTOLERANCIA = tolerancia;
        widget.currentRow.IDVISIVEL = idvisivel;

        if (widget.isValid()) {
            delete widget.currentRow.ATIVO;
            widget.saveRow().then(function (response) {
                if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;

                widget.currentRow.changed = false;
                widget.view();
                widget.reload();
            }).catch(function (e) {
                ScreenService.showMessage(e.data.error);
            });
        }
    };

    this.getWidget = function (args) {
        return args.owner.widget;
    };

    this.parseCurrencyMaskToDouble = function (mask) {
        var number = mask.replace(/\D/g, '');
        var centsPosition = number.length - 2;
        number = [number.slice(0, centsPosition), '.', number.slice(centsPosition)].join('');
        return Number(number);
    };

    this.setFieldsMasks = function (args) {
        var widget = args.owner;

        if (widget.currentRow.CDAGENCIA && widget.currentRow.CDREDUCONT) {
            ["CDAGENCIA", "CDREDUCONT"].map(function (field) {
                var compose = _this2.applyMaskToField(widget.getField(field));
                field === "CDAGENCIA" ? compose(widget.currentRow.CDPICTAGEN) : compose(widget.currentRow.CDPICTCONT);
            });
        }
    };

    this.completeSerieFilter = function (widget) {
        var serie = widget.getField("CDSERIE");
        serie.readOnly = false;

        var filterMethod = function filterMethod(needle) {
            return function (obj) {
                return obj.name == needle;
            };
        };

        var mapMethod = function mapMethod(propName) {
            return function (obj) {
                return obj.value = widget.currentRow[propName];
            };
        };

        ['CDINSCESTA', 'CDINSCMUNI', 'CDEMPRESA'].map(function (filterName) {
            serie.dataSourceFilter.filter(filterMethod(filterName)).map(mapMethod(filterName));
        });

        serie.reload();
    };

    // Se vc n sabe RegExp, vai aprender.
    this.clienteMasks = function (widget) {
        var field = widget.getField("CDCLIENTE");

        field.dataSource.data.map(function (data) {
            switch (data.IDTPIJURCLIE) {
                case "F":
                    data.NRINSJURCLIE = _this2.applyMaskToString(data.NRINSJURCLIE)(/^(\d{3})(\d{3})(\d{3})(\d{2}).*/)("$1.$2.$3-$4");
                    break;
                case "J":
                    data.NRINSJURCLIE = _this2.applyMaskToString(data.NRINSJURCLIE)(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2}).*/)("$1.$2.$3/$4-$5");
                    break;
            }
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00200_AdquirentesController', Cnc00200_AdquirentesController);
});

// FILE: ./assets/js/controllers/Cnc00300_ProdutoAdquirenteController.js
function Cnc00300_ProdutoAdquirenteController(templateManager, ScreenService, restEngine, ZhUtil, General) {

    var nrseqadq;
    var nrseqprodadq;
    var self = this;

    this.produtosGridActivateTabbed = function (args) {
        var widget = args.owner;
        var tabbed = templateManager.container.getWidget('tabbed');
        var produtoDetails = templateManager.container.getWidget('produtoTab');
        nrseqprodadq = widget.currentRow.NRPRODUTOADQ;
        produtoDetails.activate();
        produtoDetails.reload();
        ScreenService.openSwipe(tabbed);
    };

    this.populateProdutoTabFields = function (produtoTab) {
        var produtoGrid = templateManager.container.getWidget('produtosGrid');
        produtoTab.getField('NMPRODUTOADQ').setValue(produtoGrid.currentRow.NMPRODUTOADQ);
        produtoTab.getField('VRTOLERANCIA').setValue(produtoGrid.currentRow.VRTOLERANCIA);
        produtoTab.getField('CDPRODUTOADQ').setValue(produtoGrid.currentRow.CDPRODUTOADQ);
        produtoTab.getField('CDTIPORECE').setValue(produtoGrid.currentRow.CDTIPORECE);
        produtoTab.getField('NRPRODUTOADQ').setValue(produtoGrid.currentRow.NRPRODUTOADQ);
        produtoTab.getField('NMBANDEIRA').setValue(produtoGrid.currentRow.NMBANDEIRA);
        produtoTab.getField('IDVISIVEL').setValue(produtoGrid.currentRow.IDVISIVEL);
        produtoTab.getField('NRDIASVENC').setValue(produtoGrid.currentRow.NRDIASVENC);
    };

    this.populatePeriodoTabFields = function (periodoTab) {
        periodoTab.getField('NRTAXAPERADQ').setValue(periodoTab.currentRow.NRTAXAPERADQ);
        periodoTab.getField('DTINICIAL').setValue(periodoTab.currentRow.DTINICIAL);
        periodoTab.getField('DTFINAL').setValue(periodoTab.currentRow.DTFINAL);
        periodoTab.getField('VRTAXAADM').setValue(periodoTab.currentRow.VRTAXAADM);
    };

    this.handleRecebimentoField = function (widget) {
        return widget.currentRow.CDTIPORECE = widget.currentRow.NMTIPORECE === "" ? null : widget.currentRow.CDTIPORECE;
    };

    this.produtoTabActionEventEditProduct = function (args) {
        var widget = getWidget(args);

        widget.edit();
        widget.template = widget.formTemplate;
        widget.backUpRow = Util.clone(widget.currentRow);
        templateManager.container.getWidget('adquirentesTab').isVisible = false;
        templateManager.container.getWidget('tiposDeRecebimento').isVisible = false;
    };

    this.produtoTabActionEventCancelEdit = function (args) {
        var widget = getWidget(args);

        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.view();
        }

        widget.template = widget.viewTemplate;
        templateManager.container.getWidget('adquirentesTab').isVisible = true;
        templateManager.container.getWidget('tiposDeRecebimento').isVisible = true;
    };

    this.produtoTabActionEventSaveProduct = function (args) {
        var widget = getWidget(args);
        var vrtolerancia = widget.currentRow.VRTOLERANCIA;
        var produtoGrid = templateManager.container.getWidget('produtosGrid');
        var nrorg = produtoGrid.currentRow.NRORG;
        var nrdiasvenc = widget.currentRow.NRDIASVENC;

        if (widget.isValid()) {
            this.fieldNmBandeiraOnChange(widget.getField('NMBANDEIRA'));

            if (typeof vrtolerancia == 'string') widget.currentRow.VRTOLERANCIA = this.parseStringToDouble(vrtolerancia);

            widget.currentRow.__is_new = false;
            if (!widget.currentRow.NRBANDEIRAVAN) widget.currentRow.NRBANDEIRAVAN = produtoGrid.currentRow.NRBANDEIRAVAN;

            restEngine.post('cnc#/cnc00300_produtoadquirente/save', {
                "requestType": "DataSet",
                "dataset": [widget.currentRow]
            }).then(function (response) {
                ScreenService.successNotification(response.notifications[0].message);
                widget.template = widget.viewTemplate;
                templateManager.container.getWidget('adquirentesTab').isVisible = true;
                templateManager.container.getWidget('tiposDeRecebimento').isVisible = true;
                produtoGrid.dataSource.reload();
                widget.reload();
                widget.view();
            }).catch(function (error) {
                switch (error.errorCode) {
                    case 1:
                        ScreenService.showMessage('A tolerância máxima permitida é de R$0,03.', 'ERROR');
                        break;
                    case 3:
                        ScreenService.showMessage('Dias para vencimento deve ser maior que 0.', 'ERROR');
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, 'ERROR');
                }
            });
        }
    };

    this.periodoTabOnEnterGetAndFormatData = function (swipe) {
        var NRPRODUTOADQ = swipe.container.getWidget('produtoTab').currentRow.NRPRODUTOADQ;
        var NRADQUIRENTE = swipe.container.getWidget('adquirentesTab').currentRow.NRADQUIRENTE;
        var NMPRODUTOADQ = swipe.container.getWidget('produtoTab').currentRow.NMPRODUTOADQ;
        var NMADQUIRENTE = swipe.container.getWidget('adquirentesTab').currentRow.NMADQUIRENTE;

        swipe.getField('NMPRODUTOADQ').value(NMPRODUTOADQ);
        swipe.getField('NMADQUIRENTE').value(NMADQUIRENTE);
        swipe.dataSourceFilter = [{
            "name": "NRPRODUTOADQ",
            "operator": "=",
            "value": NRPRODUTOADQ
        }, {
            "name": "NRADQUIRENTE",
            "operator": "=",
            "value": NRADQUIRENTE
        }, {
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }];
        swipe.dataSource.load(swipe.dataSourceFilter).then(function (dado) {
            swipe.dataSource.data = dado.dataset.cnc00300_adquirentePeriodos;
            swipe.activate();
        });
    };

    this.periodoTabOnCancelEdit = function (widget) {
        var NRPRODUTOADQ = widget.container.getWidget('produtoTab').currentRow.NRPRODUTOADQ;
        var NRADQUIRENTE = widget.container.getWidget('adquirentesTab').currentRow.NRADQUIRENTE;
        var NRTAXAPERADQ = widget.container.getWidget('periododiferenciadoTab').currentRow.NRTAXAPERADQ;
        if (NRTAXAPERADQ === undefined || NRTAXAPERADQ === "") {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
        } else {
            widget.dataSourceFilter = [{
                "name": "NRPRODUTOADQ",
                "operator": "=",
                "value": NRPRODUTOADQ
            }, {
                "name": "NRADQUIRENTE",
                "operator": "=",
                "value": NRADQUIRENTE
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('NRORGTRAB')
            }, {
                "name": "NRTAXAPERADQ",
                "operator": "=",
                "value": NRTAXAPERADQ
            }];
            widget.dataSource.load(widget.dataSourceFilter).then(function (dado) {
                widget.currentRow = dado.dataset.Cnc00300_adquirentePeriodoRow[0];
                widget.dataSourceFilter = [{
                    "name": "NRPRODUTOADQ",
                    "operator": "=",
                    "value": NRPRODUTOADQ
                }, {
                    "name": "NRADQUIRENTE",
                    "operator": "=",
                    "value": NRADQUIRENTE
                }, {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('NRORGTRAB')
                }];
                widget.view();
                ScreenService.closeSwipe();
                widget.reload();
            });
        }
    };

    this.periodoTabActionEventEditProduct = function (widget) {
        widget.edit();
        ScreenService.updateTemplate();
    };

    this.periodoTabActionEventSaveProduct = function (PeriodTWidget) {
        var NRPRODUTOADQ = PeriodTWidget.container.getWidget('produtoTab').currentRow.NRPRODUTOADQ;
        var NRADQUIRENTE = PeriodTWidget.container.getWidget('adquirentesTab').currentRow.NRADQUIRENTE;
        var NRTAXAPERADQ = PeriodTWidget.container.getWidget('periododiferenciadoTab').currentRow.NRTAXAPERADQ;

        var row = PeriodTWidget.currentRow;
        var DTINICIAL = row.DTINICIAL;
        var DTFINAL = row.DTFINAL;
        var VRTAXAADM = row.VRTAXAADM;
        var inInterval = false;
        var nrsequencialRow = row.NRTAXAPERADQ;

        if (VRTAXAADM.toString().indexOf(",") != -1) VRTAXAADM = VRTAXAADM.replace(",", ".");
        if (DTINICIAL === "" || DTFINAL === "") {
            ScreenService.showMessage('Todas as datas devem ser informadas.', 'ERROR');
        } else {
            var dtinicmp = new Date(DTINICIAL.slice(3, 5) + '/' + DTINICIAL.slice(0, 2) + '/' + DTINICIAL.slice(6, 10));
            var dtfincmp = new Date(DTFINAL.slice(3, 5) + '/' + DTFINAL.slice(0, 2) + '/' + DTFINAL.slice(6, 10));
            var timeDiff = dtfincmp.getTime() - dtinicmp.getTime();
            var diffDays = Math.ceil(timeDiff / (1000 * 3600 * 24));

            if (diffDays < 0) {
                ScreenService.showMessage('A Data Final não pode anteceder a Data Inicial', 'ERROR');
            } else if (VRTAXAADM === "") {
                ScreenService.showMessage('A taxa deve ser informada.', 'ERROR');
            } else if (VRTAXAADM > 100) {
                ScreenService.showMessage('A taxa administrativa máxima é de 100%.', 'ERROR');
            } else {
                PeriodTWidget.dataSourceFilter = [{
                    "name": "NRPRODUTOADQ",
                    "operator": "=",
                    "value": NRPRODUTOADQ
                }, {
                    "name": "NRADQUIRENTE",
                    "operator": "=",
                    "value": NRADQUIRENTE
                }, {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('NRORGTRAB')
                }];
                PeriodTWidget.dataSource.load(PeriodTWidget.dataSourceFilter).then(function (dado) {
                    var NRPRODUTOADQ = PeriodTWidget.container.getWidget('produtoTab').currentRow.NRPRODUTOADQ;
                    var NRADQUIRENTE = PeriodTWidget.container.getWidget('adquirentesTab').currentRow.NRADQUIRENTE;
                    row.NRPRODUTOADQ = NRPRODUTOADQ;
                    row.NRADQUIRENTE = NRADQUIRENTE;
                    if (dado.dataset.cnc00300_adquirentePeriodos.length === 0) {
                        row.__is_new = true;
                        PeriodTWidget.saveRow().then(function (dados) {
                            if (!dados.messages) {
                                ScreenService.successNotification(dados.dataset.Cnc00300_InserePeriodoTaxaRepo[0]);
                                ScreenService.showMessage("Salvo com sucesso!");
                                PeriodTWidget.view();
                                self.periodoTabOnEnterGetAndFormatData(PeriodTWidget);
                                ScreenService.closeSwipe();
                            }
                        });
                    } else {
                        var NRTAXAPERADQ = PeriodTWidget.container.getWidget('periododiferenciadoTab').currentRow.NRTAXAPERADQ;
                        if (NRTAXAPERADQ === undefined || NRTAXAPERADQ === "") {
                            row.__is_new = true;
                            PeriodTWidget.saveRow().then(function (dados) {
                                if (!dados.messages) {
                                    ScreenService.successNotification(dados.dataset.Cnc00300_InserePeriodoTaxaRepo[0]);
                                    ScreenService.showMessage("Salvo com sucesso!");
                                    PeriodTWidget.view();
                                    self.periodoTabOnEnterGetAndFormatData(PeriodTWidget);
                                    ScreenService.closeSwipe();
                                }
                            });
                        } else {
                            row.__is_new = false;
                            row.NRTAXAPERADQ = NRTAXAPERADQ;
                            PeriodTWidget.saveRow().then(function (dados) {
                                if (!dados.messages) {
                                    ScreenService.successNotification(dados.dataset.Cnc00300_AlteraPeriodoTaxaRepo[0]);
                                    ScreenService.showMessage("Salvo com sucesso!");
                                    PeriodTWidget.view();
                                    self.periodoTabOnEnterGetAndFormatData(PeriodTWidget);
                                    ScreenService.closeSwipe();
                                }
                            });
                        }
                    }
                });
            }
        }
    };

    this.periodoTabActionEventDelete = function (args) {
        var tabWidget = args.container.getWidget('periododiferenciadoTab');
        ScreenService.confirmMessage('Deseja deletar este período?', 'question', function () {
            var NRPRODUTOADQ = args.container.getWidget('produtoTab').currentRow.NRPRODUTOADQ;
            var NRADQUIRENTE = args.container.getWidget('adquirentesTab').currentRow.NRADQUIRENTE;
            var NRTAXAPERADQ = args.container.getWidget('periododiferenciadoTab').currentRow.NRTAXAPERADQ;
            tabWidget.currentRow.NRPRODUTOADQ = NRPRODUTOADQ;
            tabWidget.currentRow.NRADQUIRENTE = NRADQUIRENTE;
            tabWidget.currentRow.NRTAXAPERADQ = NRTAXAPERADQ;
            tabWidget.dataSource.remove([tabWidget.currentRow]).then(function (message) {
                ScreenService.showMessage("Excluído com sucesso!");
                ScreenService.goBack();
            });
        }, function () {});
    };

    this.handleDiasVencField = function (widget) {
        var taxa = widget.getField("VRTAXAADMEDIT");
        var dias = widget.getField("NRDIASVENC");

        if (dias.value()) {
            taxa.validations = { required: true };
            dias.validations = { required: true };
        } else {
            delete taxa.validations;
            delete dias.validations;
        }
    };

    this.adquirentesTabActionEventEditAdq = function (args) {
        var widget = getWidget(args);
        var fieldTaxaAdmLabel = widget.getField('VRTAXAADM');
        var fieldTaxaAdmEdit = widget.getField('VRTAXAADMEDIT');

        self.handleDiasVencField(widget);
        fieldTaxaAdmEdit.setValue(fieldTaxaAdmLabel.value());
        widget.widgets[0].enabled = false;
        widget.widgets[1].enabled = false;
        widget.edit();
        widget.backUpRow = Util.clone(widget.currentRow);
    };

    this.adquirentesTabActionEventCancelEditAdq = function (args) {
        var widget = getWidget(args);
        widget.widgets[0].enabled = true;
        widget.widgets[1].enabled = true;
        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.reload();
            widget.view();
        }
    };

    this.adquirentesTabActionEventSaveAdq = function (args) {
        var widget = getWidget(args);
        var vrtaxaadm = widget.getField('VRTAXAADMEDIT').value();

        if (widget.isValid()) {

            if (typeof vrtaxaadm == 'string' && vrtaxaadm != '') widget.currentRow.VRTAXAADM = this.parseStringToDouble(vrtaxaadm);

            widget.currentRow.VRTAXAADM = vrtaxaadm;
            widget.currentRow.__is_new = false;
            if (widget.currentRow.VRTAXAADM.toString().indexOf(",") != -1) widget.currentRow.VRTAXAADM = widget.currentRow.VRTAXAADM.replace(",", ".");
            restEngine.post('cnc#/cnc00300_produtoadquirente/saveAdq', {
                "requestType": "DataSet",
                "dataset": [widget.currentRow]
            }).then(function (response) {
                ScreenService.successNotification(response.notifications[0].message.slice(0, response.notifications[0].message.indexOf('|')));
                widget.getField('VRTAXAADM').setValue(response.notifications[0].message.slice(response.notifications[0].message.indexOf('|') + 1, response.notifications[0].message.length));
                widget.widgets[0].enabled = true;
                widget.widgets[1].enabled = true;
                widget.reload();
                widget.view();
            }).catch(function (error) {
                switch (error.errorCode) {
                    case 1:
                        ScreenService.showMessage('O valor da taxa administrativa deve ser maior que 0.', 'ERROR');
                        break;
                    case 2:
                        ScreenService.showMessage('A taxa administrativa máxima é de 100%.', 'ERROR');
                        break;
                    case 3:
                        ScreenService.showMessage('Os dias para vencimento devem ser maior que 0. ', 'ERROR');
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, 'ERROR');
                }
            });
        }
    };

    this.estabelecTabActionEventNew = function (args) {
        var widget = getWidget(args);
        widget.newRow();
    };

    this.estabelecTabActionEventCancelEdit = function (args) {
        var widget = getWidget(args);

        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.view();
        }
    };

    this.estabelecTabActionEventEdit = function (args) {
        var widget = getWidget(args);
        var produtoGrid = templateManager.container.getWidget('produtosGrid');
        widget.edit();
        widget.dataSourceFilter[2].value = nrseqadq;
        widget.dataSourceFilter[3].value = nrseqprodadq;
        restEngine.post('cnc#/cnc00300_produtoadquirente_estabelec', {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRORG",
                "operator": "=",
                "value": produtoGrid.currentRow.NRORG
            }, {
                "name": "IDATIVO",
                "operator": "=",
                "value": widget.dataSourceFilter[1].value
            }, {
                "name": "NRADQUIRENTE",
                "operator": "=",
                "value": widget.dataSourceFilter[2].value
            }, {
                "name": "NRPRODUTOADQ",
                "operator": "=",
                "value": widget.dataSourceFilter[3].value
            }]
        }).then(function () {
            widget.currentRow.NRTAXAADQPROD = widget.dataSource.data.filter(function (param) {
                return param.NRESTABELEC == widget.currentRow.NRESTABELEC;
            }).shift().NRTAXAADQPROD;
            widget.backUpRow = Util.clone(widget.currentRow);
        });
    };
    this.estabelecTabActionEventSave = function (args) {
        var widget = getWidget(args);
        var nrorg = getLocalVar('NRORGTRAB');
        var idativo = 'S';
        var adquirenteGrid = templateManager.container.getWidget('adquirentesTab');
        var produtoGrid = templateManager.container.getWidget('produtosGrid');
        var fieldTaxaAdmEdit = adquirenteGrid.getField('VRTAXAADMEDIT');

        if (widget.isNewRow()) {
            if (widget.isValid()) {
                widget.currentRow.NRESTABELEC = widget.currentRow.NRESTABELEC;
                widget.currentRow.NRADQUIRENTE = nrseqadq;
                widget.currentRow.NRPRODUTOADQ = nrseqprodadq;
                if (!widget.currentRow.NRESTABELEC) {
                    ScreenService.showMessage('Selecione o estabelecimento', 'ERROR');
                } else {
                    widget.currentRow.__is_new = false;
                    if (widget.currentRow.VRTAXAADM.toString().indexOf(",") != -1) widget.currentRow.VRTAXAADM = widget.currentRow.VRTAXAADM.replace(",", ".");
                    restEngine.post('cnc#/cnc00300_produtoadquirente/saveEstabelec', {
                        "requestType": "DataSet",
                        "dataset": [widget.currentRow]
                    }).then(function (response) {
                        ScreenService.successNotification(response.notifications[0].message);
                        widget.reload();
                        widget.view();
                    }).catch(function (error) {
                        switch (error.errorCode) {
                            case 1:
                                ScreenService.showMessage('O formato informado da taxa é inválido.', 'ERROR');
                                widget.currentRow.__is_new = true;
                                break;
                            case 2:
                                ScreenService.showMessage('Estabelecimento informado já possui uma taxa cadastrada.', 'ERROR');
                                widget.currentRow.__is_new = true;
                                break;
                            case 3:
                                ScreenService.showMessage('A taxa administrativa máxima é de 100%.', 'ERROR');
                                widget.currentRow.__is_new = true;
                                break;
                            case 4:
                                ScreenService.showMessage('O valor da taxa administrativa deve ser maior que 0.', 'ERROR');
                                widget.currentRow.__is_new = true;
                                break;
                            case 5:
                                ScreenService.showMessage('Taxa padrão já cadastrada para todos os estabelecimentos.', 'ERROR');
                                widget.currentRow.__is_new = true;
                                break;
                            default:
                                ScreenService.showMessage(error.data.error, 'ERROR');
                        }
                    });
                }
            }
        } else {
            if (widget.isValid()) {
                widget.currentRow.NRESTABELEC = widget.currentRow.NRESTABELEC;
                widget.currentRow.NRADQUIRENTE = nrseqadq;
                widget.currentRow.NRPRODUTOADQ = nrseqprodadq;
                if (widget.currentRow.VRTAXAADM.toString().indexOf(",") != -1) widget.currentRow.VRTAXAADM = widget.currentRow.VRTAXAADM.replace(",", ".");
                if (!widget.currentRow.NRESTABELEC) {
                    ScreenService.showMessage('Selecione o estabelecimento', 'ERROR');
                } else {
                    restEngine.post('cnc#/cnc00300_produtoadquirente/saveEditEstabelec', {
                        "requestType": "DataSet",
                        "dataset": [widget.currentRow]
                    }).then(function (response) {
                        ScreenService.successNotification(response.notifications[0].message);
                        widget.reload();
                        widget.view();
                    }).catch(function (error) {
                        switch (error.errorCode) {
                            case 1:
                                ScreenService.showMessage('O formato informado da taxa é inválido.', 'ERROR');
                                break;
                            case 2:
                                ScreenService.showMessage('A taxa administrativa máxima é de 100% (cem porcento).', 'ERROR');
                                break;
                            case 3:
                                ScreenService.showMessage('O valor da taxa administrativa deve ser maior que 0.', 'ERROR');
                                break;
                            case 4:
                                ScreenService.showMessage('Já possui uma taxa cadastrada para o estabelecimento informado.', 'ERROR');
                                break;
                            default:
                                ScreenService.showMessage(error.data.error, 'ERROR');
                        }
                    });
                }
            }
        }
    };

    this.estabelecTabActionEventDelete = function (args) {
        var widget = getWidget(args);
        var nrorg = getLocalVar('NRORGTRAB');
        var idativo = 'S';
        var adquirenteGrid = templateManager.container.getWidget('adquirentesTab');
        var produtoGrid = templateManager.container.getWidget('produtosGrid');
        var fieldTaxaAdmEdit = adquirenteGrid.getField('VRTAXAADMEDIT');

        if (widget.isValid()) {

            widget.currentRow.NRESTABELEC = widget.currentRow.NRESTABELEC;
            widget.currentRow.NRADQUIRENTE = nrseqadq;
            widget.currentRow.NRPRODUTOADQ = nrseqprodadq;

            ScreenService.confirmMessage('Deseja deletar este registro?', 'question', function () {
                restEngine.post('cnc#/cnc00300_produtoadquirente/removeEstabelec', {
                    "requestType": "DataSet",
                    "dataset": [widget.currentRow]
                }).then(function (response) {
                    ScreenService.successNotification(response.notifications[0].message);
                    ScreenService.closeSwipe();
                    widget.reload();
                }).catch(function (error) {
                    ScreenService.showMessage(error.data.error, 'ERROR');
                });
            });
        }
    };

    this.reloadProdutoGrid = function (args) {
        var produtoGrid = getWidget(args).getParent().getParent();
        produtoGrid.reload();
        produtoGrid.activate();
    };

    this.fieldNmBandeiraOnChange = function (args) {
        var widget = args.widget;
        var nmbandeira = widget.currentRow.NMBANDEIRA;

        args.dataSource.data.forEach(function (row, index) {
            if (row.NMBANDEIRA === nmbandeira) widget.currentRow.NRBANDEIRAVAN = row.NRBANDEIRAVAN;
        });
    };

    this.adquirentesTabAfterMoveRowGetNrSeqAdq = function (args) {
        var widget = args.owner;
        nrseqadq = widget.currentRow.NRADQUIRENTE;
    };

    this.adquirentesTabOnEnterGetAndFormatData = function (args) {
        var widget = args.owner;
        var produtoGrid = templateManager.container.getWidget('produtosGrid');

        widget.dataSourceFilter[2].value = nrseqprodadq;

        restEngine.post('cnc#/cnc00300_produtoadquirente_adquirentes', {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('NRORGTRAB')
            }, {
                "name": "IDATIVO",
                "operator": "=",
                "value": "S"
            }, {
                "name": "NRPRODUTOADQ",
                "operator": "=",
                "value": widget.dataSourceFilter[2].value
            }]
        }).then(function () {
            widget.dataSource.load(widget.dataSourceFilter).then(function () {
                widget.dataSource.data.forEach(function (row, index) {
                    var str = 'Padrão da adquirente (' + widget.dataSource.data[index].VRTAXAPADRAO + ')';
                    typeof widget.dataSource.data[index].VRTAXAADM.moneyToFloat() != "number" ? widget.dataSource.data[index].VRTAXAADM = str : widget.currentRow.VRTAXAADM;
                });
            });
        });
    };

    this.adquirentesTabAfterLoadDataSourceFormatTaxaColumn = function (args) {
        var widget = args.owner;

        widget.dataSource.data.forEach(function (row, index) {
            var str = 'Padrão da adquirente (' + widget.dataSource.data[index].VRTAXAPADRAO + ')';
            widget.dataSource.data[index].VRTAXAADM ? widget.currentRow.VRTAXAADM : widget.dataSource.data[index].VRTAXAADM = str;
        });
    };

    this.estabelecTabOnEnterSetFilter = function (args) {
        var widget = args.owner;
        var produtosGrid = templateManager.container.getWidget('produtosGrid');

        widget.dataSourceFilter[2].value = nrseqadq;
        widget.dataSourceFilter[3].value = nrseqprodadq;

        restEngine.post('cnc#/cnc00300_produtoadquirente_estabelec', {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRORG",
                "operator": "=",
                "value": produtosGrid.currentRow.NRORG
            }, {
                "name": "IDATIVO",
                "operator": "=",
                "value": widget.dataSourceFilter[1].value
            }, {
                "name": "NRADQUIRENTE",
                "operator": "=",
                "value": widget.dataSourceFilter[2].value
            }, {
                "name": "NRPRODUTOADQ",
                "operator": "=",
                "value": widget.dataSourceFilter[3].value
            }]
        }).then(function () {
            widget.reload();
        });
    };

    this.parseStringToDouble = function (mask) {
        var number = mask.replace(/\D-,-./g, '');
        return number.replace(',', '.');
    };

    this.justInteger = function (selector, retryCount) {
        if (!retryCount || retryCount > 5) retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function () {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonInteger;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justInteger(selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonInteger(e) {
        var integerNumbers = [48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var numberCode = e.which;
        if (integerNumbers.indexOf(numberCode) < 0) {
            preventEvent(e);
        }
    };

    function preventEvent(e) {
        e.preventDefault();
    };

    var getWidget = function getWidget(args) {
        return args.owner.widget;
    };

    this.saveTaxa = function (args) {
        var widget = args.owner.widget;
        if (widget.isValid()) {
            widget.currentRow.changed = true;
            return widget.saveRow().then(function (response) {
                if (response.messages) {
                    ScreenService.showMessage(response.messages[0].message);
                } else {
                    ScreenService.closeSwipe();
                    widget.currentRow.changed = false;
                    widget.currentRow.__is_new = false;
                    widget.view();
                }
            });
        }
    };

    this.deleteTaxa = function (args) {
        var widget = args.owner.widget;
        ScreenService.confirmMessage('Remover taxa?', 'question', function (success) {
            widget.removeRow().then(function (response) {
                ScreenService.closeSwipe().then(function () {
                    widget.activate();
                });
            });
        });
    };

    this.saveTipoRece = function (args) {
        var widget = args.owner.widget;
        if (widget.isValid()) {
            widget.currentRow.NRPRODUTOADQ = General.getFieldValue('NRPRODUTOADQ', 'produtoTab');
            widget.currentRow.NRORG = getLocalVar('NRORG');
            widget.currentRow.changed = true;
            return widget.saveRow().then(function (response) {
                if (response.messages) {
                    ScreenService.showMessage(response.messages[0].message);
                } else {
                    widget.reload();
                    ScreenService.closeSwipe();
                    widget.currentRow.changed = false;
                    widget.currentRow.__is_new = false;
                    widget.list();
                }
            });
        }
    };

    this.cancelTipoRece = function (args) {
        var widget = args.owner.widget;
        General.widgetCancelForm(widget);
    };

    this.novoTipoRece = function (args) {
        var widget = args.owner.widget;
        widget.newRow();
        widget.getField('NMTIPORECE').reload();
    };

    this.deleteTipoRece = function (args) {
        var widget = templateManager.container.getWidget('tiposDeRecebimento');
        General.deleteCrud(widget).then(function () {
            widget.reload();
        });
    };

    this.selectTipoRece = function (args) {
        var row = args.row;
        var widget = args.owner.field.widget;

        if (row.REGISTRADO == 'S') {
            ScreenService.showMessage("O Tipo de Recebimento selecionado já está sendo utilizado no produto " + row.NMPRODUTOADQ + ".");
            args.owner.field.value('');
            widget.getField('CDTIPORECE').value('');
        }
    };

    this.voltarTipoRece = function (args) {
        ScreenService.closeSwipe();
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00300_ProdutoAdquirenteController', Cnc00300_ProdutoAdquirenteController);
});

// FILE: ./assets/js/controllers/Cnc00400_VendaInexisAdquirente_Controller.js
function Cnc00400_VendaInexisAdquirente_Controller(templateManager, Filter, restEngine, ScreenService, ValidationEngine, ZHPromise, UtilitarioGeral, UtilitariosTela) {
    var _this3 = this;

    this.returnLeftGrid = function (_) {
        return templateManager.container.getWidget("vendasInexistentes");
    };
    this.returnRightGridSys = function (_) {
        return templateManager.container.getWidget("conciliacaoSistema");
    };

    this.getWidget = function (args) {
        return args.owner;
    };

    this.setIntervalToday = function (widget, fieldName) {
        widget.getField(fieldName).value({
            'start': moment().format('DD/MM/YYYY'),
            'period': undefined,
            'end': moment().format('DD/MM/YYYY')
        });
    };

    this.activateWidget = function (_) {
        templateManager.container.getWidget("dashboard").activate();
    };

    this.hideSelectAllRowsBox = function (_) {
        var x = document.querySelector("#grid-5417917823230315811953 > div.header > div > div > div.th.grid-checkbox-column.fixed.ng-scope");

        if (x) x.style.visibility = "hidden";

        var y = document.querySelector('#grid-3360718033156932614868 > div.header > div > div > div.th.grid-checkbox-column.fixed.ng-scope');

        if (y) y.style.visibility = "hidden";
    };

    this.resolveGridName = function (widgetName) {
        return widgetName === "vendasInexistentes" ? _this3.returnRightGridSys() : _this3.returnLeftGrid();
    };

    this.resolveActionName = function (nrconc) {
        return nrconc ? "desconciliacaoManual" : "conciliacaoManual";
    };

    this.removeArraysFromDataSource = function (widget) {
        return widget.dataSource.data = widget.dataSource.data.filter(function (row) {
            return row.length == undefined;
        });
    };

    this.doSearchRequests = function (widget) {
        var leftGrid = _this3.returnLeftGrid();
        var rightGrid = _this3.returnRightGridSys();
        var isEqual = true;

        var filterRow = void 0;
        switch (widget.name) {
            case "vendasInexistentes":
                filterRow = leftGrid.currentRow;
                break;
            case "conciliacaoSistema":
                filterRow = rightGrid.currentRow;
                break;
        }
        var dataSourceFilter = [{
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }, {
            "name": "NRESTABELEC",
            "operator": "=",
            "value": filterRow.NMESTABELEC.length == 0 ? "" : filterRow.NMESTABELEC
        }, {
            "name": "NRADQUIRENTE",
            "operator": "=",
            "value": filterRow.NMADQUIRENTE.length == 0 ? "" : filterRow.NMADQUIRENTE
        }, {
            "name": "DTVENDA",
            "operator": "BETWEEN",
            "value": [filterRow.DTVENDA.start, filterRow.DTVENDA.end]
        }, {
            "name": "CDNSU",
            "operator": "=",
            "value": filterRow.NSU
        }, {
            "name": "NRPRODUTOADQ",
            "operator": "=",
            "value": filterRow.NMPRODUTOADQ == 0 ? "" : filterRow.NMPRODUTOADQ
        }, {
            "name": "STATUS",
            "operator": "=",
            "value": filterRow.STATUS
        }, {
            "name": "IDVISPRODUTOADQ",
            "operator": "=",
            "value": filterRow.IDVISPRODUTOADQ
        }, {
            "name": "validator",
            "operator": "=",
            "value": true
        }, {
            "name": "isEqual",
            "operator": "=",
            "value": isEqual
        }];

        leftGrid.dataSourceFilter = rightGrid.dataSourceFilter = dataSourceFilter;

        restEngine.post("cnc#/cnc00400_conciliacao_manual", {
            "requestType": "FilterData",
            "filter": dataSourceFilter
        }).then(function (response) {
            leftGrid.dataSource.data = response.dataset.cnc00400_conciliacao_manual[0];
            rightGrid.dataSource.data = response.dataset.cnc00400_conciliacao_manual[1];
            _this3.sumFilteredValues(leftGrid);
            _this3.sumFilteredValues(rightGrid);
        }).catch(function (error) {
            if (error.data) {
                switch (error.data.errorCode) {
                    case 1:
                        ScreenService.showMessage("O intervalo máximo entre datas permitido é de um mês.", "ERROR");
                        Filter.startFiltering(leftGrid);
                        break;
                    case 2:
                        ScreenService.showMessage("Não é possível realizar a filtragem devido aos estabelecimentos serem de filiais diferentes.", "ERROR");
                        Filter.startFiltering(leftGrid);
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, "ERROR");
                        break;
                }
            } else {
                ScreenService.showMessage("Não foi possível recuperar os dados. Erro desconhecido. " + error.message, "ERROR");
            }
        });
    };

    this.sumFilteredValues = function (widget) {

        var sumFunc = function sumFunc(data) {
            return data.map(function (row) {
                return row.VRBRUTVENDA;
            }).reduce(function (accumulator, value) {
                return accumulator += parseFloat(value);
            }, 0).toFixed(2).toString().replace(/\./g, ",").replace(/(\d)(?=(\d{3})+(?!\d))/g, "$1.");
        };

        var footerStripe = widget.widgets[0];

        footerStripe.getField("TotalVen").label = "Total Filtrados: R$";
        footerStripe.getField("TotalVen").value(sumFunc(widget.dataSource.data));
    };

    this.checkEvenTypes = function (args) {
        var widget = _this3.getWidget(args);
        var bool = true;

        if (widget.getCheckedRows().length > 1) {
            var checkedRows = widget.getCheckedRows();

            if (checkedRows.filter(function (row) {
                return row.NRCONCLANC;
            }).length > 0) {
                ScreenService.showMessage('Não é possível selecionar a venda pois a mesma se encontra conciliada', 'ERROR');
                widget.selectedRow.__isSelected = false;
                bool = false;
            }
        }

        return bool;
    };

    this.checkConciliacaoType = function (args) {
        if (_this3.checkEvenTypes(args)) {
            var widget = _this3.getWidget(args);
            var lastRow = widget.getCheckedRows()[widget.getCheckedRows().length - 1];

            if (lastRow && lastRow.NRCONCLANC) {
                var targetRows = widget.dataSource.data.filter(function (data) {
                    return data.NRCONCLANC == lastRow.NRCONCLANC;
                }).map(function (data) {
                    return Object.assign({}, data, { __isSelected: true });
                });

                widget.dataSource.data = targetRows;
                _this3.filterSecondGrid(_this3.resolveGridName(widget.name), lastRow.NRCONCLANC);
            }

            _this3.enablAction(widget);
        }
    };

    this.filterSecondGrid = function (widget, nrconc) {
        var targetRows = widget.dataSource.data.filter(function (data) {
            return data.NRCONCLANC == nrconc;
        }).map(function (data) {
            return Object.assign({}, data, { __isSelected: true });
        });

        widget.dataSource.data = targetRows;
        _this3.sumFilteredValues(widget);
        _this3.sumFilteredValues(_this3.resolveGridName(widget.name));
        _this3.sumCheckedRowsAndCompleteStripe(widget);
    };

    this.removeUncheckedRowsFromDataSource = function (widget) {
        var targetRows = widget.dataSource.data.filter(function (row) {
            return row.__isSelected;
        });

        if (targetRows.length > 0) widget.dataSource.data = targetRows;
    };

    this.checkTwoSidesSelection = function (_) {
        return _this3.returnLeftGrid().getCheckedRows().length && _this3.returnRightGridSys().getCheckedRows().length;
    };

    this.enablAction = function (widget) {
        var parent = widget.getParent();

        var actionName = _this3.resolveActionName(widget.getCheckedRows().slice().shift().NRCONCLANC);

        if (_this3.returnLeftGrid().getCheckedRows().length > 0 && _this3.returnRightGridSys().getCheckedRows().length > 0) {
            parent.actions.find(function (action) {
                return action.name === actionName;
            }).isVisible = true;
        }

        _this3.checkTwoSidesSelection();

        parent.list();

        parent.redraw();

        parent.activate();

        templateManager.updateTemplate();
    };

    this.disableAction = function (widget) {
        var parent = widget.getParent();

        var actionName = _this3.resolveActionName(widget.getCheckedRows().slice().shift().NRCONCLANC);

        parent.actions.find(function (action) {
            return action.name === actionName;
        }).isVisible = false;

        parent.list();

        parent.redraw();

        parent.activate();

        templateManager.updateTemplate();
    };

    this.handleUnchecking = function (args) {
        var widget = _this3.getWidget(args);
        var checked = widget.getCheckedRows();
        var bool = checked.reduce(function (acc, row) {
            return row.NRCONCLANC && acc;
        }, true);

        bool ? _this3.resetDataSource(widget, true) : checked.length == 1 ? _this3.disableAction(widget) : false;
    };

    this.resetDataSource = function (widget, bool) {
        var dataSourceLength = widget.dataSource.data.length;
        var filtredRows = widget.dataSource.data.filter(function (data) {
            return data.NRCONCLANC == widget.dataSource.data[0].NRCONCLANC;
        });

        if (filtredRows.length >= 1 && filtredRows.length == dataSourceLength || dataSourceLength == 0) {
            widget.dataSource.load(widget.dataSourceFilter).then(function (_) {
                return _this3.sumFilteredValues(widget);
            });

            dataSourceLength != 0 ? _this3.disableAction(widget) : false;
            bool ? _this3.resetDataSource(_this3.resolveGridName(widget.name), false) : false;
        }
    };

    this.sumCheckedRowsAndCompleteStripe = function (widget) {
        var sumCheckedValues = widget.getCheckedRows().map(function (row) {
            return row.VRBRUTVENDA;
        }).reduce(function (accumulator, value) {
            return accumulator += parseFloat(value);
        }, 0).toFixed(2).toString().replace(/\./g, ",").replace(/(\d)(?=(\d{3})+(?!\d))/g, "$1.");

        var footerStripe = widget.widgets[0];
        footerStripe.getField("TotalMovim").label = "Total Selecionados: R$";
        footerStripe.getField("TotalMovim").value(sumCheckedValues);
    };

    this.assignNrorgToRows = function (rows) {
        return rows.map(function (row) {
            return Object.assign({}, row, { "NRORG": getLocalVar('NRORGTRAB') });
        });
    };

    this.desconciliacaoManual = function (widget) {
        var leftGrid = _this3.returnLeftGrid();
        var rightGrid = _this3.returnRightGridSys();

        var leftRows = leftGrid.getCheckedRows();
        var rightRows = rightGrid.getCheckedRows();

        leftRows = _this3.assignNrorgToRows(leftRows);
        rightRows = _this3.assignNrorgToRows(rightRows);

        ScreenService.confirmMessage("Confirmar a desconciliação das movimentações selecionadas?", "question", function (_) {
            restEngine.post("cnc#/cnc00400_conciliar_manualmente_verificaBoletoBaixado", {
                "requestType": "Row",
                "row": [leftRows, rightRows]
            }).then(function (response) {
                var boletoBaixado = response.dataset.ConciliacaoManualFinanceiro.BOLETO;
                if (boletoBaixado) {
                    ScreenService.showMessage("As movimentações selecionadas não podem ser desconciliadas pois seus boletos se encontram baixados.");
                } else {
                    restEngine.post("cnc#/cnc00400_desconciliacao_manual", {
                        "requestType": "Row",
                        "row": [leftRows, rightRows]
                    }).then(function (_) {
                        _this3.resetFooterText();
                        leftGrid.reload();
                        rightGrid.reload();
                        widget.actions.filter(function (action) {
                            return action.name === "desconciliacaoManual";
                        }).shift().isVisible = false;
                        ScreenService.successNotification('Desconciliado!');
                    });
                }
            });
        });
    };

    this.conciliacaoManual = function (widget) {
        var leftGrid = _this3.returnLeftGrid();
        var rightGrid = _this3.returnRightGridSys();

        var contabWidget = widget.container.getWidget('errosContabil');
        var message = "";
        var isEqual = true;
        var isEqualParc = true;

        var leftRows = leftGrid.getCheckedRows();
        var rightRows = rightGrid.getCheckedRows();

        leftRows = _this3.assignNrorgToRows(leftRows);
        rightRows = _this3.assignNrorgToRows(rightRows);
        var totalLeft = 0;
        var totalRight = 0;

        leftRows.forEach(function (row, index) {
            totalLeft += row.VRBRUTVENDA;
            if (index != 0) {
                if (leftRows[0].CDFILIAL != row.CDFILIAL) isEqual = false;
            }
        });
        rightRows.forEach(function (row, index) {
            totalRight += row.VRBRUTVENDA;
            if (index != 0) {
                if (rightRows[0].CDFILIAL != row.CDFILIAL) isEqual = false;
            }
        });

        if (isEqual) {
            if (leftRows[0].CDFILIAL != rightRows[0].CDFILIAL) isEqual = false;
        }

        if (isEqualParc) {
            if (leftRows[0].NRPARCELA != rightRows[0].NRPARCELA)
                // isEqualParc = false;
                isEqualParc = true;
        }

        var diferencaMonetaria = Math.abs(totalRight - totalLeft);
        //if(diferencaMonetaria > 0.10){
        //    ScreenService.showMessage("O valor a ser conciliado ultrapassa o limite de tolerância de R$ 0,10.", "ERROR");
        //}else{
        if (isEqual && isEqualParc) {
            ScreenService.confirmMessage("Confirmar a conciliação da(s) movimentação(ões) selecionada(s)?", "question", function (_) {
                restEngine.post("cnc#/cnc00400_conciliar_manualmente_verificaFinanceiro", {
                    "requestType": "Row",
                    "row": [leftRows, rightRows]
                }).then(function (response) {
                    var resposta = response.dataset.ConciliacaoManualFinanceiro;
                    if (resposta.ONETOONE && resposta.GERAFINANCEIRO && !resposta.PERIODOCONTABIL) {
                        ScreenService.confirmMessage("Venda fora do período contabil. Ao continuar a contabilidade não será alterada! Deseja continuar?", "question", function (_) {
                            restEngine.post("cnc#/cnc00400_conciliar_manualmente", {
                                "requestType": "Row",
                                "row": [leftRows, rightRows]
                            }).then(function (response) {
                                _this3.resetFooterText();
                                leftGrid.reload();
                                rightGrid.reload();
                                widget.actions.filter(function (action) {
                                    return action.name === "conciliacaoManual";
                                }).shift().isVisible = false;
                                ScreenService.successNotification('Conciliado!');
                            }).catch(function (error) {
                                var index = error.data.error.indexOf("/");
                                var name = error.data.error.substr(0, index);
                                var valorRelac = error.data.error.substr(index + 1);
                                switch (name) {
                                    case "ERR":
                                        ScreenService.showMessage(valorRelac, 'ERROR');
                                        widget.reload();
                                        break;
                                    case "CTB":
                                        var valoresMsg = valorRelac.split("|");
                                        var CDEMPRESA = valoresMsg[0];
                                        var NRLANCCTBTEMP = valoresMsg[1];
                                        var row = contabWidget.currentRow;
                                        row.CDEMPRESA = CDEMPRESA;
                                        row.NRLANCCTBTEMP = NRLANCCTBTEMP;
                                        widget.reload();
                                        widget.activate();
                                        ScreenService.confirmMessage('Operação Bloqueada! Não foi possível realizar a conciliação devido a erros de contabilização. Deseja verificá-los?', 'question', function () {
                                            contabWidget.dataSourceFilter = [{
                                                "name": "CDEMPRESA",
                                                "operator": "=",
                                                "value": CDEMPRESA
                                            }, {
                                                "name": "NRLANCCTBTEMP",
                                                "operator": "=",
                                                "value": NRLANCCTBTEMP
                                            }];
                                            contabWidget.dataSource.load(contabWidget.dataSourceFilter).then(function (dado) {
                                                contabWidget.dataSource.data = [];
                                                contabWidget.dataSource.data = dado.dataset.Cnc00400_ManutencaoConciliacaoError;
                                                ScreenService.openPopup(contabWidget);
                                                widget.reload();
                                                widget.activate();
                                            });
                                        }, function () {
                                            widget.reload();
                                            widget.activate();
                                        });
                                        break;
                                    default:
                                        ScreenService.showMessage(error.data.error, 'ERROR');
                                }
                            });
                        });
                    } else {
                        if (resposta.BOLETO) {
                            ScreenService.confirmMessage("Os boletos para as movimentações já foram baixados. Apenas a conciliação será feita. Deseja continuar?", "question", function (_) {
                                restEngine.post("cnc#/cnc00400_conciliar_manualmente", {
                                    "requestType": "Row",
                                    "row": [leftRows, rightRows]
                                }).then(function (response) {
                                    _this3.resetFooterText();
                                    leftGrid.reload();
                                    rightGrid.reload();
                                    widget.actions.filter(function (action) {
                                        return action.name === "conciliacaoManual";
                                    }).shift().isVisible = false;
                                    ScreenService.successNotification('Conciliado!');
                                }).catch(function (error) {
                                    var index = error.data.error.indexOf("/");
                                    var name = error.data.error.substr(0, index);
                                    var valorRelac = error.data.error.substr(index + 1);
                                    switch (name) {
                                        case "ERR":
                                            ScreenService.showMessage(valorRelac, 'ERROR');
                                            widget.reload();
                                            break;
                                        case "CTB":
                                            var valoresMsg = valorRelac.split("|");
                                            var CDEMPRESA = valoresMsg[0];
                                            var NRLANCCTBTEMP = valoresMsg[1];
                                            var row = contabWidget.currentRow;
                                            row.CDEMPRESA = CDEMPRESA;
                                            row.NRLANCCTBTEMP = NRLANCCTBTEMP;
                                            widget.reload();
                                            widget.activate();
                                            ScreenService.confirmMessage('Operação Bloqueada! Não foi possível realizar a conciliação devido a erros de contabilização. Deseja verificá-los?', 'question', function () {
                                                contabWidget.dataSourceFilter = [{
                                                    "name": "CDEMPRESA",
                                                    "operator": "=",
                                                    "value": CDEMPRESA
                                                }, {
                                                    "name": "NRLANCCTBTEMP",
                                                    "operator": "=",
                                                    "value": NRLANCCTBTEMP
                                                }];
                                                contabWidget.dataSource.load(contabWidget.dataSourceFilter).then(function (dado) {
                                                    contabWidget.dataSource.data = [];
                                                    contabWidget.dataSource.data = dado.dataset.Cnc00400_ManutencaoConciliacaoError;
                                                    ScreenService.openPopup(contabWidget);
                                                    widget.reload();
                                                    widget.activate();
                                                });
                                            }, function () {
                                                widget.reload();
                                                widget.activate();
                                            });
                                            break;
                                        default:
                                            ScreenService.showMessage(error.data.error, 'ERROR');
                                    }
                                });
                            });
                        } else {
                            restEngine.post("cnc#/cnc00400_conciliar_manualmente", {
                                "requestType": "Row",
                                "row": [leftRows, rightRows]
                            }).then(function (response) {
                                _this3.resetFooterText();
                                leftGrid.reload();
                                rightGrid.reload();
                                widget.actions.filter(function (action) {
                                    return action.name === "conciliacaoManual";
                                }).shift().isVisible = false;
                                ScreenService.successNotification('Conciliado!');
                            }).catch(function (error) {
                                var index;
                                var name;
                                var valorRelac;

                                if (error.data) {
                                    index = error.data.error.indexOf("/");
                                    name = error.data.error.substr(0, index);
                                    valorRelac = error.data.error.substr(index + 1);
                                } else {
                                    index = error.error.indexOf("/");
                                    name = error.error.substr(0, index);
                                    valorRelac = error.error.substr(index + 1);
                                }

                                switch (name) {
                                    case "ERR":
                                        ScreenService.showMessage(valorRelac, 'ERROR');
                                        widget.reload();
                                        break;
                                    case "CTB":
                                        var valoresMsg = valorRelac.split("|");
                                        var CDEMPRESA = valoresMsg[0];
                                        var NRLANCCTBTEMP = valoresMsg[1];
                                        var row = contabWidget.currentRow;
                                        row.CDEMPRESA = CDEMPRESA;
                                        row.NRLANCCTBTEMP = NRLANCCTBTEMP;
                                        widget.reload();
                                        widget.activate();
                                        ScreenService.confirmMessage('Operação Bloqueada! Não foi possível realizar a conciliação devido a erros de contabilização. Deseja verificá-los?', 'question', function () {
                                            contabWidget.dataSourceFilter = [{
                                                "name": "CDEMPRESA",
                                                "operator": "=",
                                                "value": CDEMPRESA
                                            }, {
                                                "name": "NRLANCCTBTEMP",
                                                "operator": "=",
                                                "value": NRLANCCTBTEMP
                                            }];
                                            contabWidget.dataSource.load(contabWidget.dataSourceFilter).then(function (dado) {
                                                contabWidget.dataSource.data = [];
                                                contabWidget.dataSource.data = dado.dataset.Cnc00400_ManutencaoConciliacaoError;
                                                ScreenService.openPopup(contabWidget);
                                                widget.reload();
                                                widget.activate();
                                            });
                                        }, function () {
                                            widget.reload();
                                            widget.activate();
                                        });
                                        break;
                                    default:
                                        if (error.data) {
                                            ScreenService.showMessage(error.data.error, 'ERROR');
                                        } else {
                                            ScreenService.showMessage(error.error, 'ERROR');
                                        }
                                }
                            });
                        }
                    }
                });
            });
        } else if (isEqualParc) {
            ScreenService.showMessage("Não é possível executar a ação pois as filiais das movimentações são distintas.", "ERROR");
        } else {
            ScreenService.showMessage("Não é possível executar a ação pois as parcelas das movimentações são distintas.", "ERROR");
        }
    };

    this.resetFooterText = function (_) {
        var leftGrid = _this3.returnLeftGrid();
        var rightGrid = _this3.returnRightGridSys();

        leftGrid.widgets[0].getField("TotalMovim").value("Total Selecionados: R$ 0");
        rightGrid.widgets[0].getField("TotalMovim").value("Total Selecionados: R$ 0");
    };

    this.validateMinSaleValue = function (value, validationAttrs) {
        var minValueField = validationAttrs.field;
        var minValue = parseFloat(templateManager.container.getWidget(validationAttrs.widget).currentRow[minValueField]);
        value = parseFloat(value);
        return {
            "valid": minValue <= value || isNaN(minValue),
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Minimum value: ") + minValue
        };
    };
    ValidationEngine.registerValidation('minValue', this.validateMinSaleValue);

    this.validadeteMaxSaleValue = function (value, validationAttrs) {
        var maxValueField = validationAttrs.field;
        var maxValue = parseFloat(templateManager.container.getWidget(validationAttrs.widget).currentRow[maxValueField]);
        value = parseFloat(value);
        return {
            "valid": maxValue >= value || isNaN(maxValue),
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Maximum value: ") + maxValue
        };
    };
    ValidationEngine.registerValidation('maxValue', this.validadeteMaxSaleValue);

    this.customizeCssTop = function (_) {
        $(".zh-stripe").css("background-color", "#ffffff");
    };

    this.customizeCssWidget = function (_) {
        $(".zh-widget").css("top", "20px");
    };

    this.reloadRightWidget = function (_) {
        var rightGrid = _this3.returnRightGridSys();
        var left = _this3.returnLeftGrid();
        rightGrid.dataSourceFilter = left.dataSourceFilter;
        rightGrid.reload();
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00400_VendaInexisAdquirente_Controller', Cnc00400_VendaInexisAdquirente_Controller);
});

// FILE: ./assets/js/controllers/Cnc00500_VendaPorAdquirente_Controller.js
function Cnc00500_VendaPorAdquirente_Controller(ScreenService, restEngine, Filter) {
    this.openFilter = function (widget) {
        var rows = {
            DTVENDA: {
                start: moment().format('DD/MM/YYYY'),
                period: undefined,
                end: moment().format('DD/MM/YYYY')
            },
            TIPOVENDA: '1',
            TIPORECE: '1',
            NRORG: getLocalVar('NRORG')
        };

        Filter.openFilterWithRows(widget, rows);
    };

    this.setVendasAdqChartDataSourceFilter = function (widget) {
        var filterData = Filter.getAppliedFilters(widget);
        var row = Object.assign({}, filterData);

        var dashboard = widget.getParent();
        var tipovendLabel = dashboard.widgets.filter(function (widget) {
            return widget.name === "TIPOVENDALABEL";
        }).shift();

        widget.activate();

        tipovendLabel.isVisible = true;
        if (row.TIPOVENDA === '1') {
            tipovendLabel.dataSource.data[0].TIPO = 'Líquido';
        } else {
            tipovendLabel.dataSource.data[0].TIPO = 'Bruto';
        }

        if (row.TIPORECE === '1') {
            tipovendLabel.dataSource.data[1].TIPO = 'Recebidas';
        } else {
            tipovendLabel.dataSource.data[1].TIPO = 'Previstas';
        }

        if (!row.NRESTABELEC || row.NRESTABELEC.length == 0) {
            row.NRESTABELEC = "";
        }

        if (!row.NRADQUIRENTE || row.NRADQUIRENTE.length == 0) {
            row.NRADQUIRENTE = "";
        }

        restEngine.post('cnc#/cnc00500_vendaporadquirente', {
            "requestType": "Row",
            "row": row
        }).then(function (response) {
            widget.container.getWidget('vendasAdqChart').getAction('details').isVisible = response.dataset.cnc00500_vendaporadquirente.length !== 0;
            widget.dataSource.data = response.dataset.cnc00500_vendaporadquirente;
        });
    };

    this.openSwipeAndLoadDetails = function (widget) {
        ScreenService.confirmMessage("Devido à quantidade de vendas, o processamento pode demorar. Deseja continuar?", "question", function (_) {
            var swipeWidget = widget.widgets.filter(function (widget) {
                return widget.name === "detailsGrid";
            }).shift();

            var filterData = Filter.getAppliedFilters(widget);
            ScreenService.openSwipe(swipeWidget);
            swipeWidget.dataSourceFilter = [{
                "name": "NRESTABELEC",
                "operator": "IN",
                "value": filterData.NRESTABELEC
            }, {
                "name": "NRADQUIRENTE",
                "operator": "IN",
                "value": filterData.NRADQUIRENTE
            }, {
                "name": "DTVENDA",
                "operator": "BETWEEN",
                "value": [filterData.DTVENDA.start, filterData.DTVENDA.end]
            }, {
                "name": "TIPORECE",
                "operator": "=",
                "value": filterData.TIPORECE
            }, {
                "name": "TIPOVENDA",
                "operator": "=",
                "value": filterData.TIPOVENDA
            }];
            swipeWidget.dataSource.load(swipeWidget.dataSourceFilter);
        });
    };

    this.getWidget = function (args) {
        return args.owner;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00500_VendaPorAdquirente_Controller', Cnc00500_VendaPorAdquirente_Controller);
});

// FILE: ./assets/js/controllers/Cnc00600_VendaConciliada_Controller.js
function Cnc00600_VendaConciliada_Controller(templateManager, Filter, restEngine, Cnc00600_ConciliatorSellsService, ScreenService, UtilitariosTela) {

    this.updateValueConciliator = function (args) {
        var valuesWidget = templateManager.container.getWidget('vendasConciliadas');
        var nrestabelec = valuesWidget.getField('NRESTABELEC').value();
        var nmadquirente = valuesWidget.getField('NMADQUIRENTE').value();
        var nmprodutoadq = valuesWidget.getField('NMPRODUTOADQ').value();
        var dtvenda = valuesWidget.getField("DTVENDA").value();
        restEngine.post('cnc#/Cnc00600_GetVendasConciliadores', {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRESTABELEC",
                "operator": "IN",
                "value": nrestabelec
            }, {
                "name": "DTVENDA",
                "operator": "BETWEEN",
                "value": [dtvenda.start, dtvenda.end]
            }, {
                "name": "NMADQUIRENTE",
                "operator": "IN",
                "value": nmadquirente
            }, {
                "name": "NMPRODUTOADQ",
                "operator": "IN",
                "value": nmprodutoadq
            }]
        }).then(function (dataCons) {
            var valuesWidget = templateManager.container.getWidget('vendasConciliadas');
            var valuesConsWidget = templateManager.container.getWidget('dados_vendas_conciliadas');
            var NMPRODUTO = valuesWidget.getField('NMPRODUTOADQ').value();
            if (!(dataCons.dataset.Cnc00600_GetVendasConciliadores[0] === undefined)) {
                valuesWidget.dataSource.data[0] = dataCons.dataset.Cnc00600_GetVendasConciliadores[0];
                var NVCONCILIADO = valuesWidget.dataSource.data[0].NVCONCILIADO;
                var NVTOTAL = valuesWidget.dataSource.data[0].NVTOTAL;
            } else {
                var NVCONCILIADO = 0;
                var NVTOTAL = 0;
            }
            valuesWidget.label = "Vendas Conciliadas (QT)" + ' - ' + NMPRODUTO;
            if (args.owner.container == templateManager.container) {
                if (args.owner.chart) {
                    valuesWidget.activate();
                    valuesConsWidget.activate();
                    valuesConsWidget.reload();
                    args.owner.chart.maxValue(Math.round(NVTOTAL)).updateSeries();
                    args.owner.chart.value(Math.round(NVCONCILIADO)).updateSeries();
                    templateManager.container.getWidget('dados_vendas_conciliadas').dataSource.data[0].VLR = String(NVCONCILIADO);
                    templateManager.container.getWidget('dados_vendas_conciliadas').dataSource.data[1].VLR = String(NVTOTAL - NVCONCILIADO);
                    templateManager.container.getWidget('dados_vendas_conciliadas').dataSource.data[2].VLR = String(NVTOTAL);
                }
            }
        });
    };

    this.setFieldValueInt = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NMADQUIRENTE':
                    field.setValue('Todas');
                    break;
                case 'NMPRODUTOADQ':
                    field.setValue('');
                    break;
                default:
                    field.setValue('null');
            }
        }
    };

    this.clearValuesChart = function () {
        var valuesWidget = templateManager.container.getWidget('vendasConciliadas');

        Filter.startFiltering(valuesWidget);
        valuesWidget.dataSource.data = [];
    };

    this.openSwipeAndPopulateGrid = function (args) {
        var widgetTreeGrid = this.getWidget(args);
        var widgetViewTab = templateManager.container.getWidget('detalhamentoVendConcTreeGridView');
        var widgetView = templateManager.container.getWidget('detalhamentoVendasConciliadas');

        if (widgetTreeGrid.currentRow.PARENT != null) {
            ScreenService.openSwipe(widgetViewTab);
            widgetView.getField('DTVENDA').value(widgetTreeGrid.currentRow.DTVENDA);
            widgetView.getField('NMPRODUTOADQ').value(widgetTreeGrid.currentRow.NMPRODUTOADQ);
            widgetView.getField('NMADQUIRENTE').value(widgetTreeGrid.currentRow.NMADQUIRENTE);
            widgetView.getField('CDNSU').value(widgetTreeGrid.currentRow.CDNSU);
            widgetView.getField('CDCUPOM').value(widgetTreeGrid.currentRow.CDCUPOM);
            widgetView.getField('VRBRUTVENDA').value(widgetTreeGrid.currentRow.VRBRUTVENDA);
            widgetView.activate();
        }
    };

    this.showSellsDetails = function () {
        var valuesWidget = templateManager.container.getWidget('vendasConciliadas');
        var valuesDetailsWidget = templateManager.container.getWidget('detalhamentoVendConcTreeGrid');
        var NRADQUIRENTE = valuesWidget.getField('NMADQUIRENTE').value();
        var NMPRODUTOADQ = valuesWidget.getField('NMPRODUTOADQ').value();
        var NRESTABELEC = valuesWidget.getField('NRESTABELEC').value();
        var DTVENDA = valuesWidget.getField('DTVENDA').value();
        var DTVENDAINI = DTVENDA.start;
        var DTVENDAFIN = DTVENDA.end;

        Cnc00600_ConciliatorSellsService.getConciliatorSellsDetails(NRADQUIRENTE, NMPRODUTOADQ, NRESTABELEC, DTVENDAINI, DTVENDAFIN).then(function (dados) {
            valuesDetailsWidget.dataSource.data = dados;
            ScreenService.openSwipe(valuesDetailsWidget);
        });
    };

    this.clearFilterPopupFields00600 = function (widget) {

        Filter.startFiltering(widget);
        widget.getField('LABELESTABELEC').value('');
        UtilitariosTela.setIntervalToday(widget, 'DTVENDA');
        widget.getField('NMADQUIRENTE').value('Todas');
        widget.getField('NMPRODUTOADQ').value('');
        templateManager.container.getWidget('dados_vendas_conciliadas').dataSource.data[0].VLR = "";
        templateManager.container.getWidget('dados_vendas_conciliadas').dataSource.data[1].VLR = "";
        templateManager.container.getWidget('dados_vendas_conciliadas').dataSource.data[2].VLR = "";
    };

    this.getWidget = function (args) {
        return args.owner;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00600_VendaConciliada_Controller', Cnc00600_VendaConciliada_Controller);
});

// FILE: ./assets/js/controllers/Cnc00700_RecebimentoBandeira_Controller.js
function Cnc00700_RecebimentoBandeira_Controller(templateManager, Filter, restEngine, ScreenService) {

    var self = this;

    this.getWidget = function (args) {
        return args.owner;
    };

    this.clearFilterPopupFields = function (widget) {
        Filter.startFiltering(widget);
        widget.getField('NMESTABELEC').value('Todas');
        widget.getField('NMADQUIRENTE').value('Todas');
        widget.getField('NMPRODUTOADQ').value('Todos');
        widget.getField('NMBANDEIRA').value('Todas');
        widget.dataSource.data = [];
    };

    this.setFieldValue = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NMADQUIRENTE':
                    field.setValue('Todas');
                    break;
                case 'NMPRODUTOADQ':
                    field.setValue('Todos');
                    break;
                case 'NMESTABELEC':
                    field.setValue('Todas');
                    break;
                case 'NMBANDEIRA':
                    field.setValue('Todas');
                    break;
                default:
                    field.setValue('null');
            }
        }
    };

    this.toggleHabDate = function (paramsDate) {
        if (paramsDate.getField('intervalo').value() === 'M') {
            paramsDate.getField('MESINI').isVisible = true;
            paramsDate.getField('MESINI').validations.required = true;
            paramsDate.getField('MESFIN').isVisible = true;
            paramsDate.getField('MESFIN').validations.required = true;
            paramsDate.getField('DTVENDA').isVisible = false;
            paramsDate.getField('DTVENDA').value('');
        } else {
            paramsDate.getField('DTVENDA').isVisible = true;
            paramsDate.getField('MESINI').isVisible = false;
            paramsDate.getField('MESINI').validations.required = false;
            paramsDate.getField('MESINI').value('');
            paramsDate.getField('MESFIN').isVisible = false;
            paramsDate.getField('MESFIN').validations.required = false;
            paramsDate.getField('MESFIN').value('');
        }
    };

    this.estabelec = function (args) {
        var field = args.owner.owner;
        var nrField = field.widget.getField('NMESTABELEC');

        nrField.value(field.dataSource.data.filter(function (data) {
            return data.NMESTABELEC === field.value();
        }).shift().NRESTABELEC);
        field.widget.currentRow.NRESTABELEC = nrField.value();
    };

    this.updateValueReceive = function (widget) {
        self.setDescriptionValues();
        var valuesWidget = templateManager.container.getWidget('recebimentosBandeira');
        var NMESTABELEC = valuesWidget.currentRow.NMESTABELEC;
        var nmadquirente = valuesWidget.getField('NMADQUIRENTE').value();
        var nmprodutoadq = valuesWidget.getField('NMPRODUTOADQ').value();
        var nmbandeira = valuesWidget.getField('NMBANDEIRA').value();
        var mesini = valuesWidget.getField('MESINI').value();
        var mesfin = valuesWidget.getField('MESFIN').value();
        var dtvenda = valuesWidget.getField("DTVENDA").value();
        var intervalo = valuesWidget.getField("intervalo").value();
        var valorMostrado = valuesWidget.getField('valorMostrado').value();
        var condicaoRecebimento = valuesWidget.getField('condicaoRecebimento').value();
        if (intervalo === 'D') {
            restEngine.post('cnc#/Cnc00700_GetRecebimentosBandeira', {
                "requestType": "FilterData",
                "filter": [{
                    "name": "NMESTABELEC",
                    "operator": "IN",
                    "value": NMESTABELEC
                }, {
                    "name": "NMADQUIRENTE",
                    "operator": "IN",
                    "value": nmadquirente
                }, {
                    "name": "NMPRODUTOADQ",
                    "operator": "IN",
                    "value": nmprodutoadq
                }, {
                    "name": "NMBANDEIRA",
                    "operator": "IN",
                    "value": nmbandeira
                }, {
                    "name": "intervalo",
                    "operator": "LIKE",
                    "value": intervalo
                }, {
                    "name": "DTVENDA",
                    "operator": "BETWEEN",
                    "value": [dtvenda.start, dtvenda.end]
                }, {
                    "name": "valorMostrado",
                    "operator": "LIKE",
                    "value": valorMostrado
                }, {
                    "name": "condicaoRecebimento",
                    "operator": "LIKE",
                    "value": condicaoRecebimento
                }]
            }).then(function (flagD) {
                var valuesWidget = templateManager.container.getWidget('recebimentosBandeira');
                var valuesFlagWidget = templateManager.container.getWidget('dados_recebimentos_bandeira');
                valuesWidget.dataSource.data = flagD.dataset.Cnc00700_GetRecebimentosBandeira;
                var result = [];
                var oldmes = null;
                var boPrimVez = true;
                var boExisteValor = false;
                var chartSeriesFields = [];
                valuesWidget.activate();
                valuesFlagWidget.activate();
                if (valuesWidget.dataSource.data.length == 0) {
                    valuesFlagWidget.getAction('viewDetalhes').isVisible = false;
                } else {
                    valuesFlagWidget.getAction('viewDetalhes').isVisible = true;
                }
                var chartSeriesLabels = valuesWidget.dataSource.data.map(function (row) {
                    return row.NMBANDEIRA;
                }).filter(function (flagName, index, parentArray) {
                    return parentArray.indexOf(flagName) == index;
                });

                var jsonTemp = JSON.parse('{}');
                for (var i in valuesWidget.dataSource.data) {
                    var bandeira = valuesWidget.dataSource.data[i].NMBANDEIRA;
                    var periodo = valuesWidget.dataSource.data[i].DATAVENDA;
                    if (periodo) {
                        var dtordenation = new Date(periodo.slice(3, 5) + '/' + periodo.slice(0, 2) + '/' + periodo.slice(6, 8)).getTime();
                    }
                    if (boPrimVez) {
                        boPrimVez = false;
                        Object.defineProperty(jsonTemp, 'DATAVENDA', { value: periodo, enumerable: true });
                        Object.defineProperty(jsonTemp, 'DTORDENATION', { value: dtordenation, enumerable: true });
                    } else {
                        if (oldmes != periodo) {
                            result.push(jsonTemp);
                            boExisteValor = false;
                            jsonTemp = JSON.parse('{}');
                            oldmes = periodo;
                            Object.defineProperty(jsonTemp, 'DATAVENDA', { value: oldmes, enumerable: true });
                            Object.defineProperty(jsonTemp, 'DTORDENATION', { value: dtordenation, enumerable: true });
                        }
                    }
                    oldmes = periodo;

                    var strVr = "VR" + bandeira;
                    var valorbandeira = valuesWidget.dataSource.data[i].VRVENDA;
                    Object.defineProperty(jsonTemp, strVr, { value: valorbandeira, enumerable: true });
                    boExisteValor = true;
                }

                if (boExisteValor) {
                    result.push(jsonTemp);
                }

                result.forEach(function (rowdecima, indexdecima) {
                    var prpt = Object.getOwnPropertyNames(result[indexdecima]);
                    prpt.forEach(function (row, index) {
                        if (prpt[index].substr(0, 2) == "VR") {
                            if (chartSeriesFields.indexOf(prpt[index]) == -1) {
                                chartSeriesFields.push(prpt[index]);
                            }
                        }
                    });
                });

                widget.chart.drillUp();

                result.pop();
                result.sortBy("DTORDENATION");
                valuesWidget.dataSource.data = result;
                var options = {
                    dataSource: {
                        data: result
                    },
                    series: [{
                        fields: chartSeriesFields,
                        labels: chartSeriesLabels
                    }]
                };

                widget.chart.drillDown(options);
            });
        } else {
            restEngine.post('cnc#/Cnc00700_GetRecebimentosBandeira', {
                "requestType": "FilterData",
                "filter": [{
                    "name": "NMESTABELEC",
                    "operator": "IN",
                    "value": NMESTABELEC
                }, {
                    "name": "NMADQUIRENTE",
                    "operator": "IN",
                    "value": nmadquirente
                }, {
                    "name": "NMPRODUTOADQ",
                    "operator": "IN",
                    "value": nmprodutoadq
                }, {
                    "name": "NMBANDEIRA",
                    "operator": "IN",
                    "value": nmbandeira
                }, {
                    "name": "intervalo",
                    "operator": "LIKE",
                    "value": intervalo
                }, {
                    "name": "MESINI",
                    "operator": "LIKE",
                    "value": mesini
                }, {
                    "name": "MESFIN",
                    "operator": "LIKE",
                    "value": mesfin
                }, {
                    "name": "valorMostrado",
                    "operator": "LIKE",
                    "value": valorMostrado
                }, {
                    "name": "condicaoRecebimento",
                    "operator": "LIKE",
                    "value": condicaoRecebimento
                }]
            }).then(function (flagM) {
                var valuesWidget = templateManager.container.getWidget('recebimentosBandeira');
                var valuesFlagWidget = templateManager.container.getWidget('dados_recebimentos_bandeira');
                valuesWidget.dataSource.data = flagM.dataset.Cnc00700_GetRecebimentosBandeira;
                var result = [];
                var oldmes = null;
                var boPrimVez = true;
                var boExisteValor = false;
                var chartSeriesFields = [];
                valuesWidget.activate();
                valuesFlagWidget.activate();
                valuesFlagWidget.getAction('viewDetalhes').isVisible = false;
                var chartSeriesLabels = valuesWidget.dataSource.data.map(function (row) {
                    return row.NMBANDEIRA;
                }).filter(function (flagName, index, parentArray) {
                    return parentArray.indexOf(flagName) == index;
                });

                var jsonTemp = JSON.parse('{}');
                for (var i in valuesWidget.dataSource.data) {
                    var bandeira = valuesWidget.dataSource.data[i].NMBANDEIRA;
                    var periodo = valuesWidget.dataSource.data[i].DATAVENDA;
                    if (periodo) {
                        var dtordenation = new Date(periodo.slice(3, 5) + '/' + periodo.slice(0, 2) + '/' + periodo.slice(6, 8)).getTime();
                    }
                    if (boPrimVez) {
                        boPrimVez = false;
                        Object.defineProperty(jsonTemp, 'DATAVENDA', { value: periodo, enumerable: true });
                        Object.defineProperty(jsonTemp, 'DTORDENATION', { value: dtordenation, enumerable: true });
                    } else {
                        if (oldmes != periodo) {
                            result.push(jsonTemp);
                            boExisteValor = false;
                            jsonTemp = JSON.parse('{}');
                            oldmes = periodo;
                            Object.defineProperty(jsonTemp, 'DATAVENDA', { value: oldmes, enumerable: true });
                            Object.defineProperty(jsonTemp, 'DTORDENATION', { value: dtordenation, enumerable: true });
                        }
                    }
                    oldmes = periodo;

                    var strVr = "VR" + bandeira;
                    var valorbandeira = valuesWidget.dataSource.data[i].VRVENDA;
                    Object.defineProperty(jsonTemp, strVr, { value: valorbandeira, enumerable: true });
                    boExisteValor = true;
                }

                if (boExisteValor) {
                    result.push(jsonTemp);
                }

                result.forEach(function (rowdecima, indexdecima) {
                    var prpt = Object.getOwnPropertyNames(result[indexdecima]);
                    prpt.forEach(function (row, index) {
                        if (prpt[index].substr(0, 2) == "VR") {
                            if (chartSeriesFields.indexOf(prpt[index]) == -1) {
                                chartSeriesFields.push(prpt[index]);
                            }
                        }
                    });
                });

                widget.chart.drillUp();

                result.pop();
                result.sortBy("DTORDENATION");
                valuesWidget.dataSource.data = result;
                var options = {
                    dataSource: {
                        data: result
                    },
                    series: [{
                        fields: chartSeriesFields,
                        labels: chartSeriesLabels
                    }]
                };

                widget.chart.drillDown(options);
            });
        }
    };

    this.setDescriptionValues = function () {
        var valuesWidget = templateManager.container.getWidget('recebimentosBandeira');
        var NMESTABELEC = valuesWidget.currentRow.NMESTABELEC;
        var nmadquirente = valuesWidget.getField('NMADQUIRENTE').value();
        var nmprodutoadq = valuesWidget.getField('NMPRODUTOADQ').value();
        var nmbandeira = valuesWidget.getField('NMBANDEIRA').value();
        var mesini = valuesWidget.getField('MESINI').value();
        var mesfin = valuesWidget.getField('MESFIN').value();
        var dtvenda = valuesWidget.getField("DTVENDA").value();
        var intervalo = valuesWidget.getField("intervalo").value();
        var valorMostrado = valuesWidget.getField('valorMostrado').value();
        var condicaoRecebimento = valuesWidget.getField('condicaoRecebimento').value();
        if (dtvenda != "") {
            var dtinicmp = new Date(dtvenda.start.slice(3, 5) + '/' + dtvenda.start.slice(0, 2) + '/' + dtvenda.start.slice(6, 10));
            var dtfincmp = new Date(dtvenda.end.slice(3, 5) + '/' + dtvenda.end.slice(0, 2) + '/' + dtvenda.end.slice(6, 10));
            var timeDiff = dtfincmp.getTime() - dtinicmp.getTime();
            var diffDays = Math.ceil(timeDiff / (1000 * 3600 * 24));
        }if (mesini != "" && mesfin != "") {
            var mesinicmp = parseInt(mesini.slice(3, 7) + mesini.slice(0, 2));
            var mesfincmp = parseInt(mesfin.slice(3, 7) + mesfin.slice(0, 2));
            var mesinidt = "01" + "/" + mesini;
            var mesfinff = new Date(new Date(mesfin.slice(3, 7), mesfin.slice(0, 2), 1) - 1);
            var lastday = mesfinff.getDate();
            var mesfindt = lastday + "/" + mesfin;
        }
        if (valorMostrado === "L") {
            templateManager.container.getWidget('dados_recebimentos_bandeira').dataSource.data[1].VLR = "Líquido";
        } else {
            templateManager.container.getWidget('dados_recebimentos_bandeira').dataSource.data[1].VLR = "Bruto";
        }if (condicaoRecebimento === "R") {
            templateManager.container.getWidget('dados_recebimentos_bandeira').dataSource.data[0].VLR = "Recebidas";
        } else {
            templateManager.container.getWidget('dados_recebimentos_bandeira').dataSource.data[0].VLR = "Previstas";
        }
        if (mesfincmp < mesinicmp || mesfincmp - mesinicmp > 99) {
            Filter.startFiltering(valuesWidget);
            restEngine.post('cnc#/Cnc00700_GetRecebimentosBandeira', {
                "requestType": "FilterData",
                "filter": [{
                    "name": "NMESTABELEC",
                    "operator": "IN",
                    "value": NMESTABELEC
                }, {
                    "name": "NMADQUIRENTE",
                    "operator": "IN",
                    "value": nmadquirente
                }, {
                    "name": "NMPRODUTOADQ",
                    "operator": "IN",
                    "value": nmprodutoadq
                }, {
                    "name": "NMBANDEIRA",
                    "operator": "IN",
                    "value": nmbandeira
                }, {
                    "name": "intervalo",
                    "operator": "LIKE",
                    "value": intervalo
                }, {
                    "name": "MESINI",
                    "operator": "LIKE",
                    "value": mesini
                }, {
                    "name": "MESFIN",
                    "operator": "LIKE",
                    "value": mesfin
                }, {
                    "name": "valorMostrado",
                    "operator": "LIKE",
                    "value": valorMostrado
                }, {
                    "name": "condicaoRecebimento",
                    "operator": "LIKE",
                    "value": condicaoRecebimento
                }]
            }).then(function () {}).catch(function (error) {
                switch (error.data.error) {
                    case 'MESERROR':
                        ScreenService.showMessage('Data inicial maior que data final.', 'ERROR');
                        break;
                    case 'MESEXCEDEU':
                        ScreenService.showMessage('Devido ao espaço do gráfico, o intervalo deve ser menor ou igual a um ano.', 'ERROR');
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, 'ERROR');
                }
            });
        }if (diffDays > 14) {
            Filter.startFiltering(valuesWidget);
            restEngine.post('cnc#/Cnc00700_GetRecebimentosBandeira', {
                "requestType": "FilterData",
                "filter": [{
                    "name": "NMESTABELEC",
                    "operator": "IN",
                    "value": NMESTABELEC
                }, {
                    "name": "NMADQUIRENTE",
                    "operator": "IN",
                    "value": nmadquirente
                }, {
                    "name": "NMPRODUTOADQ",
                    "operator": "IN",
                    "value": nmprodutoadq
                }, {
                    "name": "NMBANDEIRA",
                    "operator": "IN",
                    "value": nmbandeira
                }, {
                    "name": "intervalo",
                    "operator": "LIKE",
                    "value": intervalo
                }, {
                    "name": "DTVENDA",
                    "operator": "BETWEEN",
                    "value": [dtvenda.start, dtvenda.end]
                }, {
                    "name": "valorMostrado",
                    "operator": "LIKE",
                    "value": valorMostrado
                }, {
                    "name": "condicaoRecebimento",
                    "operator": "LIKE",
                    "value": condicaoRecebimento
                }]
            }).then(function () {}).catch(function (error) {
                switch (error.data.error) {
                    case 'DIAERROR':
                        ScreenService.showMessage('Devido ao espaço do gráfico, o intervalo de data deve ser menor ou igual a 15 dias.', 'ERROR');
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, 'ERROR');
                }
            });
        }
    };

    this.openSwipeAndPopulateReceiptGrid = function (args) {
        var widgetTreeGrid = this.getWidget(args);
        var widgetViewTab = templateManager.container.getWidget('detalhamentoRecTreeGridView');
        var widgetView = templateManager.container.getWidget('detalhamentoRecebimentos');

        if (widgetTreeGrid.currentRow.PARENT != null) {
            ScreenService.openSwipe(widgetViewTab);
            widgetView.getField('DTVENDA').value(widgetTreeGrid.currentRow.DTVENDA);
            widgetView.getField('NMESTABELEC').value(widgetTreeGrid.currentRow.NMESTABELEC);
            widgetView.getField('NMPRODUTOADQ').value(widgetTreeGrid.currentRow.NMPRODUTOADQ);
            widgetView.getField('NMADQUIRENTE').value(widgetTreeGrid.currentRow.NMADQUIRENTE);
            widgetView.getField('NMBANDEIRA').value(widgetTreeGrid.currentRow.PARENT);
            widgetView.getField('CDNSU').value(widgetTreeGrid.currentRow.CDNSU);
            widgetView.getField('VRBRUTVENDA').value(widgetTreeGrid.currentRow.VRBRUTO);
            widgetView.activate();
        }
    };

    this.showReceiptDetails = function () {
        var valuesWidget = templateManager.container.getWidget('recebimentosBandeira');
        var valuesDetailsWidget = templateManager.container.getWidget('detalhamentoRecTreeGrid');
        var NRADQUIRENTE = valuesWidget.getField('NMADQUIRENTE').value();
        var NRPRODUTOADQ = valuesWidget.getField('NMPRODUTOADQ').value();
        var NRBANDEIRAVAN = valuesWidget.getField('NMBANDEIRA').value();
        var NRESTABELEC = valuesWidget.getField('NMESTABELEC').value();
        var DTVENDA = valuesWidget.getField('DTVENDA').value();
        if (DTVENDA === "" || !DTVENDA) {
            var DTVENDAINI = valuesWidget.getField('MESINI').value();
            var DTVENDAFIN = valuesWidget.getField('MESFIN').value();
        } else {
            var DTVENDAINI = DTVENDA.start;
            var DTVENDAFIN = DTVENDA.end;
        }
        var MEIORECE = valuesWidget.getField('condicaoRecebimento').value();
        var INTERVALO = valuesWidget.getField('intervalo').value();

        ScreenService.confirmMessage('Devido à quantidade de vendas, o processamento pode demorar. Deseja continuar?', 'question', function () {
            valuesDetailsWidget.dataSourceFilter = [{
                "name": "NRADQUIRENTE",
                "operator": "=",
                "value": NRADQUIRENTE
            }, {
                "name": "NRPRODUTOADQ",
                "operator": "=",
                "value": NRPRODUTOADQ
            }, {
                "name": "NRBANDEIRAVAN",
                "operator": "=",
                "value": NRBANDEIRAVAN
            }, {
                "name": "NRESTABELEC",
                "operator": "=",
                "value": NRESTABELEC
            }, {
                "name": "DTVENDAINI",
                "operator": "=",
                "value": DTVENDAINI
            }, {
                "name": "DTVENDAFIN",
                "operator": "=",
                "value": DTVENDAFIN
            }, {
                "name": "MEIORECE",
                "operator": "=",
                "value": MEIORECE
            }, {
                "name": "INTERVALO",
                "operator": "=",
                "value": INTERVALO
            }];
            valuesDetailsWidget.dataSource.load(valuesDetailsWidget.dataSourceFilter).then(function (dados) {
                valuesDetailsWidget.dataSource.data = dados.dataset.Cnc00700_GetRecebimentosBandeiraDetalhes;
                ScreenService.openSwipe(valuesDetailsWidget);
            });
        }), function () {};
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00700_RecebimentoBandeira_Controller', Cnc00700_RecebimentoBandeira_Controller);
});

// FILE: ./assets/js/controllers/Cnc00800_CompValoresRecebidos_Controller.js
function Cnc00800_CompValoresRecebidos_Controllers(templateManager, ScreenService, Filter, restEngine) {
    var _this4 = this;

    this.setCompVendasChartDataSourceFilter = function (args) {
        var widget = _this4.getWidget(args);
        widget.chart.drillUp();

        templateManager.container.getWidget('compValoresChart').activate();

        widget.getAction('viewDetalhes').isVisible = true;

        var filterData = Filter.getAppliedFilters(widget);

        if (widget.currentRow.PERIODO === 'M') {
            widget.getAction('viewDetalhes').isVisible = false;
        }

        restEngine.post('cnc#/cnc00800_compvendas', {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRESTABELEC",
                "operator": "IN",
                "value": filterData.NRESTABELEC
            }, {
                "name": "NRADQUIRENTE",
                "operator": "IN",
                "value": filterData.NRADQUIRENTE
            }, {
                "name": "NRPRODUTOADQ",
                "operator": "IN",
                "value": filterData.NRPRODUTOADQ
            }, {
                "name": "NRBANDEIRAVAN",
                "operator": "IN",
                "value": filterData.NRBANDEIRAVAN
            }, {
                "name": "DTVENDAINI",
                "operator": "=",
                "value": filterData.PERIODO === 'D' ? filterData.DTVENDA.start : filterData.DTVENDAMESINI
            }, {
                "name": "DTVENDAFIN",
                "operator": "=",
                "value": filterData.PERIODO === 'D' ? filterData.DTVENDA.end : filterData.DTVENDAMESFIN
            }, {
                "name": "PERIODO",
                "operator": "=",
                "value": filterData.PERIODO
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": '@getLocalVar("NRORGTRAB")@'
            }, {
                "name": "IDATIVO",
                "operator": "=",
                "value": 'S'
            }]
        }).then(function (response) {
            widget.dataSource.data = response.dataset.dataSet;
        }).catch(function (error) {
            switch (error.data.errorCode) {
                case 1:
                    ScreenService.showMessage('O mês inicial é posterior ao mês final.', 'ERROR');
                    Filter.startFiltering(widget);
                    break;
                case 2:
                    ScreenService.showMessage('O ano final é posterior ao ano inicial', 'ERROR');
                    Filter.startFiltering(widget);
                    break;
                case 3:
                    ScreenService.showMessage('Devido ao espaço do gráfico, o intervalo máximo é um ano.', 'ERROR');
                    Filter.startFiltering(widget);
                    break;
                case 4:
                    ScreenService.showMessage('Devido ao espaço do gráfico, o intervalo máximo é de quinze dias.', 'ERROR');
                    Filter.startFiltering(widget);
                    widget.getField('DTVENDAMESINI').isVisible = false;
                    widget.getField('DTVENDAMESFIN').isVisible = false;
                    widget.getField('DTVENDA').isVisible = true;
                    break;
                default:
                    ScreenService.showMessage(error.data.error, 'ERROR');
                    Filter.startFiltering(widget);
            }
        });
    };

    this.fieldPeriodoOnChangeSetIntervalVisibility = function (widget) {
        var fieldPeriodo = widget.getField('PERIODO');
        var chartWidget = fieldPeriodo.widget;

        if (fieldPeriodo.value() === 'M') {
            chartWidget.getField('DTVENDAMESINI').isVisible = true;
            chartWidget.getField('DTVENDAMESFIN').isVisible = true;
            chartWidget.getField('DTVENDAMESINI').validations.required = true;
            chartWidget.getField('DTVENDAMESFIN').validations.required = true;
            chartWidget.getField('DTVENDA').isVisible = false;
        } else {
            chartWidget.getField('DTVENDAMESINI').isVisible = false;
            chartWidget.getField('DTVENDAMESFIN').isVisible = false;
            chartWidget.getField('DTVENDAMESINI').validations.required = false;
            chartWidget.getField('DTVENDAMESFIN').validations.required = false;
            chartWidget.getField('DTVENDA').isVisible = true;
        }

        chartWidget.redraw();
        templateManager.updateTemplate();
    };

    this.fixFieldsVisibility = function (args) {
        var chartWidget = templateManager.container.getWidget('compValoresChart');
        var filterData = Filter.getAppliedFilters(chartWidget);

        if (filterData.PERIODO === 'D') {
            chartWidget.getField('DTVENDAMESINI').isVisible = false;
            chartWidget.getField('DTVENDAMESFIN').isVisible = false;
            chartWidget.getField('DTVENDA').isVisible = true;
        }
    };

    this.openSwipeAndPopulateComGrid = function (args) {
        var widgetTreeGrid = this.getWidget(args);
        var widgetViewTab = templateManager.container.getWidget('comparacaoValoresView');
        var widgetView = templateManager.container.getWidget('recebimentosComparados');

        ScreenService.openSwipe(widgetViewTab);
        widgetView.getField('DTVENDA').value(widgetTreeGrid.currentRow.DTVENDA);
        widgetView.getField('NMPRODUTOADQ').value(widgetTreeGrid.currentRow.NMPRODUTOADQ);
        widgetView.getField('NMADQUIRENTE').value(widgetTreeGrid.currentRow.NMADQUIRENTE);
        widgetView.getField('NMBANDEIRA').value(widgetTreeGrid.currentRow.NMBANDEIRA);
        widgetView.getField('CDNSU').value(widgetTreeGrid.currentRow.CDNSU);
        widgetView.getField('VRESPERADO').value(widgetTreeGrid.currentRow.VRESPERADO);
        widgetView.getField('VRRECEBIDO').value(widgetTreeGrid.currentRow.VRRECEBIDO);
        widgetView.activate();
    };

    this.showReceiptedValuesDetails = function () {
        var valuesWidget = templateManager.container.getWidget('compValoresChart');
        var filterData = Filter.getAppliedFilters(valuesWidget);
        var valuesDetailsWidget = templateManager.container.getWidget('comparacaoValoresSwipe');

        var NRADQUIRENTE = filterData.NRADQUIRENTE;
        var NRPRODUTOADQ = filterData.NRPRODUTOADQ;
        var NRBANDEIRAVAN = filterData.NRBANDEIRAVAN;
        var NRESTABELEC = filterData.NRESTABELEC;
        var DTVENDA = filterData.DTVENDA;
        var PERIODO = filterData.PERIODO;

        var DTVENDAINI = void 0;
        var DTVENDAFIN = void 0;

        if (PERIODO === 'M') {
            DTVENDAINI = filterData.DTVENDAMESINI;
            DTVENDAFIN = filterData.DTVENDAMESFIN;
        } else {
            DTVENDAINI = filterData.DTVENDA.start;
            DTVENDAFIN = filterData.DTVENDA.end;
        }

        var INTERVALO = filterData.PERIODO;

        ScreenService.confirmMessage('Devido à quantidade de vendas, o processamento pode demorar. Deseja continuar?', 'question', function () {
            valuesDetailsWidget.dataSourceFilter = [{
                "name": "NRADQUIRENTE",
                "operator": "=",
                "value": NRADQUIRENTE
            }, {
                "name": "NRPRODUTOADQ",
                "operator": "=",
                "value": NRPRODUTOADQ
            }, {
                "name": "NRBANDEIRAVAN",
                "operator": "=",
                "value": NRBANDEIRAVAN
            }, {
                "name": "NRESTABELEC",
                "operator": "=",
                "value": NRESTABELEC
            }, {
                "name": "DTVENDAINI",
                "operator": "=",
                "value": DTVENDAINI
            }, {
                "name": "DTVENDAFIN",
                "operator": "=",
                "value": DTVENDAFIN
            }, {
                "name": "PERIODO",
                "operator": "=",
                "value": PERIODO
            }];
            valuesDetailsWidget.dataSource.load(valuesDetailsWidget.dataSourceFilter).then(function (dados) {
                valuesDetailsWidget.dataSource.data = dados.dataset.Cnc00800_GetComparacaoValoresDetalhes;
                ScreenService.openSwipe(valuesDetailsWidget);
            });
        }), function () {};
    };

    this.getWidget = function (args) {
        return args.owner;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00800_CompValoresRecebidos_Controllers', Cnc00800_CompValoresRecebidos_Controllers);
});

// FILE: ./assets/js/controllers/Cnc00900_VendasAntecipadas_Controller.js
function Cnc00900_VendasAntecipadas_Controller(templateManager, Filter, UtilitariosTela, ScreenService, Cnc00900_VendasAntecipadasDetalhesRepo) {

    var self = this;

    this.setIntervalToday = function (widget, fieldName) {
        widget.getField(fieldName).value({
            'start': moment().format('DD/MM/YYYY'),
            'period': undefined,
            'end': moment().format('DD/MM/YYYY')
        });
    };

    this.vendaAntecipadaWidgetOnEnter = function (args) {
        self.clearFilterPopupFields(templateManager.container.getWidget('vendasAntecipadas'));
    };

    this.clearFilterPopupFields = function (widget) {
        var row = {
            NMESTABELEC: null,
            NMADQUIRENTE: 'Todas',
            NMPRODUTOADQ: 'Todas',
            DTCREDITO: {
                'start': moment().format('DD/MM/YYYY'),
                'period': undefined,
                'end': moment().format('DD/MM/YYYY')
            }
        };

        Filter.openFilterWithRows(widget, row);
    };

    this.loadDetalhesVendasAntecipadas = function (swipe) {
        var row = swipe.container.getWidget('vendasAntecipadas').currentRow;

        var NRADQUIRENTE = row.NRADQUIRENTE;
        var CDLOTEVENDA = row.CDLOTEVENDA;
        var NRPRODUTOADQ = row.NMPRODUTOADQ;
        var VRTAXAANT = row.VRTAXAANT;
        var NRESTABELEC = row.NRESTABELEC;

        swipe.dataSourceFilter = [{
            "name": "NRADQUIRENTE",
            "operator": "IN",
            "value": NRADQUIRENTE
        }, {
            "name": "CDLOTEVENDA",
            "operator": "IN",
            "value": CDLOTEVENDA
        }, {
            "name": "NRPRODUTOADQ",
            "operator": "IN",
            "value": NRPRODUTOADQ
        }, {
            "name": "VRTAXAANT",
            "operator": "IN",
            "value": VRTAXAANT
        }, {
            "name": "NRESTABELEC",
            "operator": "IN",
            "value": NRESTABELEC
        }];
        swipe.dataSource.load(swipe.dataSourceFilter).then(function (dados) {
            ScreenService.openSwipe(swipe);
            swipe.dataSource.data = dados.dataset.Cnc00900_VendasAntecipadasDetalhesRepo || [];
            swipe.activate();
        });
    };

    this.setFieldValueChar = function (field) {
        if (field.value() === '') {
            field.value('null');
        }
    };

    this.setFieldValueInt = function (field) {
        if (field.value().length === 0) {
            field.value('Todas');
        }
    };

    this.setFieldValueCharSimple = function (field) {
        if (field.value() === '') {
            field.value('Todas');
        }
    };

    this.setFieldValueIntSimple = function (field) {
        if (field.value().length === 0) {
            field.value('Todas');
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc00900_VendasAntecipadas_Controller', Cnc00900_VendasAntecipadas_Controller);
});

// FILE: ./assets/js/controllers/Cnc01000_VendasConciliadas_Controller.js
function Cnc01000_VendasConciliadas_Controller(ScreenService, templateManager, Filter, ValidationEngine, restEngine) {

    this.openFilter = function (widget, fieldDateName) {
        var rows = {
            STATUSCON: 'T',
            STATUSSIT: 'T',
            FINRECEBIDO: 'T'
        };

        rows[fieldDateName] = {
            'start': moment().format('DD/MM/YYYY'),
            'period': undefined,
            'end': moment().format('DD/MM/YYYY')
        };

        Filter.openFilterWithRows(widget, rows);
    };

    var populateFooterStripeFirstField = function populateFooterStripeFirstField(widgetGrid, widgetTreeGrid) {
        widgetTreeGrid.widgets[1].getField('ADQTOTAL').value(widgetGrid.currentRow.NMADQUIRENTE + ": R$" + widgetGrid.currentRow.VRLIQPGTO.toString().replace('.', ','));

        widgetTreeGrid.widgets[1].getField('ADQTOTALPREV').value(": R$" + widgetGrid.currentRow.VRLIQVENDA.toString().replace('.', ','));
    };

    this.clearCurrentFooterStripeValeu = function () {
        var widgetTreeGrid = templateManager.container.getWidget('detalhamentoLoteTreeGrid');

        widgetTreeGrid.widgets[1].getField('BANDTOTAL').value('');
    };

    this.setFieldValueSt = function (field) {
        if (field.value() === "") {
            switch (field.name) {
                case 'CDNSU':
                    field.setValue('');
                    break;
                case 'CDLOTEVENDA':
                    field.setValue('');
                    break;
                default:
                    field.setValue('null');
            }
        }
    };

    this.setFieldValueIn = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NMADQUIRENTE':
                    field.setValue([]);
                    break;
                case 'NMPRODUTOADQ':
                    field.setValue([]);
                    break;
                case 'LABELESTABELEC':
                    field.setValue([]);
                    break;
                default:
                    field.setValue('');
            }
        }
    };

    this.calculateFooterValues = function (widget) {
        var mainFooter = widget.widgets[1];
        if (widget.dataSource.data) {
            var dates = widget.dataSource.data;
            var TVRBRUTVENDASUM = null;
            var TVRLIQVENDASUM = null;
            var TVRLIQPGTOSUM = null;
            if (dates.length != 0) {
                dates.forEach(function (row, index) {
                    TVRBRUTVENDASUM += parseFloat(row.VRBRUTVENDA);
                    TVRLIQVENDASUM += parseFloat(row.VRLIQVENDA);
                    TVRLIQPGTOSUM += parseFloat(row.VRLIQPGTO);
                });
            } else {
                TVRBRUTVENDASUM = 0;
                TVRLIQVENDASUM = 0;
                TVRLIQPGTOSUM = 0;
            }
            mainFooter.getField('TVRBRUTVENDA').value("R$" + TVRBRUTVENDASUM.formatMoney(2, ",", "."));

            mainFooter.getField('TVRLIQVENDA').value("R$" + TVRLIQVENDASUM.formatMoney(2, ",", "."));

            mainFooter.getField('TVRLIQPGTO').value("R$" + TVRLIQPGTOSUM.formatMoney(2, ",", "."));
        }
    };

    this.openSwipeAndPopulateTreeGrid = function (args) {
        var widgetGrid = this.getWidget(args);
        var widgetTreeGrid = templateManager.container.getWidget('detalhamentoLoteTreeGrid');
        var row = widgetGrid.currentRow;
        var dataSourceFilter = [{
            "name": "NRESTABELEC",
            "operator": "=",
            "value": row.NRESTABELEC
        }, {
            "name": "NRADQUIRENTE",
            "operator": "IN",
            "value": row.NRADQUIRENTE
        }, {
            "name": "NRPRODUTOADQ",
            "operator": "IN",
            "value": row.NRPRODUTOADQ
        }, {
            "name": "NRLOTEVENDA",
            "operator": "=",
            "value": row.NRLOTEVENDA
        }, {
            "name": "CDNSU",
            "operator": "=",
            "value": row.CDNSU
        }, {
            "name": "IDSITUACAO",
            "operator": "IN",
            "value": row.STATUSSIT
        }, {
            "name": "IDCONCILIADO",
            "operator": "IN",
            "value": row.STATUSCON
        }, {
            "name": "IDFINRECEBIDO",
            "operator": "IN",
            "value": row.FINRECEBIDO
        }];

        widgetTreeGrid.dataSourceFilter = dataSourceFilter;
        ScreenService.openSwipe(widgetTreeGrid);

        restEngine.post("cnc#/cnc01000_vendasconciliadas_tree", {
            "requestType": "FilterData",
            "filter": dataSourceFilter
        }).then(function (response) {
            widgetTreeGrid.dataSource.data = response.dataset.dataSet;
            widgetTreeGrid.dataSource.data.forEach(function (row, index) {
                if (row.PARENT != null) {
                    switch (row.STATUSSIT) {
                        case 'C':
                            row.STATUSSIT = 'Correto';
                            break;
                        case 'O':
                            row.STATUSSIT = 'Divergente';
                            break;
                        default:
                            row.STATUSSIT = 'Não Conciliado';
                    }
                    switch (row.STATUSCON) {
                        case 'C':
                            row.STATUSCON = 'Conciliado';
                            break;
                        case 'I':
                            row.STATUSCON = 'Incompleto';
                            break;
                        default:
                            row.STATUSCON = 'Pendente';
                    }
                }
            });
            populateFooterStripeFirstField(widgetGrid, widgetTreeGrid);
        });
    };

    this.openSwipeAndPopulateView = function (args) {
        var widgetTreeGrid = this.getWidget(args);
        var widgetView = templateManager.container.getWidget('detalhamentoParcelaVenda');
        var widgetViewDivConc = templateManager.container.getWidget('divergenciaconciliacao');
        var widgetViewDivPgto = templateManager.container.getWidget('divergenciapagamento');
        var widgetViewCorPgto = templateManager.container.getWidget('correcaoconciliacao');
        var widgetTab = templateManager.container.getWidget('tabbed');
        var NRRETVENDAADQ = templateManager.container.getWidget('detalhamentoLoteTreeGrid').currentRow.NRRETVENDAADQ;

        if (widgetTreeGrid.currentRow.CDNSU) {
            widgetViewDivConc.dataSourceFilter = [{
                "name": "NRRETVENDAADQ",
                "operator": "=",
                "value": NRRETVENDAADQ
            }];
            widgetViewDivPgto.dataSourceFilter = [{
                "name": "NRRETVENDAADQ",
                "operator": "=",
                "value": NRRETVENDAADQ
            }];
            widgetViewCorPgto.dataSourceFilter = [{
                "name": "NRRETVENDAADQ",
                "operator": "=",
                "value": NRRETVENDAADQ
            }];
            widgetViewDivConc.dataSource.load(widgetViewDivConc.dataSourceFilter).then(function (dadoConc) {
                widgetViewDivPgto.dataSource.load(widgetViewDivPgto.dataSourceFilter).then(function (dadoPgto) {
                    widgetViewCorPgto.dataSource.load(widgetViewCorPgto.dataSourceFilter).then(function (dadoCor) {
                        widgetViewDivConc.isVisible = true;
                        widgetViewDivPgto.isVisible = true;
                        widgetViewCorPgto.isVisible = true;
                        if (dadoConc.dataset.Cnc01000_DivergenciaConciliacao.length == 0) {
                            widgetViewDivConc.isVisible = false;
                        }
                        if (dadoPgto.dataset.Cnc01000_DivergenciaPagamento.length == 0) {
                            widgetViewDivPgto.isVisible = false;
                        }
                        if (dadoCor.dataset.Cnc01000_CorrecaoPagamento.length == 0) {
                            widgetViewCorPgto.isVisible = false;
                        }
                        ScreenService.openSwipe(widgetTab);
                        widgetView.getField('DTVENDA').value(widgetTreeGrid.currentRow.DTVENDA);
                        widgetView.getField('DTPREV').value(widgetTreeGrid.currentRow.DTCREDITOPREV);
                        widgetView.getField('DTPGTO').value(widgetTreeGrid.currentRow.DTCREDITO);
                        widgetView.getField('IDSTATUSVENDA').value(widgetTreeGrid.currentRow.IDSTATUSVENDA === 'A' ? 'Sim' : 'Não');
                        widgetView.getField('VRBRUTVENDA').value(widgetTreeGrid.currentRow.VRBRUTVENDA);
                        widgetView.getField('VRLIQVENDA').value(widgetTreeGrid.currentRow.VRLIQVENDA);
                        widgetView.getField('VRLIQPGTO').value(widgetTreeGrid.currentRow.VRLIQPGTO);
                        widgetView.getField('CDNSU').value(widgetTreeGrid.currentRow.CDNSU);
                        widgetView.getField('NMPRODUTO').value(widgetTreeGrid.currentRow.NMPRODUTOADQ);
                        widgetView.getField('VRTAXAPREV').value(widgetTreeGrid.currentRow.VRTAXAPREV);
                        widgetView.getField('VRCOBRADOTAXA').value(widgetTreeGrid.currentRow.VRCOBRADOTAXA);
                        widgetView.getField('VRTAXAADMPREV').value(widgetTreeGrid.currentRow.VRTAXAADMPREV);
                        widgetView.getField('VRTAXAADMPGTO').value(widgetTreeGrid.currentRow.VRTAXAADMPGTO);
                        widgetView.getField('VRTAXAANTVENDA').value(widgetTreeGrid.currentRow.VRTAXAANTVENDA);
                        widgetView.getField('NRAUTORIZACAO').value(widgetTreeGrid.currentRow.NRAUTORIZACAO);
                        widgetView.getField('DSAJUSTE').value(widgetTreeGrid.currentRow.DSAJUSTE);
                        var taxaParaRealParam = widgetTreeGrid.currentRow.VRTAXAPREV;
                        taxaParaRealParam = taxaParaRealParam * widgetTreeGrid.currentRow.VRBRUTVENDA / 100;
                        widgetView.getField('VRTAXAPARAM').value(taxaParaRealParam);
                        var taxaParaRealPrev = widgetTreeGrid.currentRow.VRTAXAADMPREV;
                        taxaParaRealPrev = taxaParaRealPrev * widgetTreeGrid.currentRow.VRBRUTVENDA / 100;
                        widgetView.getField('VRTAXAPREVR').value(taxaParaRealPrev);
                        widgetTreeGrid.currentRow.IDSTATUSVENDA === 'C' ? widgetView.getField('DSAJUSTE').isVisible = true : widgetView.getField('DSAJUSTE').isVisible = false;
                        widgetView.activate();
                    });
                });
            });
        } else if (widgetView.active) {
            ScreenService.closeSwipe();
        }
    };

    this.getWidget = function (args) {
        return args.owner;
    };

    this.getDivergenceConciliator = function (swipe) {
        var widgetViewDivConc = templateManager.container.getWidget('divergenciaconciliacao');
        var widgetGrid = templateManager.container.getWidget('detalhamentoLoteTreeGrid');

        var NRRETVENDAADQ = widgetGrid.currentRow.NRRETVENDAADQ;
        widgetViewDivConc.dataSourceFilter = [{
            "name": "NRRETVENDAADQ",
            "operator": "=",
            "value": NRRETVENDAADQ
        }];
        widgetViewDivConc.dataSource.load(widgetViewDivConc.dataSourceFilter).then(function (dado) {
            swipe.dataSource.data = dado.dataset.Cnc01000_DivergenciaConciliacao;
        });
    };

    this.getPaymentConciliator = function (swipe) {
        var widgetGrid = templateManager.container.getWidget('detalhamentoLoteTreeGrid');
        var widgetViewDivPgto = templateManager.container.getWidget('divergenciapagamento');

        var NRRETVENDAADQ = widgetGrid.currentRow.NRRETVENDAADQ;
        widgetViewDivPgto.dataSourceFilter = [{
            "name": "NRRETVENDAADQ",
            "operator": "=",
            "value": NRRETVENDAADQ
        }];
        widgetViewDivPgto.dataSource.load(widgetViewDivPgto.dataSourceFilter).then(function (dado) {
            swipe.dataSource.data = dado.dataset.Cnc01000_DivergenciaPagamento;
        });
    };

    this.getAdjustmentConciliator = function (swipe) {
        var widgetGrid = templateManager.container.getWidget('detalhamentoLoteTreeGrid');
        var widgetViewCorPgto = templateManager.container.getWidget('correcaoconciliacao');

        var NRRETVENDAADQ = widgetGrid.currentRow.NRRETVENDAADQ;
        widgetViewCorPgto.dataSourceFilter = [{
            "name": "NRRETVENDAADQ",
            "operator": "=",
            "value": NRRETVENDAADQ
        }];
        widgetViewCorPgto.dataSource.load(widgetViewCorPgto.dataSourceFilter).then(function (dado) {
            swipe.dataSource.data = dado.dataset.Cnc01000_CorrecaoPagamento;
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01000_VendasConciliadas_Controller', Cnc01000_VendasConciliadas_Controller);
});

// FILE: ./assets/js/controllers/Cnc01100_SimulacaoAntecipada_Controller.js
function Cnc01100_SimulacaoAntecipada_Controller(templateManager, Cnc01100_SimulacaoAntecipada_Service, ScreenService, Filter, UtilitariosTela, restEngine) {

    var keyPaths = [];

    var saveKeyPath = function saveKeyPath(path) {
        keyPaths.push({
            sign: path[0] === '+' || path[0] === '-' ? parseInt(path.shift() + 1) : 1,
            path: path
        });
    };

    var valueOf = function valueOf(object, path) {
        var ptr = object;
        for (var i = 0, l = path.length; i < l; i++) {
            ptr = ptr[path[i]];
        }return ptr;
    };
    var comparer = function comparer(a, b) {
        var aVal = null;
        var bVal = null;
        for (var i = 0, l = keyPaths.length; i < l; i++) {
            aVal = valueOf(a, keyPaths[i].path);
            bVal = valueOf(b, keyPaths[i].path);
            if (aVal > bVal) return keyPaths[i].sign;
            if (aVal < bVal) return -keyPaths[i].sign;
        }
        return 0;
    };

    this.estabelec = function (args) {
        var field = args.owner.owner;
        var nrField = field.widget.getField('NRESTABELEC');

        nrField.value(field.dataSource.data.filter(function (data) {
            return data.NMESTABELEC === field.value();
        }).shift().NRESTABELEC);
        field.widget.currentRow.NRESTABELEC = nrField.value();
    };

    this.backActivate = function () {
        var valuesResult = templateManager.container.getWidget('resultadoSimulacaoSwipe');
        var valuesWidget = templateManager.container.getWidget('valoresSimulacao');

        valuesResult.dataSource.data = [];
        ScreenService.closeSwipe();
        valuesWidget.activate();
    };

    this.calcularAntecipacao = function (row) {
        var widget = templateManager.container.getWidget('taxaAdmTipoPopUp');
        var valuesWidget = templateManager.container.getWidget('valoresSimulacao');
        var valuesResult = templateManager.container.getWidget('resultadoSimulacaoSwipe');
        var footerResult = templateManager.container.getWidget('stripe-footer');
        var checkedRows = valuesWidget.getCheckedRows();
        var VRTAXAANT = parseFloat(row.vrTaxaAnt);
        var h = new Date();
        var hojeFormat = ("0" + h.getDate()).slice(-2) + "/" + ("0" + (h.getMonth() + 1)).slice(-2) + "/" + h.getFullYear();
        var hojecmp = new Date(hojeFormat.slice(3, 5) + '/' + hojeFormat.slice(0, 2) + '/' + hojeFormat.slice(6, 10));
        var dataIndex = 0;
        var arr = {};
        var TXADM = {};
        var VRBRUTVENDATOT = null;
        var VRLIQTVENDATOT = null;
        var VRTOTADM = null;
        var VRTOTANTAPL = null;
        if (widget.isValid()) {
            if (VRTAXAANT <= 0) {
                ScreenService.showMessage('Valor da taxa deve ser maior que 0%.', 'ERROR');
            } else if (VRTAXAANT >= 100) {
                ScreenService.showMessage('Valor da taxa deve ser menor que 100%', 'ERROR');
            } else {
                if (row.consTaxaAdm == "A") {
                    checkedRows.forEach(function (row, index) {
                        if (!(checkedRows[index].DTCREDITO == null)) {
                            var CDFILIAL = checkedRows[index].NRESTABELEC;
                            var NRSEQADQ = checkedRows[index].NRADQUIRENTE;
                            var NRSEQPRODADQ = checkedRows[index].NRPRODUTOADQ;
                            var keyPr = NRSEQADQ + '-' + NRSEQPRODADQ;

                            if (arr[keyPr]) {
                                arr[keyPr].then(function () {
                                    var dtcredito = checkedRows[index].DTCREDITO;
                                    var dtcreditocmp = new Date(dtcredito.slice(3, 5) + '/' + dtcredito.slice(0, 2) + '/' + dtcredito.slice(6, 10));
                                    var dtvenda = checkedRows[index].DTVENDA;
                                    var dtvendacmp = new Date(dtvenda.slice(3, 5) + '/' + dtvenda.slice(0, 2) + '/' + dtvenda.slice(6, 10));
                                    var dtordenation = dtvendacmp.getTime();
                                    var timeDiff = Math.abs(hojecmp.getTime() - dtcreditocmp.getTime());
                                    var diffDays = Math.ceil(timeDiff / (1000 * 3600 * 24));
                                    var VRPORCTAXADIARIA = VRTAXAANT / 30;
                                    var VRPORCTAXATOTAL = VRPORCTAXADIARIA * diffDays / 100;
                                    checkedRows[index].VRTAXAADM = TXADM[keyPr];
                                    checkedRows[index].QTDIASANT = diffDays;
                                    //checkedRows[index].VRTAXAANTDEF = (VRTAXAANT/30)*diffDays;

                                    var VRLIQVENDA = checkedRows[index].VRBRUTVENDA - checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;

                                    //Calcula o valor Líquido, sem considerar o valor da taxa de antecipação aplicada
                                    checkedRows[index].VRLIQNOVOVENDA = checkedRows[index].VRBRUTVENDA - checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;

                                    //Atualiza o valor líquido, considerando o valor de Antecipação
                                    checkedRows[index].VRLIQNOVOVENDA = checkedRows[index].VRLIQNOVOVENDA - checkedRows[index].VRLIQNOVOVENDA * VRPORCTAXATOTAL;
                                    checkedRows[index].DTORDENATION = dtordenation;
                                    valuesResult.dataSource.data[dataIndex] = checkedRows[index];
                                    checkedRows[index].VRTAXAANTDEF = VRPORCTAXATOTAL * 100;
                                    VRLIQVENDA = checkedRows[index].VRBRUTVENDA - checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;
                                    VRBRUTVENDATOT += checkedRows[index].VRBRUTVENDA;
                                    VRLIQTVENDATOT += checkedRows[index].VRLIQNOVOVENDA;
                                    VRTOTADM += checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;
                                    VRTOTANTAPL += VRLIQVENDA * VRPORCTAXATOTAL;
                                    dataIndex++;
                                    footerResult.getField('VRBRUTVENDATOT').value("R$" + VRBRUTVENDATOT.formatMoney(2));
                                    footerResult.getField('VRLIQTVENDATOT').value("R$" + VRLIQTVENDATOT.formatMoney(2));
                                    footerResult.getField('VRTOTADM').value("R$" + VRTOTADM.formatMoney(2));
                                    footerResult.getField('VRTOTANTAPL').value("R$" + VRTOTANTAPL.formatMoney(2));
                                }).finally(function () {
                                    valuesResult.dataSource.data.sortBy('DTORDENATION', 'VRBRUTVENDA', 'CDNSU');
                                });
                            } else {
                                arr[keyPr] = restEngine.post('cnc#/Cnc01100_ValoresSimProcedureRepo', {
                                    "requestType": "FilterData",
                                    "filter": [{
                                        "name": "CDFILIAL",
                                        "operator": "=",
                                        "value": CDFILIAL
                                    }, {
                                        "name": "NRADQUIRENTE",
                                        "operator": "=",
                                        "value": NRSEQADQ
                                    }, {
                                        "name": "NRSEQPRODADQ",
                                        "operator": "=",
                                        "value": NRSEQPRODADQ
                                    }]
                                });
                                arr[keyPr].then(function (dadosProcedure) {
                                    var dtcredito = checkedRows[index].DTCREDITO;
                                    var dtcreditocmp = new Date(dtcredito.slice(3, 5) + '/' + dtcredito.slice(0, 2) + '/' + dtcredito.slice(6, 10));
                                    var dtvenda = checkedRows[index].DTVENDA;
                                    var dtvendacmp = new Date(dtvenda.slice(3, 5) + '/' + dtvenda.slice(0, 2) + '/' + dtvenda.slice(6, 10));
                                    var dtordenation = dtvendacmp.getTime();
                                    var timeDiff = Math.abs(hojecmp.getTime() - dtcreditocmp.getTime());
                                    var diffDays = Math.ceil(timeDiff / (1000 * 3600 * 24));
                                    var VRPORCTAXADIARIA = VRTAXAANT / 30;
                                    var VRPORCTAXATOTAL = VRPORCTAXADIARIA * diffDays / 100;

                                    checkedRows[index].VRTAXAADM = parseFloat(dadosProcedure.dataset.Cnc01100_ValoresSimProcedureRepo.P_TAXA);
                                    checkedRows[index].QTDIASANT = diffDays;

                                    //checkedRows[index].VRTAXAANTDEF = (VRTAXAANT/30)*diffDays;

                                    var VRLIQVENDA = checkedRows[index].VRBRUTVENDA - checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;

                                    //Calcula o valor Líquido, sem considerar o valor da taxa de antecipação aplicada
                                    checkedRows[index].VRLIQNOVOVENDA = checkedRows[index].VRBRUTVENDA - checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;

                                    //Atualiza o valor líquido, considerando o valor de Antecipação
                                    checkedRows[index].VRLIQNOVOVENDA = checkedRows[index].VRLIQNOVOVENDA - checkedRows[index].VRLIQNOVOVENDA * VRPORCTAXATOTAL;

                                    checkedRows[index].DTORDENATION = dtordenation;
                                    valuesResult.dataSource.data[dataIndex] = checkedRows[index];
                                    checkedRows[index].VRTAXAANTDEF = VRPORCTAXATOTAL * 100;
                                    TXADM[keyPr] = dadosProcedure.dataset.Cnc01100_ValoresSimProcedureRepo.P_TAXA;
                                    VRBRUTVENDATOT += checkedRows[index].VRBRUTVENDA;
                                    VRLIQTVENDATOT += checkedRows[index].VRLIQNOVOVENDA;
                                    VRTOTADM += checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;
                                    VRTOTANTAPL += VRLIQVENDA * VRPORCTAXATOTAL;
                                    dataIndex++;
                                    footerResult.getField('VRBRUTVENDATOT').value("R$" + VRBRUTVENDATOT.formatMoney(2));
                                    footerResult.getField('VRLIQTVENDATOT').value("R$" + VRLIQTVENDATOT.formatMoney(2));
                                    footerResult.getField('VRTOTADM').value("R$" + VRTOTADM.formatMoney(2));
                                    footerResult.getField('VRTOTANTAPL').value("R$" + VRTOTANTAPL.formatMoney(2));
                                }).finally(function () {
                                    valuesResult.dataSource.data.sortBy('DTORDENATION', 'VRBRUTVENDA', 'CDNSU');
                                });
                            }
                        }
                    });
                    footerResult.getField('TAXAANTESC').value(VRTAXAANT.formatMoney(2) + "%");
                    ScreenService.closePopup();
                    ScreenService.openSwipe(valuesResult);
                    valuesResult.activate();
                    ScreenService.restoreDefaultLoadingMessage();
                } else {
                    checkedRows.forEach(function (row, index) {
                        if (!(checkedRows[index].DTCREDITO == null)) {
                            var dtcredito = checkedRows[index].DTCREDITO;
                            var dtcreditocmp = new Date(dtcredito.slice(3, 5) + '/' + dtcredito.slice(0, 2) + '/' + dtcredito.slice(6, 10));
                            var dtvenda = checkedRows[index].DTVENDA;
                            var dtvendacmp = new Date(dtvenda.slice(3, 5) + '/' + dtvenda.slice(0, 2) + '/' + dtvenda.slice(6, 10));
                            var dtordenation = dtvendacmp.getTime();
                            var timeDiff = Math.abs(hojecmp.getTime() - dtcreditocmp.getTime());
                            var diffDays = Math.ceil(timeDiff / (1000 * 3600 * 24));
                            var VRPORCTAXADIARIA = VRTAXAANT / 30;
                            var VRPORCTAXATOTAL = VRPORCTAXADIARIA * diffDays / 100;
                            checkedRows[index].QTDIASANT = diffDays;
                            //checkedRows[index].VRTAXAANTDEF = (VRTAXAANT/30)*diffDays;
                            var VRLIQVENDA = checkedRows[index].VRBRUTVENDA - checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;

                            //Calcula o valor Líquido, sem considerar o valor da taxa de antecipação aplicada
                            checkedRows[index].VRLIQNOVOVENDA = checkedRows[index].VRBRUTVENDA - checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;

                            //Atualiza o valor líquido, considerando o valor de Antecipação
                            checkedRows[index].VRLIQNOVOVENDA = checkedRows[index].VRLIQNOVOVENDA - checkedRows[index].VRLIQNOVOVENDA * VRPORCTAXATOTAL;

                            checkedRows[index].DTORDENATION = dtordenation;
                            checkedRows[index].CDNSU = parseInt(checkedRows[index].CDNSU);
                            valuesResult.dataSource.data[dataIndex] = checkedRows[index];
                            checkedRows[index].VRTAXAANTDEF = VRPORCTAXATOTAL * 100;
                            //VRLIQVENDA = checkedRows[index].VRBRUTVENDA - ((checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA)/100);
                            VRBRUTVENDATOT += checkedRows[index].VRBRUTVENDA;
                            VRLIQTVENDATOT += checkedRows[index].VRLIQNOVOVENDA;
                            VRTOTADM += checkedRows[index].VRTAXAADM * checkedRows[index].VRBRUTVENDA / 100;
                            VRTOTANTAPL += VRLIQVENDA * VRPORCTAXATOTAL;
                            dataIndex++;
                        }
                    });
                    valuesResult.dataSource.data.sortBy('DTORDENATION', 'VRBRUTVENDA', 'CDNSU');
                    footerResult.getField('TAXAANTESC').value(VRTAXAANT.formatMoney(2) + "%");
                    footerResult.getField('VRBRUTVENDATOT').value("R$" + VRBRUTVENDATOT.formatMoney(2));
                    footerResult.getField('VRLIQTVENDATOT').value("R$" + VRLIQTVENDATOT.formatMoney(2));
                    footerResult.getField('VRTOTADM').value("R$" + VRTOTADM.formatMoney(2));
                    footerResult.getField('VRTOTANTAPL').value("R$" + VRTOTANTAPL.formatMoney(2));
                    ScreenService.closePopup();
                    ScreenService.openSwipe(valuesResult);
                    valuesResult.activate();
                }
            }
        }
    };

    Array.prototype.sortBy = function () {
        keyPaths = [];
        for (var i = 0, l = arguments.length; i < l; i++) {
            switch (_typeof(arguments[i])) {
                case "object":
                    saveKeyPath(arguments[i]);break;
                case "string":
                    saveKeyPath(arguments[i].match(/[+-]|[^.]+/g));break;
            }
        }
        return this.sort(comparer);
    };

    this.validationButton = function () {
        var valuesWidget = templateManager.container.getWidget('valoresSimulacao');
        var anulado = false;

        if (valuesWidget.hasCheckedRows() === false) {
            ScreenService.showMessage('Pelo menos uma venda deve ser selecionada', 'ERROR');
        } else {
            var checkedRows = valuesWidget.getCheckedRows();
            var h = new Date();
            var hojeFormat = ("0" + h.getDate()).slice(-2) + "/" + ("0" + (h.getMonth() + 1)).slice(-2) + "/" + h.getFullYear();

            checkedRows.forEach(function (row, index) {
                if (!(checkedRows[index].DTCREDITO == null)) {
                    var hojecmp = new Date(hojeFormat.slice(3, 5) + '/' + hojeFormat.slice(0, 2) + '/' + hojeFormat.slice(6, 10));
                    var dtcredito = checkedRows[index].DTCREDITO;
                    var dtcreditocmp = new Date(dtcredito.slice(3, 5) + '/' + dtcredito.slice(0, 2) + '/' + dtcredito.slice(6, 10));
                    var timeDiff = hojecmp.getTime() - dtcreditocmp.getTime();
                    var diffDays = Math.ceil(timeDiff / (1000 * 3600 * 24));
                    if (diffDays >= 0) {
                        anulado = true;
                    }
                }
            });

            if (anulado == true) {
                ScreenService.showMessage('A data de Previsão de Pagamento da(s) venda(s) deve ser posterior ao dia atual.', 'ERROR');
            } else {

                if (templateManager.container.getWidget('taxaAdmTipoPopUp').getField('consTaxaAdm').value() === undefined) {
                    templateManager.container.getWidget('taxaAdmTipoPopUp').getField('consTaxaAdm').value('B');
                }
                ScreenService.openPopup(templateManager.container.getWidget('taxaAdmTipoPopUp'));
            }
        }
    };

    Number.prototype.formatMoney = function (c, d, t) {
        var n = this,
            c = isNaN(c = Math.abs(c)) ? 2 : c,
            d = d == undefined ? "," : d,
            t = t == undefined ? "." : t,
            s = n < 0 ? "-" : "",
            i = String(parseInt(n = Math.abs(Number(n) || 0).toFixed(c))),
            j = (j = i.length) > 3 ? j % 3 : 0;
        return s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
    };

    this.clearFilterPopupFields = function (widget) {
        Filter.startFiltering(widget);
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01100_SimulacaoAntecipada_Controller', Cnc01100_SimulacaoAntecipada_Controller);
});

// FILE: ./assets/js/controllers/Cnc01200_TaxaParcelaAdquirente_Controller.js
function Cnc01200_TaxaParcelaAdquirente_Controller(ScreenService, ZhUtil, templateManager, restEngine) {

    var getWidget = function getWidget(args) {
        return args.owner;
    };

    /*this.backUpFieldValue = function (args) {
        const field = args.owner.owner;
        const ownerWidget = field.widget;
         (typeof ownerWidget.currentRow.fieldChanged === 'undefined') ||
        !ownerWidget.currentRow.fieldChanged ? ownerWidget.currentRow.backUpFieldValue = field.value() : false;
    };
     this.flagFieldQtParcelaChange = function (args) {
        const field = args.owner.owner;
        const ownerWidget = field.widget;
         field.value() == ownerWidget.currentRow.backUpFieldValue ? ownerWidget.currentRow.fieldChanged = false :
            ownerWidget.currentRow.fieldChanged = true;
    };*/

    this.openSwipe = function (args) {
        var mainGridWidget = getWidget(args);
        var tabbedWidget = templateManager.container.getWidget('tabbed');
        var viewGridWidget = templateManager.container.getWidget('viewGridTaxaParcelasAdquirente');

        ScreenService.openSwipe(tabbedWidget).then(function () {
            viewGridWidget.dataSourceFilter.map(function (object) {
                object.name === 'NRADQUIRENTE' ? object.value = mainGridWidget.currentRow.NRADQUIRENTE : object;
            });

            viewGridWidget.dataSource.load(viewGridWidget.dataSourceFilter);
        });
    };

    this.newTaxaPorParcela = function (args) {
        var widget = getWidget(args).widget;
        widget.newRow();
    };

    this.editTaxaPorParcela = function (args) {
        var widget = getWidget(args).widget;
        widget.edit();
        widget.redraw();
        widget.reload();
    };

    this.cancelEditTaxaPorParcela = function (args) {
        var widget = getWidget(args).widget;

        if (widget.isNewRow()) {
            widget.currentRow.fieldChanged = null;
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
        } else {
            widget.currentRow.fieldChanged = null;
            widget.view();
        }
        widget.redraw();
        widget.reload();
    };

    this.saveTaxaPorParcela = function (args) {
        var widget = getWidget(args).widget;
        var widgetAdquirenteGrid = templateManager.container.getWidget('gridAdquirentes');
        widget.currentRow.NRADQUIRENTE = widgetAdquirenteGrid.currentRow.NRADQUIRENTE;

        if (widget.isValid()) {
            if (widget.currentRow.__is_new) {

                restEngine.post('cnc#/cnc01200_taxa_parcela_view_grid/saveNewRow', {
                    "requestType": "DataSet",
                    "dataset": [widget.currentRow]
                }).then(function (response) {

                    // Opaopa, é uma pequena gambiarra passando pela seu script!!
                    widget.currentRow.NRTAXAADQPARC = response.dataset.dataSet[0];

                    if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;

                    ScreenService.successNotification(response.notifications[0].message);
                    widget.currentRow.changed = false;
                    widget.view();
                }).catch(function (error) {
                    switch (error.data.errorCode) {
                        case 1:
                            ScreenService.showMessage('A taxa máxima por parcela é de 100%.', 'ERROR');
                            break;
                        case 2:
                            ScreenService.showMessage('O valor da taxa por parcela deve ser maior que 0.', 'ERROR');
                            break;
                        case 3:
                            ScreenService.showMessage('O máximo de parcelas permitidas são 999.', 'ERROR');
                            break;
                        case 4:
                            ScreenService.showMessage('O número de parcelas deve ser maior que 0.', 'ERROR');
                            break;
                        case 5:
                            ScreenService.showMessage('Já existe uma taxa cadastrada para essa quantidade de parcelas', 'ERROR');
                            break;
                        default:
                            ScreenService.showMessage(error.data.error, 'ERROR');
                    }
                });
            } else {
                typeof widget.currentRow.changed === 'undefined' ? widget.currentRow.changed = false : true;

                widget.saveRow().then(function () {

                    if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;

                    ScreenService.successNotification('Dados salvos com sucesso!');
                    widget.currentRow.changed = false;
                    widget.view();
                }).catch(function (error) {
                    switch (error.data.errorCode) {
                        case 1:
                            ScreenService.showMessage('A taxa máxima por parcela é de 100%.', 'ERROR');
                            break;
                        case 2:
                            ScreenService.showMessage('O valor da taxa por parcela deve ser maior que 0.', 'ERROR');
                            break;
                        case 3:
                            ScreenService.showMessage('O máximo de parcelas permitidas são 999.', 'ERROR');
                            break;
                        case 4:
                            ScreenService.showMessage('O número de parcelas deve ser maior que 0.', 'ERROR');
                            break;
                        case 5:
                            ScreenService.showMessage('Já existe uma taxa cadastrada para essa quantidade de parcelas', 'ERROR');
                            break;
                        default:
                            ScreenService.showMessage(error.data.error, 'ERROR');
                    }
                });
            }
        }
    };

    this.deleteTaxaPorParcela = function (args) {
        var widget = getWidget(args).widget;

        ScreenService.confirmMessage('Deseja deletar este resgistro?', 'question', function () {
            widget.dataSource.remove([widget.currentRow]).then(function () {
                ScreenService.closeSwipe();
                widget.reload();
            }).catch(function (response) {
                ScreenService.showMessage(response.data.error, 'ERROR');
                widget.active();
                widget.reload();
            });
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01200_TaxaParcelaAdquirente_Controller', Cnc01200_TaxaParcelaAdquirente_Controller);
});

// FILE: ./assets/js/controllers/Cnc01300_MovimentacaoFinanceira_Controller.js
function Cnc01300_MovimentacaoFinanceira_Controller(templateManager, ScreenService, Filter, maskEngine, ZhUtil, restEngine, ValidationEngine) {

    var self = this;

    this.getWidget = function (args) {
        return args.owner;
    };

    this.openFilter = function (widget, fieldName) {
        widget.getField(fieldName).value({
            'start': moment().format('DD/MM/YYYY'),
            'period': undefined,
            'end': moment().format('DD/MM/YYYY')
        });
    };

    this.setFieldValue = function (args, fieldName) {
        var widget = args.owner.owner.widget;
        var field = args.owner.owner;

        field.value(widget.currentRow[fieldName]);
    };

    this.enableAndFilterAgen = function (args) {
        var bancoField = args.owner.field;
        var agenciaField = bancoField.widget.getField('CDAGENCIA');
        var contaField = bancoField.widget.getField('CDCONTCORR');

        // bancoField.widget is the Filter Widget
        // If you get the Widget by the templateManager, it will the main widget
        // They are different, so that's why we don't use widget.currentRow. 

        var CDBANCO = bancoField.widget.currentRow.CDBANCO;

        agenciaField.readOnly = true;
        agenciaField.value('');
        contaField.readOnly = true;
        contaField.value('');

        if (CDBANCO) {
            agenciaField.readOnly = false;
            restEngine.post('cnc#/cnc01300_movimentacao_financeira_agencia', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': CDBANCO
                }]
            }).then(function (response) {
                self.getAgenContMask(CDBANCO).then(function (maskObject) {
                    // agenciaField.fields[0].mask.params.mask = maskObject.CDPICTAGEN;
                    agenciaField.dataSource.data = response.dataset.dataSet;
                });
            });
        }
    };

    this.enableAndFilterCont = function (args) {
        var agenciaField = args.owner.field;
        var contaField = agenciaField.widget.getField('CDCONTCORR');

        // agenciaField.widget is the Filter Widget
        // If you get the Widget by the templateManager, it will the main widget
        // They are different, so that's why we don't use widget.currentRow. 

        var CDBANCO = agenciaField.widget.currentRow.CDBANCO;
        var CDAGENCIA = agenciaField.widget.currentRow.CDAGENCIA;

        contaField.readOnly = true;
        contaField.value('');

        if (CDBANCO) {
            contaField.readOnly = false;
            restEngine.post('cnc#/cnc01300_movimentacao_financeira_conta', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': CDBANCO
                }, {
                    'name': 'CDAGENCIA',
                    'operator': '=',
                    'value': CDAGENCIA
                }]
            }).then(function (response) {
                self.getAgenContMask(CDBANCO).then(function (maskObject) {
                    contaField.fields[0].mask.params.mask = maskObject.CDPICTCONT;
                    contaField.dataSource.data = response.dataset.dataSet;
                });
            });
        } else {
            contaField.readOnly = true;
            contaField.value('');
        }
    };

    this.filterDataSet = function (args) {
        var widget = this.getWidget(args);
        var cdbanco = widget.getField('NMBANCO').dataSource.data.filter(function (data) {
            return data.NMBANCO === widget.currentRow.NMBANCO;
        });
        if (widget.isValid()) {
            widget.dataSourceFilter = [{
                'name': 'NRESTABELEC',
                'operator': '=',
                'value': widget.currentRow.LABELESTABELEC
            }, {
                'name': 'DTCREDITOINI',
                'operator': '=',
                'value': widget.currentRow.DTCREDITO.start
            }, {
                'name': 'DTCREDITOFIN',
                'operator': '=',
                'value': widget.currentRow.DTCREDITO.end
            }, {
                'name': 'IDSTATUSVENDA',
                'operator': 'IN',
                'value': widget.currentRow.IDSTATUSVENDA
            }, {
                'name': 'VRMOVIMENTACAOMIN',
                'operator': '=',
                'value': widget.currentRow.VRMOVIMENTACAOMIN ? widget.currentRow.VRMOVIMENTACAOMIN : ''
            }, {
                'name': 'VRMOVIMENTACAOMAX',
                'operator': '=',
                'value': widget.currentRow.VRMOVIMENTACAOMAX ? widget.currentRow.VRMOVIMENTACAOMAX : ''
            }, {
                'name': 'CDBANCO',
                'operator': '=',
                'value': widget.currentRow.CDBANCO ? widget.currentRow.CDBANCO : ''
            }, {
                'name': 'CDAGENCIA',
                'operator': '=',
                'value': widget.currentRow.CDAGENCIA
            }, {
                'name': 'CDCONTCORR',
                'operator': '=',
                'value': widget.currentRow.CDCONTCORR
            }, {
                'name': 'NRADQUIRENTE',
                'operator': '=',
                'value': widget.currentRow.NMADQUIRENTE
            }, {
                'name': 'NRPRODUTOADQ',
                'operator': '=',
                'value': widget.currentRow.NMPRODUTOADQ
            }, {
                'name': 'CDNSU',
                'operator': '=',
                'value': widget.currentRow.CDNSU ? widget.currentRow.CDNSU : ''
            }];
        }
    };

    this.populateView = function (args) {
        var widget = this.getWidget(args);

        switch (widget.getField('IDSTATUSVENDA').value()) {
            case 'P':
                widget.getField('IDSTATUSLABEL').value('Aluguel');
                break;
            case 'C':
                widget.getField('IDSTATUSLABEL').value('Cancelada');
                break;
            case 'A':
                widget.getField('IDSTATUSLABEL').value('Antecipada');
                break;
            case 'L':
                widget.getField('IDSTATUSLABEL').value('Liquidada');
                break;
            case 'E':
                widget.getField('IDSTATUSLABEL').value('Estorno');
                break;
        }
    };

    this.setIdStatusLabelForAllRows = function (args) {
        var widget = this.getWidget(args);

        widget.dataSource.data.forEach(function (data) {
            switch (data.IDSTATUSVENDA) {
                case 'P':
                    data.IDSTATUSLABEL = 'Aluguel';
                    break;
                case 'C':
                    data.IDSTATUSLABEL = 'Cancelada';
                    break;
                case 'A':
                    data.IDSTATUSLABEL = 'Antecipada';
                    break;
                case 'L':
                    data.IDSTATUSLABEL = 'Liquidada';
                    break;
                case 'E':
                    data.IDSTATUSLABEL = 'Estorno';
                    break;
            }
        });
    };

    this.getAgenContMask = function (CDBANCO) {
        return restEngine.post('cnc#/cnc01300_movimentacao_financeira_masks', {
            'requestType': 'FilterData',
            'filter': [{
                'name': 'CDBANCO',
                'operator': '=',
                'value': CDBANCO
            }]
        }).then(function (response) {
            return response.dataset.dataSet[0];
        });
    };

    this.contCorrMaskOnChange = function (args) {
        var contaField = args.owner.field;
        var widget = contaField.widget;
        var CDBANCO = widget.currentRow.CDBANCO;
        var CDCONTCORR = contaField.value();

        if (CDCONTCORR) {
            self.getAgenContMask(CDBANCO).then(function (maskObject) {
                widget.currentRow.CONTMASK = Util.applyMask(CDCONTCORR, { type: "fix", params: { mask: maskObject.CDPICTCONT } }, maskEngine, ZhUtil);
            });
        }
    };

    this.agenciaMaskOnChange = function (args) {
        var widget = args.owner.field.widget;

        self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
            widget.currentRow.AGENCIAMASK = Util.applyMask(widget.currentRow.CDAGENCIA, { type: "fix", params: { mask: maskObject.CDPICTAGEN } }, maskEngine, ZhUtil);
        });
    };

    this.setStripeFooter = function (args) {
        var widget = templateManager.container.getWidget('stripe-footer');
        var widgetPrincipal = templateManager.container.getWidget('financMovGrid');

        var VRTOTAlBRUTO = 0;
        var VRTOTAlLIQ = 0;
        var VRTOTAlTX = 0;
        var VRBRUTOAJUSTE = 0;
        var VRLIQAJUSTE = 0;

        widgetPrincipal.dataSource.data.forEach(function (data) {
            if (data.IDSTATUSVENDA === "L" || data.IDSTATUSVENDA === "A") {
                VRTOTAlBRUTO += parseFloat(data.VRBRUTVENDA);
                VRTOTAlLIQ += parseFloat(data.VRLIQPGTO);
                VRTOTAlTX += parseFloat(data.VRCOBRADOTAXA);
            } else {
                VRBRUTOAJUSTE += parseFloat(data.VRBRUTVENDA);
                VRLIQAJUSTE += parseFloat(data.VRLIQPGTO);
            }
        });

        VRTOTAlBRUTO = "R$ " + VRTOTAlBRUTO.formatMoney(2, ",", ".");
        VRTOTAlLIQ = "R$ " + VRTOTAlLIQ.formatMoney(2, ",", ".");
        VRTOTAlTX = "R$ " + VRTOTAlTX.formatMoney(2, ",", ".");
        VRBRUTOAJUSTE = "R$ " + VRBRUTOAJUSTE.formatMoney(2, ",", ".");
        VRLIQAJUSTE = "R$ " + VRLIQAJUSTE.formatMoney(2, ",", ".");

        widget.currentRow.VRTOTAlBRUTO = VRTOTAlBRUTO;
        widget.currentRow.VRTOTAlLIQ = VRTOTAlLIQ;
        widget.currentRow.VRTOTAlTX = VRTOTAlTX;
        widget.currentRow.VRBRUTOAJUSTE = VRBRUTOAJUSTE;
        widget.currentRow.VRLIQAJUSTE = VRLIQAJUSTE;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01300_MovimentacaoFinanceira_Controller', Cnc01300_MovimentacaoFinanceira_Controller);
});

// FILE: ./assets/js/controllers/Cnc01400_ajustesCancelamentos_Controller.js
function Cnc01400_ajustesCancelamentos_Controller(templateManager, Filter, UtilitariosTela, restEngine, ScreenService, UtilitarioGeral, ZHPromise) {

    var vlChck = "";

    this.openFilter = function (widget) {
        var rows = {
            DTVENDA: {
                start: moment().format('DD/MM/YYYY'),
                period: undefined,
                end: moment().format('DD/MM/YYYY')
            },
            DTCREDITO: {
                start: moment().format('DD/MM/YYYY'),
                period: undefined,
                end: moment().format('DD/MM/YYYY')
            },
            IDCONCILIADO: 'T'
        };

        Filter.openFilterWithRows(widget, rows);
        widget.getField('LABELESTABELEC').value('Todos');
        UtilitariosTela.setIntervalToday(widget, 'DTVENDA');
        UtilitariosTela.setIntervalToday(widget, 'DTCREDITO');
        widget.getField('NMADQUIRENTE').value('Todas');
        widget.getField('NMPRODUTOADQ').value('Todos');
        widget.getField('IDCONCILIADO').value('T');
    };

    this.setFieldValueChar = function (field) {
        if (field.value() === '') {
            field.value('null');
        }
    };

    this.setFieldValueInt = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'LABELESTABELEC':
                    field.setValue('Todos');
                    break;
                case 'NMADQUIRENTE':
                    field.setValue('Todas');
                    break;
                case 'NMPRODUTOADQ':
                    field.setValue('Todos');
                    break;
                default:
                    field.setValue('null');
            }
        }
    };

    this.cnc01400setStripeValues = function () {
        var valuesWidget = templateManager.container.getWidget('cancelamentosAjustes');
        var nrestabelec = valuesWidget.currentRow.NRESTABELEC;
        var nmadquirente = valuesWidget.getField('NMADQUIRENTE').value();
        var nmprodutoadq = valuesWidget.getField('NMPRODUTOADQ').value();
        var dtvenda = valuesWidget.getField("DTVENDA").value();
        var dtcredito = valuesWidget.getField("DTCREDITO").value();
        var conciliado = valuesWidget.getField("IDCONCILIADO").value();

        if (dtvenda.start === null && dtcredito.start === null) {
            Filter.openFilterWithRows(valuesWidget);
            restEngine.post('cnc#/Cnc01400_CancelamentosAjustesRepo', {
                "requestType": "FilterData",
                "filter": [{
                    "name": "DTVENDA",
                    "operator": "BETWEEN",
                    "value": [dtvenda.start, dtvenda.end]
                }, {
                    "name": "DTCREDITO",
                    "operator": "BETWEEN",
                    "value": [dtcredito.start, dtcredito.end]
                }, {
                    "name": "NMADQUIRENTE",
                    "operator": "IN",
                    "value": nmadquirente
                }, {
                    "name": "NMPRODUTOADQ",
                    "operator": "IN",
                    "value": nmprodutoadq
                }, {
                    "name": "NRESTABELEC",
                    "operator": "IN",
                    "value": nrestabelec
                }, {
                    "name": "IDCONCILIADO",
                    "operator": "IN",
                    "value": conciliado
                }, {
                    "name": "IDVALIDADO",
                    "operator": "IN",
                    "value": ""
                }]
            }).then(function () {}).catch(function (error) {
                var error;

                if (error.data) {
                    error = error.data.error;
                } else {
                    error = error.error;
                }

                switch (error) {
                    case 'DTVENDCREDERROR':
                        ScreenService.showMessage('Pelo menos uma das datas (de venda ou de crédito) devem ser informadas.', 'ERROR');
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, 'ERROR');
                }
            });
        }
    };

    this.validateConciliateSells = function (widget) {
        var currentRow = widget.currentRow;
        var contabWidget = widget.container.getWidget('errosContabil');
        var rows = widget.getCheckedRows();
        var standarts = rows.filter(function (standart) {
            return !standart.__isSelected == false;
        });
        var message = "";

        if (standarts[0].IDCONCILIADO == "Não") {
            message = "Deseja conciliar os ajustes selecionados?";
        } else {
            message = "Deseja cancelar a conciliação dos ajustes selecionados?";
        }
        ScreenService.confirmMessage(message, 'question', function () {
            var promises = standarts.map(function (row, index) {
                currentRow.NRESTABELEC = row.NRESTABELEC;
                currentRow.NRADQUIRENTE = row.NRADQUIRENTE;
                currentRow.NMADQUIRENTE = row.NMADQUIRENTE;
                currentRow.NRPRODUTOADQ = row.NRPRODUTOADQ;
                currentRow.DTVENDA = row.DTVENDA;
                currentRow.DTCREDITO = row.DTCREDITO;
                currentRow.VRLIQPGTO = row.VRLIQPGTO;
                currentRow.CDNSU = row.CDNSU;
                currentRow.NRORG = row.NRORG;
                currentRow.CDCLASSFINA = row.CDCLASSFINA;
                currentRow.CDEMPRESA = row.CDEMPRESA;
                currentRow.CDFILIAL = row.CDFILIAL;
                currentRow.CDCLIENTE = row.CDCLIENTE;
                currentRow.CDTPCREDBANC = row.CDTPCREDBANC;
                currentRow.CDTPDEBIBANC = row.CDTPDEBIBANC;
                currentRow.NRRETFINANCADQ = row.NRRETFINANCADQ;
                currentRow.IDCONCILIADO = row.IDCONCILIADO;
                currentRow.IDSITUACAO = row.IDSITUACAO;
                currentRow.IDCTBLCRBANC = row.IDCTBLCRBANC;
                currentRow.IDCTBLDBBANC = row.IDCTBLDBBANC;
                currentRow.NRMOVBANC = row.NRMOVBANC;
                currentRow.CDREDUCONT = row.CDREDUCONT;
                currentRow.CDTPDBDEBGER = row.CDTPDBDEBGER;
                currentRow.IDSTATUSVENDA = row.IDSTATUSVENDA;
                currentRow.IDGERAFINANC = row.IDGERAFINANC;
                switch (row.IDSTATUSVENDA) {
                    case "P":
                        currentRow.DESCMOV = 'TAXAS DE ADQUIRENTE';
                        currentRow.DESCMOVBANC = 'AjusteNegativoAdquirente';
                        currentRow.IDDEBCRED = 'D';
                        break;
                    case "C":
                        currentRow.DESCMOV = 'CANCELAMENTO DE VENDA';
                        currentRow.DESCMOVBANC = 'CancVenda';
                        currentRow.IDDEBCRED = 'D';
                        break;
                    default:
                        currentRow.DESCMOV = 'AJUSTE POSITIVO DE ADQUIRENTE';
                        currentRow.DESCMOVBANC = 'AjustePositivoAdquirente';
                        currentRow.IDDEBCRED = 'C';
                }
                return widget.saveRow();
            });

            ZHPromise.all(promises).then(function (_) {
                ScreenService.showMessage("Alterado com sucesso!");
                widget.reload();
                widget.activate();
            }).catch(function (error) {
                var index = error.data.error.indexOf("/");
                var name = error.data.error.substr(0, index);
                var valorRelac = error.data.error.substr(index + 1);
                switch (name) {
                    case "ERR":
                        ScreenService.showMessage(valorRelac, 'ERROR');
                        widget.reload();
                        break;
                    case "CTB":
                        var valoresMsg = valorRelac.split("|");
                        var CDEMPRESA = valoresMsg[0];
                        var NRLANCCTBTEMP = valoresMsg[1];
                        var row = contabWidget.currentRow;
                        row.CDEMPRESA = CDEMPRESA;
                        row.NRLANCCTBTEMP = NRLANCCTBTEMP;
                        widget.reload();
                        widget.activate();
                        ScreenService.confirmMessage('Operação Bloqueada! Não foi possível realizar a conciliação devido a erros de contabilização. Deseja verificá-los?', 'question', function () {
                            contabWidget.dataSourceFilter = [{
                                "name": "CDEMPRESA",
                                "operator": "=",
                                "value": CDEMPRESA
                            }, {
                                "name": "NRLANCCTBTEMP",
                                "operator": "=",
                                "value": NRLANCCTBTEMP
                            }];
                            contabWidget.dataSource.load(contabWidget.dataSourceFilter).then(function (dado) {
                                contabWidget.dataSource.data = [];
                                contabWidget.dataSource.data = dado.dataset.Cnc01400_ErrosContabRepo;
                                ScreenService.openPopup(contabWidget);
                                widget.reload();
                                widget.activate();
                            });
                        }, function () {
                            ZHPromise.all(contabWidget.dataSource.remove([contabWidget.currentRow])).then(function (_) {
                                widget.reload();
                                widget.activate();
                            });
                        });
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, 'ERROR');
                }
            });
        }), function () {};
    };

    this.validateCheckedAdjustment = function (widget) {
        var conciliados = [];
        var standarts = widget.getCheckedRows();
        var actionLabelCon = widget.getAction('conciliar');
        var actionLabelDes = widget.getAction('desconciliar');

        if (standarts.length !== 1) {
            standarts.forEach(function (row, index) {
                conciliados.push(row.IDCONCILIADO);
            });
            var filtrado = conciliados.filter(function (elem, pos, arr) {
                return arr.indexOf(elem) == pos;
            });
            if (filtrado.length !== 1) {
                var divStandarts = standarts.filter(function (standart) {
                    return standart.IDCONCILIADO != vlChck;
                });
                var butStandarts = standarts.filter(function (standart) {
                    return standart.IDCONCILIADO == vlChck;
                });
                divStandarts[0].__isSelected = false;
                widget.dataSource.checkedRows = butStandarts;
                widget.activate();
                if (butStandarts[0].IDCONCILIADO == "Não") {
                    actionLabelCon.isVisible = true;
                    actionLabelDes.isVisible = false;
                } else {
                    actionLabelDes.isVisible = true;
                    actionLabelCon.isVisible = false;
                }
                ScreenService.showMessage('Não é possível tratar os ajustes com situações diferentes.', 'ERROR');
            }
        } else {
            vlChck = standarts[0].IDCONCILIADO;
            if (standarts[0].IDCONCILIADO == "Não") {
                actionLabelCon.isVisible = true;
                actionLabelDes.isVisible = false;
            } else {
                actionLabelDes.isVisible = true;
                actionLabelCon.isVisible = false;
            }
        }
    };

    this.validateAllChecked = function (widget) {
        var standarts = widget.getCheckedRows();
        var actionLabelCon = widget.getAction('conciliar');
        var actionLabelDes = widget.getAction('desconciliar');
        var conciliados = [];
        if (standarts.length !== 1) {
            standarts.forEach(function (row, index) {
                conciliados.push(row.IDCONCILIADO);
            });
            var filtrado = conciliados.filter(function (elem, pos, arr) {
                return arr.indexOf(elem) == pos;
            });
            if (filtrado.length !== 1) {
                widget.unSelectAll();
                ScreenService.showMessage('Não é possível tratar os ajustes com situações diferentes.', 'ERROR');
            } else {
                if (standarts[0].IDCONCILIADO == "Não") {
                    actionLabelCon.isVisible = true;
                    actionLabelDes.isVisible = false;
                } else {
                    actionLabelDes.isVisible = true;
                    actionLabelCon.isVisible = false;
                }
            }
        } else {
            if (standarts[0].IDCONCILIADO == "Não") {
                actionLabelCon.isVisible = true;
                actionLabelDes.isVisible = false;
            } else {
                actionLabelDes.isVisible = true;
                actionLabelCon.isVisible = false;
            }
        }
    };

    this.updateFooterValues = function () {
        var widget = templateManager.container.getWidget('cancelamentosAjustes');
        var footerWidget = templateManager.container.getWidget('stripe-footer');
        var valuesWidget = widget.dataSource.data;
        var valuesCancel = 0;
        var valuesAdjustments = 0;
        valuesWidget.forEach(function (row, index) {
            if (valuesWidget[index].VRLIQPGTO < 0) {
                valuesCancel += valuesWidget[index].VRLIQPGTO;
            } else if (valuesWidget[index].VRLIQPGTO > 0) {
                valuesAdjustments += valuesWidget[index].VRLIQPGTO;
            }
        });
        valuesCancel = Math.abs(valuesCancel);
        footerWidget.getField('SUMNEGVRLIQPGTO').value("R$ " + valuesCancel.formatMoney(2));
        footerWidget.getField('SUMPOSVRLIQPGTO').value("R$ " + valuesAdjustments.formatMoney(2));
    };

    Number.prototype.formatMoney = function (c, d, t) {
        var n = this,
            c = isNaN(c = Math.abs(c)) ? 2 : c,
            d = d == undefined ? "," : d,
            t = t == undefined ? "." : t,
            s = n < 0 ? "-" : "",
            i = String(parseInt(n = Math.abs(Number(n) || 0).toFixed(c))),
            j = (j = i.length) > 3 ? j % 3 : 0;
        return s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01400_ajustesCancelamentos_Controller', Cnc01400_ajustesCancelamentos_Controller);
});

// FILE: ./assets/js/controllers/Cnc01500_ajustesAdquirentes_Controller.js
function Cnc01500_ajustesAdquirentes_Controller(templateManager, Filter, restEngine, ScreenService, UtilitariosTela) {
    var _this5 = this;

    this.setFilterValues01500 = function (value) {
        var widget = templateManager.container.getWidget('ajustesAdquirentes');
        var values = widget.fields.filter(function (field) {
            return field.group;
        }).map(function (field) {
            return field.value();
        });

        widget.currentRow.intervalo === "D" ? _this5.ajusteEmIntervaloDiario(widget, values) : _this5.ajusteEmIntervaloMensal(widget, values);
    };

    this.ajusteEmIntervaloMensal = function (widget, fieldValues) {
        var _ref = [].concat(_toConsumableArray(fieldValues)),
            estabelec = _ref[0],
            cdadquirente = _ref[1],
            dtvenda = _ref[2],
            intervalo = _ref[3],
            mesini = _ref[4],
            mesfin = _ref[5],
            dtcredito = _ref[6];

        dtcredito = dtcredito == null ? "" : dtcredito;
        dtvenda = dtvenda == null ? "" : dtvenda;

        restEngine.post('cnc#/Cnc01500_ajustesAdquirentesRepo', {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRESTABELEC",
                "operator": "IN",
                "value": estabelec
            }, {
                "name": "NRADQUIRENTE",
                "operator": "IN",
                "value": cdadquirente
            }, {
                "name": "intervalo",
                "operator": "LIKE",
                "value": intervalo
            }, {
                "name": "DTVENDA",
                "operator": "BETWEEN",
                "value": [dtvenda.start, dtvenda.end]
            }, {
                "name": "MESINI",
                "operator": "LIKE",
                "value": moment('01/' + mesini, "DD/MM/YYYY")._d.toLocaleDateString("en-US")
            }, {
                "name": "MESFIN",
                "operator": "LIKE",
                "value": moment('01/' + mesfin, "DD/MM/YYYY").endOf("month")._d.toLocaleDateString("en-US")
            }]
        }).then(function (response) {
            widget.activate();
            widget.dataSource.data = response.dataset.Cnc01500_ajustesAdquirentesRepo;
            _this5.disableDetailsAction(widget);
            widget.updateChart();
        }).catch(function (error) {
            switch (error.data.error) {
                case 'MESERROR':
                    ScreenService.showMessage('Data inicial maior que data final.', 'ERROR');
                    Filter.openFilterWithRows(templateManager.container.getWidget('ajustesAdquirentes'));
                    break;
                case 'MESEXCEDEU':
                    ScreenService.showMessage('Devido ao espaço do gráfico, o intervalo deve ser menor ou igual a um ano.', 'ERROR');
                    Filter.openFilterWithRows(templateManager.container.getWidget('ajustesAdquirentes'));
                    break;
                default:
                    ScreenService.showMessage(error.data.error, 'ERROR');
            }
        });
    };

    this.ajusteEmIntervaloDiario = function (widget, fieldValues) {
        var _ref2 = [].concat(_toConsumableArray(fieldValues)),
            estabelec = _ref2[0],
            cdadquirente = _ref2[1],
            dtvenda = _ref2[2],
            intervalo = _ref2[3],
            mesini = _ref2[4],
            mesfin = _ref2[5],
            dtcredito = _ref2[6];

        dtvenda = dtvenda == null ? "" : dtvenda;
        mesini = mesini == null ? "" : mesini;
        mesfin = mesfin == null ? "" : mesfin;

        restEngine.post('cnc#/Cnc01500_ajustesAdquirentesRepo', {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRESTABELEC",
                "operator": "IN",
                "value": estabelec
            }, {
                "name": "NRADQUIRENTE",
                "operator": "IN",
                "value": cdadquirente
            }, {
                "name": "intervalo",
                "operator": "LIKE",
                "value": intervalo
            }, {
                "name": "DTVENDA",
                "operator": "BETWEEN",
                "value": [dtvenda.start, dtvenda.end]
            }, {
                "name": "DTCREDITO",
                "operator": "BETWEEN",
                "value": [moment(dtcredito.start, "DD/MM/YYYY")._d.toLocaleDateString("en-US"), moment(dtcredito.end, "DD/MM/YYYY")._d.toLocaleDateString("en-US")]
            }]
        }).then(function (response) {
            widget.activate();
            widget.dataSource.data = response.dataset.Cnc01500_ajustesAdquirentesRepo;
            _this5.enableDetailsAction(widget);
            widget.updateChart();
        }).catch(function (error) {
            switch (error.data.error) {
                case 'DIAERROR':
                    ScreenService.showMessage('Devido ao espaço do gráfico, o intervalo de data deve ser menor ou igual a 15 dias.', 'ERROR');
                    Filter.openFilterWithRows(templateManager.container.getWidget('ajustesAdquirentes'));
                    break;
                default:
                    ScreenService.showMessage(error.data.error, 'ERROR');
            }
        });
    };

    this.disableDetailsAction = function (widget) {
        return widget.actions.filter(function (action) {
            return action.name === "details";
        }).shift().isVisible = false;
    };

    this.enableDetailsAction = function (widget) {
        return widget.actions.filter(function (action) {
            return action.name === "details";
        }).shift().isVisible = true;
    };

    this.toggleHabDate = function (widget) {
        var dtCredito = widget.getField("DTCREDITO");
        var mesIni = widget.getField("MESINI");
        var mesFin = widget.getField("MESFIN");

        if (widget.currentRow.intervalo === 'M') {
            mesIni.isVisible = true;
            mesIni.validations.required = true;
            mesIni.filterProperties.validations.required = true;
            mesFin.isVisible = true;
            mesFin.filterProperties.validations.required = true;
            mesFin.validations.required = true;
            dtCredito.isVisible = false;
            dtCredito.value('');
        } else {
            dtCredito.isVisible = true;
            mesIni.isVisible = false;
            mesIni.validations.required = false;
            mesIni.filterProperties.validations.required = false;
            mesIni.value('');
            mesFin.isVisible = false;
            mesFin.validations.required = false;
            mesFin.filterProperties.validations.required = false;
            mesFin.value('');
        }
    };

    this.setFieldValueInt01500 = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NRADQUIRENTE':
                    field.setValue('Todas');
                    break;
                case 'NRESTABELEC':
                    field.setValue('Todas');
                    break;
                default:
                    field.setValue('null');
            }
        }
    };

    this.setFieldValueChar = function (field) {
        if (field.value() === "") {
            field.setValue('Todas');
        }
    };

    this.clearFilterPopupFields01500 = function (widget) {
        Filter.openFilterWithRows(widget);
        widget.getField('NRESTABELEC').value('Todas');
        UtilitariosTela.setIntervalToday(widget, 'DTVENDA');
        widget.getField('NRADQUIRENTE').value('Todas');
        widget.getField('MESINI').value('');
        widget.getField('MESFIN').value('');
        widget.dataSource.data = [];
    };

    this.formatDailyPeriod = function (interval) {
        return [moment(interval.start, "DD/MM/YYYY")._d.toLocaleDateString("en-US"), moment(interval.end, "DD/MM/YYYY")._d.toLocaleDateString("en-US")];
    };

    this.chartDetails = function (widget) {
        var filterParams = widget.currentRow;
        var detailsWidget = widget.widgets.filter(function (widget) {
            return widget.name === "detailsWidget";
        }).shift();

        filterParams.DTCREDITO = _this5.formatDailyPeriod(filterParams.DTCREDITO);

        detailsWidget.dataSourceFilter.map(function (param) {
            return filterParams[param.name] ? param.value = filterParams[param.name] : false;
        });

        ScreenService.openSwipe(detailsWidget);
        detailsWidget.dataSource.load(detailsWidget.dataSourceFilter);
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01500_ajustesAdquirentes_Controller', Cnc01500_ajustesAdquirentes_Controller);
});

// FILE: ./assets/js/controllers/Cnc01600_cadastroEstabelecimento_Controller.js
function Cnc01600_cadastroEstabelecimento_Controller(ScreenService, templateManager, Filter, maskEngine, ZhUtil, restEngine) {
    var _this6 = this;

    var unidadeAnterior;
    var empresaAnterior;
    var inscEstadualAnt;
    var inscMunicipalAnt;

    this.getWidget = function (args) {
        return args.owner;
    };

    this.editEstabelec = function (args) {
        var widget = this.getWidget(args).widget;
        unidadeAnterior = Util.clone(widget.currentRow.CDFILIAL);
        empresaAnterior = Util.clone(widget.currentRow.CDEMPRESA);
        inscEstadualAnt = Util.clone(widget.currentRow.CDINSCESTA);
        inscMunicipalAnt = Util.clone(widget.currentRow.CDINSCMUNI);

        widget.edit();
        widget.backUpRow = Util.clone(widget.currentRow);
    };

    this.cancelEditEstabelec = function (args) {
        var widget = this.getWidget(args).widget;
        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.view();
        }
    };

    this.saveEstabelec = function (args) {
        var widget = this.getWidget(args).widget;
        var currentRow = widget.currentRow;
        var unidadeNova = Util.clone(widget.currentRow.CDFILIAL);

        var dataSourceFilter = [{
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }, {
            "name": "CDFILIAL",
            "operator": "=",
            "value": currentRow.CDFILIAL
        }, {
            "name": "NRESTABELEC",
            "operator": "=",
            "value": currentRow.NRESTABELEC
        }];

        if (widget.isValid()) {
            if (unidadeAnterior != unidadeNova) {
                restEngine.post('cnc#/cnc01600_estabelec_filterFilialParams', {
                    "requestType": "FilterData",
                    "filter": dataSourceFilter
                }).then(function (response) {
                    if ((response.dataset.dataSet[0].CDINSCMUNI != inscMunicipalAnt || response.dataset.dataSet[0].CDINSCESTA != inscEstadualAnt) && response.dataset.dataSet[0].CDEMPRESA != empresaAnterior) {
                        ScreenService.confirmMessage('Ao realizar esta operação as parametrizações de Conta Corrente e Série para este estabelecimento serão apagadas. Deseja continuar?', 'question', function () {
                            widget.saveRow().then(function (response) {
                                if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;

                                widget.currentRow.changed = false;
                                widget.view();
                                ScreenService.closeSwipe();
                                widget.reload();
                            }).catch(function (e) {
                                ScreenService.showMessage(e.data.error, "ERROR");
                            });
                            ScreenService.showMessage("Salvo com sucesso!");
                            ScreenService.closePopup();
                        }), function () {};
                    } else if (response.dataset.dataSet[0].CDEMPRESA != empresaAnterior) {
                        ScreenService.confirmMessage('Ao realizar esta operação as parametrizações de Conta Corrente para este estabelecimento serão apagadas. Deseja continuar?', 'question', function () {
                            widget.saveRow().then(function (response) {
                                if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;

                                widget.currentRow.changed = false;
                                widget.view();
                                ScreenService.closeSwipe();
                                widget.reload();
                            }).catch(function (e) {
                                ScreenService.showMessage(e.data.error, "ERROR");
                            });
                            ScreenService.showMessage("Salvo com sucesso!");
                            ScreenService.closePopup();
                        }), function () {};
                    } else if (response.dataset.dataSet[0].CDINSCMUNI != inscMunicipalAnt || response.dataset.dataSet[0].CDINSCESTA != inscEstadualAnt) {
                        ScreenService.confirmMessage('Ao realizar esta operação a parametrização de Série para este estabelecimento será apagada apagadas. Deseja continuar?', 'question', function () {
                            widget.saveRow().then(function (response) {
                                if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;

                                widget.currentRow.changed = false;
                                widget.view();
                                ScreenService.closeSwipe();
                                widget.reload();
                            }).catch(function (e) {
                                ScreenService.showMessage(e.data.error, "ERROR");
                            });
                            ScreenService.showMessage("Salvo com sucesso!");
                            ScreenService.closePopup();
                        }), function () {};
                    } else {
                        restEngine.post('cnc#/cnc01600_estabelecName', {
                            "requestType": "DataSet",
                            "dataset": [widget.currentRow]
                        }).then(function (response) {
                            if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;

                            widget.currentRow.changed = false;
                            widget.view();
                            ScreenService.closeSwipe();
                            widget.reload();
                        }).catch(function (e) {
                            ScreenService.showMessage(e.data.error, "ERROR");
                        });
                        ScreenService.showMessage("Salvo com sucesso!");
                        ScreenService.closePopup();
                    }
                }).catch(function (error) {
                    switch (error.data.errorCode) {
                        default:
                            ScreenService.showMessage(error.data.error, 'ERROR');
                    }
                });
            } else {
                restEngine.post('cnc#/cnc01600_estabelecName', {
                    "requestType": "DataSet",
                    "dataset": [widget.currentRow]
                }).then(function (response) {
                    if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;

                    widget.currentRow.changed = false;
                    widget.view();
                    ScreenService.closeSwipe();
                    widget.reload();
                }).catch(function (e) {
                    ScreenService.showMessage(e.data.error, "ERROR");
                });
                ScreenService.showMessage("Salvo com sucesso!");
                ScreenService.closePopup();
            }
        }
    };

    this.handleContDataSource = function (widget) {
        var contaField = widget.getField("DSCONTCORR");

        _this6.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
            contaField.dataSource.data.map(function (row) {
                var groups = maskObject.CDPICTCONT.split("-");
                var mask = groups.reduce(function (acc, charSet) {
                    return acc + '(\\d{' + charSet.length + '})';
                }, "^") + '.*';
                var format = groups.length == 1 ? "$1" : "$1-$2";

                row.CONT = row.CDCONTCORR.replace(new RegExp(mask), format) + ' - ' + row.CDREDUCONT;
            });
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01600_cadastroEstabelecimento_Controller', Cnc01600_cadastroEstabelecimento_Controller);
});

// FILE: ./assets/js/controllers/Cnc01700_ParametrizacaoProduto_Controller.js
function Cnc01700_ParametrizacaoProduto_Controller(ScreenService, templateManager, restEngine, Filter) {
    var _this7 = this;

    var produtoAntigo;
    var estabelecAnt;

    this.getWidget = function (args) {
        return args.owner;
    };

    this.applyMaskToString = function (string) {
        return function (expression) {
            return function (format) {
                return string.replace(expression, format);
            };
        };
    };

    this.completeFieldFilterParam = function (field) {
        return function (paramName) {
            return function (paramValue) {
                field.dataSourceFilter.filter(function (param) {
                    return param.name === paramName;
                }).slice().shift().value = paramValue;

                return field;
            };
        };
    };

    this.disableWidget = function (widget) {
        return function (chieldName) {
            return widget.widgets.filter(function (widget) {
                return widget.name === chieldName;
            }).shift().isVisible = false;
        };
    };

    this.enableWidget = function (widget) {
        return function (chieldName) {
            return widget.widgets.filter(function (widget) {
                return widget.name === chieldName;
            }).shift().isVisible = true;
        };
    };

    this.newAdqParams = function (widget) {
        return widget.newRow();
    };

    this.editAdqParams = function (args) {
        var widget = _this7.getWidget(args).widget;

        widget.edit();
        widget.backUpRow = Util.clone(widget.currentRow);
        widget.reload();

        templateManager.container.getWidget('produtoTab').isVisible = false;
    };

    this.cancelEditAdqParams = function (args) {
        var widget = _this7.getWidget(args).widget;

        templateManager.container.getWidget('produtoTab').isVisible = true;
        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            widget.list();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.view();
        }
    };

    this.saveAdqParams = function (args) {
        var widget = _this7.getWidget(args).widget;

        if (widget.isValid()) {
            templateManager.container.getWidget('produtoTab').isVisible = true;

            restEngine.post("cnc#/cnc01700_parametrizacao_produto_cliente/saveCliente", {
                "requestType": "DataSet",
                "dataset": [widget.currentRow]
            }).then(function (response) {
                ScreenService.successNotification("Dados salvos com sucesso!");
                widget.reload();
                widget.view();
            }).catch(function (error) {
                switch (error.data.errorCode) {
                    default:
                        ScreenService.showMessage(error.data.error, "ERROR");
                        break;
                }
            });
        }
    };

    this.applyMaskToField = function (field) {
        return function (maskString) {
            return field.mask.params.mask = maskString;
        };
    };

    this.clienteMasks = function (widget) {
        var field = widget.getField("NMRAZSOCCLIE");

        field.dataSource.data.map(function (data) {
            switch (data.IDTPIJURCLIE) {
                case "F":
                    data.NRINSJURCLIE = _this7.applyMaskToString(data.NRINSJURCLIE)(/^(\d{3})(\d{3})(\d{3})(\d{2}).*/)("$1.$2.$3-$4");
                    break;
                case "J":
                    data.NRINSJURCLIE = _this7.applyMaskToString(data.NRINSJURCLIE)(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2}).*/)("$1.$2.$3/$4-$5");
                    break;
            }
        });
    };

    this.newProduto = function (widget) {
        widget.newRow();
        templateManager.container.getWidget('integracaoFinanceiro').isVisible = false;
    };

    this.newEstabelec = function (widget) {
        widget.newRow();
    };

    this.saveProduto = function (widget) {
        if (widget.isValid()) {
            if (widget.currentRow.__is_new) {
                _this7.saveNewProduto(widget);
            } else {
                _this7.saveEditProduto(widget);
            }
        }
    };

    this.saveEstabelec = function (widget) {
        if (widget.isValid()) {
            if (widget.currentRow.__is_new) {
                _this7.saveNewEstabelec(widget);
            } else {
                _this7.saveEditEstabelec(widget);
            }
        }
    };

    this.saveNewProduto = function (widget) {
        var parentRow = widget.getParent().currentRow;
        widget.currentRow.NRADQUIRENTE = parentRow.NRADQUIRENTE;

        restEngine.post("cnc#/cnc01700_parametrizacao_produto_cliente/saveNewProdutoParams", {
            "requestType": "DataSet",
            "dataset": [widget.currentRow]
        }).then(function (response) {
            widget.currentRow.__is_new = widget.currentRow.__is_new ? false : false;
            ScreenService.successNotification("Dados salvos com sucesso!");
            templateManager.container.getWidget('integracaoFinanceiro').isVisible = true;
            widget.reload();
            ScreenService.closeSwipe();
            widget.list();
        }).catch(function (error) {
            switch (error.data.errorCode) {
                case 1:
                    ScreenService.showMessage("Produto já parametrizado para a Adquirente selecionada.", "ERROR");
                    break;
                default:
                    ScreenService.showMessage(error.data.error, "ERROR");
                    break;
            }
        });
    };

    this.saveNewEstabelec = function (widget) {
        var parentRow = widget.getParent().currentRow;
        var parentParentRow = widget.getParent().getParent().currentRow;

        widget.currentRow.NRADQUIRENTE = parentParentRow.NRADQUIRENTE;
        widget.currentRow.NRPRODUTOADQ = parentRow.NRPRODUTOADQ;

        restEngine.post("cnc#/cnc01700_parametrizacao_produto_cliente/saveNewEstabelecParams", {
            "requestType": "DataSet",
            "dataset": [widget.currentRow]
        }).then(function (response) {
            widget.currentRow.__is_new = widget.currentRow.__is_new ? false : false;
            ScreenService.successNotification("Dados salvos com sucesso!");
            widget.reload();
            ScreenService.closeSwipe();
            widget.list();
        }).catch(function (error) {
            switch (error.data.errorCode) {
                case 1:
                    ScreenService.showMessage("Estabelecimento já parametrizado para a Adquirente e Produto selecionados.", "ERROR");
                    break;
                default:
                    ScreenService.showMessage(error.data.error, "ERROR");
                    break;
            }
        });
    };

    this.saveEditProduto = function (widget) {
        var parentRow = widget.getParent().currentRow;
        widget.currentRow.NRADQUIRENTE = parentRow.NRADQUIRENTE;

        if (produtoAntigo == widget.currentRow.NRPRODUTOADQ) {
            restEngine.post("cnc#/cnc01700_parametrizacao_produto_cliente/saveProdutoParamsEdit", {
                "requestType": "DataSet",
                "dataset": [widget.currentRow]
            }).then(function (response) {
                ScreenService.successNotification("Dados salvos com sucesso!");
                widget.reload();
                widget.view();
                templateManager.container.getWidget('integracaoFinanceiro').isVisible = true;
            }).catch(function (error) {
                switch (error.data.errorCode) {
                    case 1:
                        ScreenService.showMessage("Produto já parametrizado para a Adquirente selecionada.", "ERROR");
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, "ERROR");
                        break;
                }
            });
        } else {
            widget.currentRow.NRPRODUTOADQANT = produtoAntigo;

            restEngine.post("cnc#/cnc01700_parametrizacao_produto_filterEstabelec", {
                "requestType": "FilterData",
                "filter": [{
                    "name": "NRADQUIRENTE",
                    "operator": "=",
                    "value": widget.currentRow.NRADQUIRENTE
                }, {
                    "name": "NRPRODUTOADQ",
                    "operator": "=",
                    "value": produtoAntigo
                }, {
                    "name": "NRORG",
                    "operator": "=",
                    "value": widget.currentRow.NRORG
                }]
            }).then(function (response) {
                if (response.dataset.cnc01700_parametrizacao_produto_filterEstabelec.length > 0) {
                    ScreenService.confirmMessage('Existem estabelecimentos parametrizados para este Produto. Ao altera-lo, essas parametrizações serão apagadas. Deseja continuar?', 'question', function () {
                        restEngine.post("cnc#/cnc01700_parametrizacao_produto_cliente/saveProdutoParamsEditProd", {
                            "requestType": "DataSet",
                            "dataset": [widget.currentRow]
                        }).then(function (response) {
                            ScreenService.successNotification("Dados salvos com sucesso!");
                            widget.reload();
                            widget.view();
                            templateManager.container.getWidget('integracaoFinanceiro').isVisible = true;
                        }).catch(function (error) {
                            switch (error.data.errorCode) {
                                case 1:
                                    ScreenService.showMessage("Produto já parametrizado para a Adquirente selecionada.", "ERROR");
                                    break;
                                default:
                                    ScreenService.showMessage(error.data.error, "ERROR");
                                    break;
                            }
                        });
                    });
                } else {
                    restEngine.post("cnc#/cnc01700_parametrizacao_produto_cliente/saveProdutoParamsEditProd", {
                        "requestType": "DataSet",
                        "dataset": [widget.currentRow]
                    }).then(function (response) {
                        ScreenService.successNotification("Dados salvos com sucesso!");
                        widget.reload();
                        widget.view();
                        templateManager.container.getWidget('integracaoFinanceiro').isVisible = true;
                    }).catch(function (error) {
                        switch (error.data.errorCode) {
                            case 1:
                                ScreenService.showMessage("Produto já parametrizado para a Adquirente selecionada.", "ERROR");
                                break;
                            default:
                                ScreenService.showMessage(error.data.error, "ERROR");
                                break;
                        }
                    });
                }
            });
        }
    };

    this.saveEditEstabelec = function (widget) {
        var parentRow = widget.getParent().currentRow;
        widget.currentRow.NRADQUIRENTE = parentRow.NRADQUIRENTE;
        widget.currentRow.NRPRODUTOADQ = parentRow.NRPRODUTOADQ;
        widget.currentRow.NRESTABELECANT = estabelecAnt;

        if (estabelecAnt == widget.currentRow.NRESTABELEC) {
            restEngine.post("cnc#/cnc01700_parametrizacao_produto_editestabelec", {
                "requestType": "DataSet",
                "dataset": [widget.currentRow]
            }).then(function (response) {
                ScreenService.successNotification("Dados salvos com sucesso!");
                widget.reload();
                widget.view();
            }).catch(function (error) {
                switch (error.data.errorCode) {
                    case 1:
                        ScreenService.showMessage("Estabelecimento já parametrizado para o Produto e a Adquirente selecionada.", "ERROR");
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, "ERROR");
                        break;
                }
            });
        } else {
            restEngine.post("cnc#/cnc01700_parametrizacao_produto_editestabelecNew", {
                "requestType": "DataSet",
                "dataset": [widget.currentRow]
            }).then(function (response) {
                ScreenService.successNotification("Dados salvos com sucesso!");
                widget.reload();
                widget.view();
            }).catch(function (error) {
                switch (error.data.errorCode) {
                    case 1:
                        ScreenService.showMessage("Estabelecimento já parametrizado para o Produto e a Adquirente selecionada.", "ERROR");
                        break;
                    default:
                        ScreenService.showMessage(error.data.error, "ERROR");
                        break;
                }
            });
        }
    };

    this.cancelEditProduto = function (args) {
        var widget = _this7.getWidget(args).widget;
        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
            widget.list();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.view();
        }
        templateManager.container.getWidget('integracaoFinanceiro').isVisible = true;
    };

    this.cancelEditEstabelec = function (args) {
        var widget = _this7.getWidget(args).widget;
        templateManager.container.getWidget('integracaoFinanceiro').isVisible = true;
        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
            widget.list();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.view();
        }
    };

    this.editProduto = function (args) {
        var widget = _this7.getWidget(args).widget;
        produtoAntigo = widget.currentRow.NRPRODUTOADQ;

        widget.edit();
        widget.backUpRow = Util.clone(widget.currentRow);
        templateManager.container.getWidget('integracaoFinanceiro').isVisible = false;
    };

    this.editEstabelec = function (args) {
        var widget = _this7.getWidget(args).widget;
        estabelecAnt = widget.currentRow.NRESTABELEC;
        widget.edit();
        widget.backUpRow = Util.clone(widget.currentRow);
    };

    this.setFilterProdutos = function (args) {
        var widget = args.owner;
        /*Preenchimento do datasource para o grid de produtos para realizar o filtro de produtos */

        widget.dataSourceFilter[1].value = widget.getParent().currentRow.NRADQUIRENTE;
        widget.reload();

        widget.fields.forEach(function (f) {
            return f.reload();
        });
    };

    this.deleteProduto = function (args) {
        var widget = args.owner.widget;
        var parentRow = widget.getParent().currentRow;

        widget.currentRow.NRADQUIRENTE = parentRow.NRADQUIRENTE;

        restEngine.post("cnc#/cnc01700_parametrizacao_produto_filterEstabelec", {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRADQUIRENTE",
                "operator": "=",
                "value": widget.currentRow.NRADQUIRENTE
            }, {
                "name": "NRPRODUTOADQ",
                "operator": "=",
                "value": widget.currentRow.NRPRODUTOADQ
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": widget.currentRow.NRORG
            }]
        }).then(function (response) {
            if (response.dataset.cnc01700_parametrizacao_produto_filterEstabelec.length > 0) {
                ScreenService.confirmMessage('Ao excluir este produto as parametrizações de estabelecimento contidos nele serão apagadas. Deseja continuar?', 'question', function () {
                    restEngine.post('cnc#/cnc01700_parametrizacao_produto_deleteproduto', {
                        "requestType": "DataSet",
                        "dataset": [widget.currentRow]
                    }).then(function (response) {
                        ScreenService.successNotification(response.notifications[0].message);
                        ScreenService.closeSwipe();
                        widget.reload();
                    }).catch(function (error) {
                        ScreenService.showMessage(error.data.error, 'ERROR');
                    });
                });
            } else {
                ScreenService.confirmMessage('Deseja excluir este registo?', 'question', function () {
                    restEngine.post('cnc#/cnc01700_parametrizacao_produto_deleteproduto', {
                        "requestType": "DataSet",
                        "dataset": [widget.currentRow]
                    }).then(function (response) {
                        ScreenService.successNotification(response.notifications[0].message);
                        ScreenService.closeSwipe();
                        widget.reload();
                    }).catch(function (error) {
                        ScreenService.showMessage(error.data.error, 'ERROR');
                    });
                });
            }
        });
    };

    this.deleteEstabelec = function (args) {

        var widget = args.owner.widget;
        var parentRow = widget.getParent().currentRow;
        var parentParentRow = widget.getParent().getParent().currentRow;

        widget.currentRow.NRADQUIRENTE = parentParentRow.NRADQUIRENTE;
        widget.currentRow.NRPRODUTOADQ = parentRow.NRPRODUTOADQ;

        ScreenService.confirmMessage('Deseja excluir este registro?', 'question', function () {
            restEngine.post('cnc#/cnc01700_parametrizacao_produto_deleteestabelec', {
                "requestType": "DataSet",
                "dataset": [widget.currentRow]
            }).then(function (response) {
                ScreenService.successNotification(response.notifications[0].message);
                ScreenService.closeSwipe();
                widget.reload();
            }).catch(function (error) {
                ScreenService.showMessage(error.data.error, 'ERROR');
            });
        });
    };

    this.setFilterEstabelec = function (args) {
        var widget = args.owner;
        //Preenchimento do datasource para o grid de estabelecimentos para realizar o filtro de estabelecimentos

        widget.dataSourceFilter[1].value = widget.getParent().currentRow.NRADQUIRENTE;
        widget.dataSourceFilter[2].value = widget.getParent().currentRow.NRPRODUTOADQ;

        widget.reload();
    };

    this.reloadEstabelec = function (args) {
        var widget = args.owner;
        widget.reload();
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01700_ParametrizacaoProduto_Controller', Cnc01700_ParametrizacaoProduto_Controller);
});

// FILE: ./assets/js/controllers/Cnc01800_reconciliacao_Controller.js
function Cnc01800_reconciliacao_Controller(ScreenService, templateManager, restEngine, UtilitarioGeral) {
    var _this8 = this;

    var RECONCILIACAO_MSG = ["Realizar a reconciliação do período?", "Devido à quantidade de dias a reconciliação pode levar um tempo maior para ser executada. Deseja continuar?", "Devido à quantidade de estabelecimentos selecionados, a reconciliação pode levar um tempo maior para ser executada. Deseja continuar?", "Devido à quantidade de estabelecimentos e a quantidade de dias selecionados, a reconciliação pode levar um longo tempo para ser executada. Deseja continuar?"];

    this.setIntervalToday = function (widget, fieldName) {
        widget.getField(fieldName).value({
            'start': moment().format('DD/MM/YYYY'),
            'period': undefined,
            'end': moment().format('DD/MM/YYYY')
        });
    };

    this.reconciliacaoAutomatica = function (args) {
        var widget = args.owner.widget;

        if (widget.isValid()) {
            var intervalLength = Math.round(
            // _d de um objeto moment é um objeto Date. A subtração de dois Dates retorna um inteiro com a dferença entre as datas
            // em milisegundos
            (moment(widget.currentRow.DTVENDA.end, "DD/MM/YYYY")._d - moment(widget.currentRow.DTVENDA.start, "DD/MM/YYYY")._d) / 1000 / 60 / 60 / 24);
            var qtdEstabelec = widget.currentRow.LABELESTABELEC.length;

            var msg = intervalLength < 30 ? qtdEstabelec > 1 ? RECONCILIACAO_MSG[2] : RECONCILIACAO_MSG[0] : qtdEstabelec > 1 ? RECONCILIACAO_MSG[3] : RECONCILIACAO_MSG[1];

            ScreenService.confirmMessage(msg, "question", function (_) {
                restEngine.post("cnc#/Cnc01800_reconciliar", {
                    "requestType": "Row",
                    "row": widget.currentRow
                }, null, null).then(function (response) {
                    _this8.verifyDivergences(widget, response.dataset.log);
                }).catch(function (error) {
                    switch (error.data.errorCode) {
                        default:
                            ScreenService.showMessage(error.data.error, "ERROR");
                            break;
                    }
                });
            });
        }
    };

    this.verifyDivergences = function (widget, log) {
        restEngine.post("cnc#/Cnc01800_verifyDivergences", {
            "requestType": "Row",
            "row": widget.currentRow
        }).then(function (response) {
            if (log.length > 0) {
                ScreenService.showMessage("Atenção! Alguns estabelecimentos não tiveram o financeiro reprocessado. Favor verificar o log.", "success");
                var widtemp = templateManager.container.getWidget('reconciliacaoAutomatica');
                UtilitarioGeral.showErrorMessages(widtemp, log, "Log de Reconciliação Automática");
            } else {
                ScreenService.showMessage("Reconciliação realizada com sucesso.", "success");
            }
        }).catch(function (error) {
            ScreenService.confirmMessage("Foram encontrados divergências na reconciliação, deseja ir para a tela de divergencias?", "question", function (_) {
                return ScreenService.openWindow("cnc#cnc02200_error_log");
            });
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01800_reconciliacao_Controller', Cnc01800_reconciliacao_Controller);
});

// FILE: ./assets/js/controllers/Cnc01900_relatorio_taxa_venda_Controller.js
function Cnc01900_relatorio_taxa_venda_Controller(ScreenService, restEngine, templateManager) {

    this.setIntervalToday = function (widget, fieldName) {
        widget.getField(fieldName).value({
            'start': moment().format('DD/MM/YYYY'),
            'period': undefined,
            'end': moment().format('DD/MM/YYYY')
        });
    };

    this.reduce = function (array) {
        return array.reduce(function (acc, elem) {
            return acc + "_" + elem;
        }, "") + "_";
    };

    this.setFieldValue = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NRPRODUTOADQ':
                    field.setValue(null);
                    break;
                default:
                    field.setValue(null);
            }
        }
    };

    this.generateReport = function (widget) {
        if (widget.isValid()) {
            var NRESTABELEC = widget.currentRow.NMESTABELEC;
            var NRADQUIRENTE = widget.currentRow.NMADQUIRENTE;
            var DTVENDAINI = widget.currentRow.DTVENDA.start;
            var DTVENDAFIN = widget.currentRow.DTVENDA.end;
            var NRPRODUTOADQ = widget.currentRow.NRPRODUTOADQ ? widget.currentRow.NRPRODUTOADQ : "T";

            var row = {
                "__report_name": "taxaVenda",
                "nrorg": getLocalVar('NRORG'),
                "NRORG": getLocalVar('NRORG'),
                "p_nrestabelec": widget.currentRow.NMESTABELEC,
                "p_nradquirente": widget.currentRow.NMADQUIRENTE,
                "dtvendaini": widget.currentRow.DTVENDA.start,
                "dtvendafin": widget.currentRow.DTVENDA.end,
                "p_nrprodutoadq": widget.currentRow.NRPRODUTOADQ ? this.reduce(widget.currentRow.NRPRODUTOADQ) : "T"
            };
            widget.dataSourceFilter = [{
                "name": "NRESTABELEC",
                "operator": "=",
                "value": NRESTABELEC
            }, {
                "name": "NRADQUIRENTE",
                "operator": "=",
                "value": NRADQUIRENTE
            }, {
                "name": "DTVENDAINI",
                "operator": "=",
                "value": DTVENDAINI
            }, {
                "name": "DTVENDAFIN",
                "operator": "=",
                "value": DTVENDAFIN
            }, {
                "name": "NRPRODUTOADQ",
                "operator": "=",
                "value": NRPRODUTOADQ
            }];
            widget.dataSource.load(widget.dataSourceFilter).then(function (dado) {
                if (dado.dataset.cnc01900_relTaxaVendaCheck.length > 0) {
                    return restEngine.post("cnc#/cnc01900_relatorio_taxa_venda", {
                        "requestType": "Row",
                        "row": row
                    }).then(function (response) {
                        if (response && response.messages) {
                            var report = response.messages.pop();
                            window.open(report.message);
                        }
                    }).catch(function (response) {
                        ScreenService.showMessage(response.data.error, "ERROR");
                    });
                } else {
                    ScreenService.showMessage("Não há dados com os parâmetros escolhidos.");
                }
            });
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc01900_relatorio_taxa_venda_Controller', Cnc01900_relatorio_taxa_venda_Controller);
});

// FILE: ./assets/js/controllers/Cnc02000_VendasInexistentesAdquirente_Controller.js
function Cnc02000_VendasInexistentesAdquirente_Controller(ScreenService, templateManager, restEngine, UtilitarioGeral, ValidationEngine) {

    var WIDGET_PRINCIPAL = "relatoriocaixa",
        CAIXA_FIELD = 'CDCAIXA',
        FILIAL_FIELD = 'CDFILIAL',
        LOJA_FIELD = 'CDLOJA',
        DATAFINAL_FIELD = 'DTFINAL',
        DTENTRVENDA_FIELD = 'DTENTRVENDA',
        REPORT_NAME = 'REPORT_NAME',
        FECHAMENTO_CAIXA_REL = 'rfc';

    this.activateMainWidget = function (widget) {
        widget.activate();
    };

    this.vendasInexistentesAdquirenteGerarRelatorio = function (args) {
        var widget = args.owner.widget;

        if (widget.isValid()) {
            widget.currentRow.NRORG = getLocalVar('NRORGTRAB');
            widget.currentRow.DTVENDAINI = widget.currentRow.DTVENDA.start;
            widget.currentRow.DTVENDAFIN = widget.currentRow.DTVENDA.end;
            if ((widget.currentRow.VALINICIAL === undefined || widget.currentRow.VALINICIAL === "" || widget.currentRow.VALINICIAL === null) && (widget.currentRow.VALFINAL === undefined || widget.currentRow.VALFINAL === "" || widget.currentRow.VALFINAL === null)) {
                widget.currentRow.VALINICIAL = null;
                widget.currentRow.VALFINAL = null;
                widget.currentRow.TVALOR = 'T';
            } else {
                widget.currentRow.TVALOR = null;
            }
            var CDFILIAL = widget.currentRow.CDFILIAL;
            var DTVENDAINI = widget.currentRow.DTVENDAINI;
            var DTVENDAFIN = widget.currentRow.DTVENDAFIN;
            var VALINICIAL = widget.currentRow.VALINICIAL;
            var VALFINAL = widget.currentRow.VALFINAL;
            var TVALOR = widget.currentRow.TVALOR;
            widget.dataSourceFilter = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": CDFILIAL
            }, {
                "name": "DTVENDAINI",
                "operator": "=",
                "value": DTVENDAINI
            }, {
                "name": "DTVENDAFIN",
                "operator": "=",
                "value": DTVENDAFIN
            }, {
                "name": "VALINICIAL",
                "operator": "=",
                "value": VALINICIAL
            }, {
                "name": "VALFINAL",
                "operator": "=",
                "value": VALFINAL
            }, {
                "name": "TVALOR",
                "operator": "=",
                "value": TVALOR
            }];
            widget.dataSource.load(widget.dataSourceFilter).then(function (dado) {
                if (dado.dataset.cnc02000_relVendaInexisAdqCheck.length > 0) {
                    geraRelatorio(widget.currentRow, widget);
                } else {
                    ScreenService.showMessage("Não há dados com os parâmetros escolhidos.");
                }
            });
        }
    };

    var geraRelatorio = function geraRelatorio(row) {
        restEngine.post("cnc#/cnc02000_relVendaInexisAdquirente", {
            "requestType": "Row",
            "row": row
        }).then(function (response) {
            window.open(response.messages[0].message);
        }).catch(function (response) {
            ScreenService.showMessage(response.data.error, "ERROR");
        });
    };

    this.validateMinSaleValue = function (value, validationAttrs) {
        var minValueField = validationAttrs.field;
        var minValue = parseFloat(templateManager.container.getWidget(validationAttrs.widget).currentRow[minValueField]);
        value = parseFloat(value);
        return {
            "valid": minValue <= value || isNaN(minValue),
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Minimum value: ") + minValue
        };
    };
    ValidationEngine.registerValidation('minValor', this.validateMinSaleValue);

    this.validadeteMaxSaleValue = function (value, validationAttrs) {
        var maxValueField = validationAttrs.field;
        var maxValue = parseFloat(templateManager.container.getWidget(validationAttrs.widget).currentRow[maxValueField]);
        value = parseFloat(value);
        return {
            "valid": maxValue >= value || isNaN(maxValue),
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Maximum value: ") + maxValue
        };
    };
    ValidationEngine.registerValidation('maxValor', this.validadeteMaxSaleValue);

    this.widgetOnEnter = function (widget) {
        widget.getField('NMFILIAL').reload().then(function () {
            widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
            widget.currentRow.NMFILIAL = getLocalVar('NMFILIAL');
            widget.currentRow.DTVENDA = {
                start: moment().format('DD/MM/YYYY'),
                period: undefined,
                end: moment().format('DD/MM/YYYY')
            };
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc02000_VendasInexistentesAdquirente_Controller', Cnc02000_VendasInexistentesAdquirente_Controller);
});

// FILE: ./assets/js/controllers/Cnc02100_vendas_antecipadas_relatorio_Controller.js
function Cnc02100_vendas_antecipadas_relatorio_Controller(ScreenService, templateManager, restEngine) {

    this.setIntervalToday = function (widget, fieldName) {
        widget.getField(fieldName).value({
            'start': moment().format('DD/MM/YYYY'),
            'period': undefined,
            'end': moment().format('DD/MM/YYYY')
        });
    };

    this.reduce = function (array) {
        return array.reduce(function (acc, elem) {
            return acc + "|" + elem;
        }, "") + "|";
    };

    this.validadeSelect = function (row) {
        restEngine.post("cnc#/cnc02100_vendas_antecipadas_report_validation", {
            "requestType": "Row",
            "row": row
        }).then(function (response) {
            var count = response.dataset.cnc02100_vendas_antecipadas_report_validation.pop();
            this.createReport(row, count);
        }.bind(this)).catch(function (response) {
            ScreenService.showMessage(response.data.error, "ERROR");
        });
    };

    this.createReport = function (row, count) {
        if (count > 0) {
            return restEngine.post("cnc#/cnc02100_vendas_antecipadas_report", {
                "requestType": "Row",
                "row": row
            }).then(function (response) {
                if (response && response.messages) window.open(response.messages.pop().message);
            }).catch(function (response) {
                ScreenService.showMessage(response.data.error, "ERROR");
            });
        } else {
            ScreenService.showMessage("Não existem dados para o filtro realizado.");
        }
    };

    this.generateReport = function (widget) {
        if (widget.isValid()) {
            var nrorg = widget.getField("NRESTABELEC").dataSource.data.map(function (data) {
                return data.NRORG;
            }).shift();

            var row = {
                "__report_name": "venda_antecipada",
                "nrorg": nrorg,
                "nrestabelec": widget.currentRow.NMESTABELEC,
                "nradquirente": widget.currentRow.NMADQUIRENTE,
                "dtvenda_ini": widget.currentRow.DTVENDA.start,
                "dtvenda_fin": widget.currentRow.DTVENDA.end,
                "nrprodutoadq": widget.currentRow.NRPRODUTOADQ ? this.reduce(widget.currentRow.NRPRODUTOADQ) : "T",
                "vrvenda_min": widget.currentRow.VRVENDA_MIN ? widget.currentRow.VRVENDA_MIN : "T",
                "vrvenda_max": widget.currentRow.VRVENDA_MAX ? widget.currentRow.VRVENDA_MAX : "T",
                "vrvenda": widget.currentRow.VRVENDA_MIN ? "s" : "T"
            };

            this.validadeSelect(row);
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc02100_vendas_antecipadas_relatorio_Controller', Cnc02100_vendas_antecipadas_relatorio_Controller);
});

// FILE: ./assets/js/controllers/Cnc02200_error_log_Controller.js
function Cnc02200_error_log_Controller(ScreenService, restEngine, ZHPromise, Filter, templateManager, localStorageEngine) {
    var _this9 = this;

    var estabelecimentosArr = [];
    var countEstabelecErr = 0;
    var countEstabelecs = 0;
    var widgetAtual;

    this.checkFlag = function (widget) {
        return localStorageEngine.getLocalVar("shouldNotShowFilter") ? _this9.processOriginDashboard(widget) : _this9.processOriginMenu(widget);
    };

    this.activateMainWidget = function (widget) {
        return widget.activate();
    };

    this.processOriginMenu = function (widget) {
        widget.dataSource.lazyLoad = true;
        Filter.startFiltering(widget);
    };

    this.processOriginDashboard = function (widget) {
        localStorageEngine.setLocalVar("shouldNotShowFilter", false);
        Filter.startFiltering(widget);
        widget.dataSource.lazyLoad = false;

        var dataSourceFilter = [{
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }];

        restEngine.post("cnc#/select_estabelec", {
            "requestType": "FilterData",
            "filter": dataSourceFilter
        }).then(function (response) {
            var estabelecimentos = [];
            for (var quantidadeDeEstabelec = 0; quantidadeDeEstabelec < response.dataset.select_estabelec.length; quantidadeDeEstabelec++) {
                estabelecimentos.push(response.dataset.select_estabelec[quantidadeDeEstabelec].NRESTABELEC);
            }
            widget.currentRow.NRESTABELEC = estabelecimentos;
        });

        widget.getField('NRESTABELEC').reload();
    };

    this.handleActionVisibility = function (widget, actionName) {
        var action = widget.actions.filter(function (action) {
            return action.name.includes(actionName);
        }).shift();
        action.isVisible = widget.dataSource.data.length > 0 ? true : false;
    };

    this.doProcedureRequest = function (row) {
        restEngine.post("cnc#/cnc02200_generateFinancProc", {
            "requestType": "Row",
            "row": row
        }).then(function (response) {
            countEstabelecErr += 1;
            if (countEstabelecErr == countEstabelecs) {
                _this9.processResult(widgetAtual);
            }
        });
    };

    this.generateFinancProc = function (widget) {
        widgetAtual = widget;
        ScreenService.confirmMessage("Certifique-se de corrigir todos os erros antes de prosseguir. Gerar dados do financeiro?", "question", function (_) {
            countEstabelecErr = 0;
            countEstabelecs = 0;
            estabelecimentosArr = [];

            for (var quantidadeDeEstabelec = 0; quantidadeDeEstabelec < widget.dataSource.data.length; quantidadeDeEstabelec++) {
                estabelecimentosArr.push(widget.dataSource.data[quantidadeDeEstabelec].NRESTABELEC);
            }
            widget.currentRow.NRESTABELEC = estabelecimentosArr;
            widget.currentRow.NRORG = widget.dataSource.data[0].NRORG;
            var valueBackEstabelec = -1;
            var estabelecAnt = null;
            for (var countF = 0; countF < estabelecimentosArr.length; countF++) {
                if (estabelecAnt) {
                    if (widget.dataSource.data[countF].NRESTABELEC != estabelecAnt.NRESTABELEC) {
                        countEstabelecs += 1;
                        estabelecAnt = widget.dataSource.data[countF];
                    }
                } else {
                    countEstabelecs += 1;
                    estabelecAnt = widget.dataSource.data[countF];
                }
            }
            for (var numeroDeEstabelecimentos = 0; numeroDeEstabelecimentos < widget.dataSource.data.length; numeroDeEstabelecimentos++) {
                if (widget.currentRow.NRESTABELEC[numeroDeEstabelecimentos]) {
                    var NRESTABELEC = widget.currentRow.NRESTABELEC[numeroDeEstabelecimentos];
                }

                if (NRESTABELEC != valueBackEstabelec) {
                    valueBackEstabelec = widget.dataSource.data[numeroDeEstabelecimentos].NRESTABELEC;
                    _this9.doProcedureRequest(widget.dataSource.data[numeroDeEstabelecimentos]);
                }
            }
        });
    };

    this.mountFilter = function (filterParams) {
        return filterParams.map(function (param) {
            return param.name.includes("NRORG") ? Object.assign({}, param, { value: getLocalVar("NRORGTRAB") }) : param;
        });
    };

    this.processResult = function (widget) {
        var NRESTABELEC = widget.currentRow.NRESTABELEC;
        var dataSourceFilter = [{
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }, {
            "name": "NRESTABELEC",
            "operator": "=",
            "value": NRESTABELEC
        }];
        restEngine.post("cnc#/cnc02200_error_log_countErrors", {
            "requestType": "FilterData",
            "filter": dataSourceFilter
        }).then(function (response) {
            ScreenService.showMessage(response.dataset.cnc02200_error_log_countErrors.shift(), "suceess");
            widget.dataSource.load(widget.dataSourceFilter);
        }).catch(function (error) {
            ScreenService.showMessage(error.data.error, "ERROR");
            widget.dataSource.load(widget.dataSourceFilter);
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc02200_error_log_Controller', Cnc02200_error_log_Controller);
});

// FILE: ./assets/js/controllers/Cnc02300_StatusVendaController.js
function Cnc02300_StatusVendaController(restEngine, ScreenService, Filter) {

    var self = this;

    this.openFilter = function (widget) {
        var rows = {
            DTVENDA: {
                start: moment().format('DD/MM/YYYY'),
                period: undefined,
                end: moment().format('DD/MM/YYYY')
            }
        };

        Filter.openFilterWithRows(widget, rows);
    };

    this.setFieldValue02300 = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NMESTABELEC':
                    field.setValue('');
                    break;
                case 'NMPRODUTOADQ':
                    field.setValue('');
                    break;
                default:
                    field.setValue('');
            }
        }
    };

    this.getStatusVendas = function (widget) {
        var filterRow = Filter.getAppliedFilters(widget);

        restEngine.post("cnc#/cnc02300_status_venda_grafico", {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("NRORGTRAB")
            }, {
                "name": "NRESTABELEC",
                "operator": "IN",
                "value": filterRow.NMESTABELEC
            }, {
                "name": "NRPRODUTOADQ",
                "operator": "IN",
                "value": filterRow.NMPRODUTOADQ
            }, {
                "name": "DTVENDA",
                "operator": "BETWEEN",
                "value": filterRow.DTVENDA
            }]
        }).then(function (response) {
            widget.dataSource.data = response.dataset.cnc02300_status_venda_grafico;
            var serieColors = [];
            //#1987c4 AZUL(Corrigidas)
            //#c83737 VERMELHO(Não Conciliadas)
            //#00d149 VERDE(Corretas)
            if (widget.dataSource.data.length > 0) {
                if (widget.dataSource.data[0].LABELCONC == "Não Conciliadas" && !widget.dataSource.data[1]) serieColors = ["#c83737"];
                if (widget.dataSource.data[0].LABELCONC == "Não Conciliadas" && !!widget.dataSource.data[1]) {
                    if (widget.dataSource.data[0].LABELCONC == "Não Conciliadas" && widget.dataSource.data[1].LABELCONC == "Corretas" && !widget.dataSource.data[2]) serieColors = ["#c83737", "#00d149"];
                    if (widget.dataSource.data[0].LABELCONC == "Não Conciliadas" && widget.dataSource.data[1].LABELCONC == "Corrigidas" && !widget.dataSource.data[2]) serieColors = ["#c83737", "#1987c4"];
                    if (widget.dataSource.data[0].LABELCONC == "Não Conciliadas" && widget.dataSource.data[1].LABELCONC == "Corrigidas" && !!widget.dataSource.data[2]) serieColors = ["#c83737", "#1987c4", "#00d149"];
                    if (widget.dataSource.data[0].LABELCONC == "Não Conciliadas" && widget.dataSource.data[1].LABELCONC == "Corretas" && !!widget.dataSource.data[2]) serieColors = ["#c83737", "#00d149", "#1987c4"];
                }
                if (widget.dataSource.data[0].LABELCONC == "Corretas" && !widget.dataSource.data[1]) serieColors = ["#00d149"];
                if (widget.dataSource.data[0].LABELCONC == "Corretas" && !!widget.dataSource.data[1]) {
                    if (widget.dataSource.data[0].LABELCONC == "Corretas" && widget.dataSource.data[1].LABELCONC == "Não Conciliadas" && !widget.dataSource.data[2]) serieColors = ["#00d149", "#c83737"];
                    if (widget.dataSource.data[0].LABELCONC == "Corretas" && widget.dataSource.data[1].LABELCONC == "Corrigidas" && !widget.dataSource.data[2]) serieColors = ["#00d149", "#1987c4"];
                    if (widget.dataSource.data[0].LABELCONC == "Corretas" && widget.dataSource.data[1].LABELCONC == "Não Conciliadas" && !!widget.dataSource.data[2]) serieColors = ["#00d149", "#c83737", "#1987c4"];
                    if (widget.dataSource.data[0].LABELCONC == "Corretas" && widget.dataSource.data[1].LABELCONC == "Corrigidas" && !!widget.dataSource.data[2]) serieColors = ["#00d149", "#1987c4", "#c83737"];
                }
                if (widget.dataSource.data[0].LABELCONC == "Corrigidas" && !widget.dataSource.data[1]) serieColors = ["#1987c4"];
                if (widget.dataSource.data[0].LABELCONC == "Corrigidas" && !!widget.dataSource.data[1]) {
                    if (widget.dataSource.data[0].LABELCONC == "Corrigidas" && widget.dataSource.data[1].LABELCONC == "Não Conciliadas" && !widget.dataSource.data[2]) serieColors = ["#1987c4", "#c83737"];
                    if (widget.dataSource.data[0].LABELCONC == "Corrigidas" && widget.dataSource.data[1].LABELCONC == "Corretas" && !widget.dataSource.data[2]) serieColors = ["#1987c4", "#00d149"];
                    if (widget.dataSource.data[0].LABELCONC == "Corrigidas" && widget.dataSource.data[1].LABELCONC == "Não Conciliadas" && !!widget.dataSource.data[2]) serieColors = ["#1987c4", "#c83737", "#00d149"];
                    if (widget.dataSource.data[0].LABELCONC == "Corrigidas" && widget.dataSource.data[1].LABELCONC == "Corretas" && !!widget.dataSource.data[2]) serieColors = ["#1987c4", "#00d149", "#c83737"];
                }
                var grafico = [self.getPieChart(serieColors)];
                self.setWidgetChartSeries(widget, grafico);
            }
            widget.updateChart();
        }).catch(function (error) {
            ScreenService.showMessage(error.data.errMsg, "ERROR");
        }).finally(function () {
            widget.getParent().activate();
        });
    };

    this.getPieChart = function (color) {
        var chartSeries = {
            "serieType": "pie",
            "serieColors": color,
            "serieFieldLabel": "LABELCONC",
            "serieFieldValue": "QTDVENDA",
            "serieShowValueLabels": "value",
            "serieStrokeColor": "white",
            "serieStrokeWidth": 1
        };
        return chartSeries;
    };

    this.setWidgetChartSeries = function (widget, chartSeries) {
        widget.chartSeries = chartSeries;
    };

    this.showChartDetails = function (dashboard) {
        ScreenService.confirmMessage("Devido à quantidade de vendas, o processamento pode demorar. Deseja continuar?", "question", function (_) {
            var widget = dashboard.container.getWidget("statusSalesChart");
            var swipeWidget = widget.container.getWidget("chartDetails");

            ScreenService.openSwipe(swipeWidget);

            var filterRow = Filter.getAppliedFilters(widget);
            restEngine.post("cnc#/cnc02300_status_vendas_details", {
                "requestType": "FilterData",
                "filter": [{
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("NRORGTRAB")
                }, {
                    "name": "NRESTABELEC",
                    "operator": "IN",
                    "value": filterRow.NMESTABELEC
                }, {
                    "name": "NRPRODUTOADQ",
                    "operator": "IN",
                    "value": filterRow.NMPRODUTOADQ
                }, {
                    "name": "DTVENDA",
                    "operator": "BETWEEN",
                    "value": filterRow.DTVENDA
                }]
            }).then(function (response) {
                return swipeWidget.dataSource.data = response.dataset.cnc02300_status_vendas_details;
            }).catch(function (error) {
                return ScreenService.showMessage(error.data.errMsg, "ERROR");
            });
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc02300_StatusVendaController', Cnc02300_StatusVendaController);
});

// FILE: ./assets/js/controllers/Cnc02400_arquivos_adquirentes_Controller.js
function Cnc02400_arquivos_adquirentes_Controller(ScreenService, restEngine, templateManager, Filter) {
    var _this10 = this;

    this.openFilter = function (widget) {
        var rows = {
            DTLOTE: {
                'start': moment().format('DD/MM/YYYY'),
                'period': undefined,
                'end': moment().format('DD/MM/YYYY')
            },
            TYPE: 'T'
        };

        Filter.openFilterWithRows(widget, rows);
    };

    this.setIntervalToday = function (widget, fieldName) {
        widget.getField(fieldName).value({
            'start': moment().format('DD/MM/YYYY'),
            'period': undefined,
            'end': moment().format('DD/MM/YYYY')
        });
    };

    this.sumDataSourceRows = function (widget) {
        var rows = widget.dataSource.data;

        var totLoteVenda = rows.filter(function (row) {
            return row.TYPE === "V";
        }).reduce(function (acc, row) {
            return parseFloat(row.VRTOTLIQVENDA) + acc;
        }, 0).toFixed(2).toString().replace(/\./g, ",").replace(/(\d)(?=(\d{3})+(?!\d))/g, "$1.");

        var totLotePagto = rows.filter(function (row) {
            return row.TYPE === "P";
        }).reduce(function (acc, row) {
            return parseFloat(row.VRTOTLIQVENDA) + acc;
        }, 0).toFixed(2).toString().replace(/\./g, ",").replace(/(\d)(?=(\d{3})+(?!\d))/g, "$1.");

        var footer = widget.container.getWidget("lotes_footer");
        _this10.handleFooter("TotLoteVendas", "Total Lote Vendas: R$ ", totLoteVenda)(footer);
        _this10.handleFooter("TotLotePagamentos", "Total Lote Pagamentos: R$ ", totLotePagto)(footer);
    };

    this.resetFooterLabel = function (fieldName, newLabel) {
        return function (footer) {
            footer.getField(fieldName).label = newLabel;
            return footer;
        };
    };

    this.setFooterValue = function (fieldName, value) {
        return function (footer) {
            footer.getField(fieldName).value(value);
            return footer;
        };
    };

    this.handleFooter = function (fieldName, newLabel, value) {
        return function (footer) {
            return _this10.setFooterValue(fieldName, value)(_this10.resetFooterLabel(fieldName, newLabel)(footer));
        };
    };

    this.detailLoteInfo = function (widget) {
        var swipe = widget.container.getWidget("arquivos_adquirentes_vendas_lote");
        var row = widget.currentRow;

        ScreenService.openSwipe(swipe);

        swipe.dataSourceFilter = [{
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('NRORGTRAB')
        }, {
            "name": "CDLOTEVENDA",
            "operator": "=",
            "value": row.CDLOTEVENDA
        }, {
            "name": "TYPE",
            "operator": "=",
            "value": row.TYPE
        }];

        swipe.dataSource.load(swipe.dataSourceFilter).then(function (_) {
            swipe.dataSource.data.filter(function (row) {
                return row.IDANTECIPADO === "N";
            }).map(function (row) {
                return row.IDANTECIPADO = "Não";
            });
            swipe.dataSource.data.filter(function (row) {
                return row.IDANTECIPADO === "S";
            }).map(function (row) {
                return row.IDANTECIPADO = "Sim";
            });
        });
    };

    this.populateVendaView = function (widget) {
        var swipe = widget.container.getWidget("saleView");
        var row = widget.currentRow;

        ScreenService.openSwipe(swipe);

        swipe.fields.map(function (field) {
            return field.value(row[field.name]);
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc02400_arquivos_adquirentes_Controller', Cnc02400_arquivos_adquirentes_Controller);
});

// FILE: ./assets/js/controllers/Cnc02500_serie_estabelecimento_Controller.js
function Cnc02500_serie_estabelecimento_Controller(ScreenService, templateManager, Filter, maskEngine, ZhUtil, restEngine, ZHPromise, UtilitarioGeral) {
    var _this11 = this;

    this.getWidget = function (args) {
        return args.owner;
    };

    this.editSerie = function (args) {
        var widget = this.getWidget(args).widget;
        widget.edit();
        widget.backUpRow = Util.clone(widget.currentRow);
    };

    this.cancelEditSerie = function (args) {
        var widget = this.getWidget(args).widget;

        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.view();
        }
    };

    this.saveSerie = function (args) {
        var widget = this.getWidget(args).widget;
        widget.currentRow.__is_new = false;

        if (widget.isValid()) {
            restEngine.post("cnc#/cnc02500_serie_estabelecimento/save", {
                "requestType": "DataSet",
                "dataset": [widget.currentRow]
            }).then(function (response) {

                widget.currentRow.changed = false;
                widget.view();
                ScreenService.closeSwipe();
                widget.reload();
            }).catch(function (e) {
                ScreenService.showMessage(e.data.error, "ERROR");
            });
        }
    };

    this.setFieldValue02500 = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NRESTABELEC':
                    field.setValue('');
                    field.widget.currentRow.CDEMPRESA = "";
                    field.widget.getField('DSSERIE').value('');
                    field.widget.getField('DSSERIE').reload();
                    break;
                default:
                    field.setValue('');
            }
        }
    };

    this.infoEstabelecToSerie = function (widget) {
        var estabelec = widget.getField("NRESTABELEC");

        if (estabelec.value()) {
            restEngine.post("cnc#/cnc00200_adquirentes_getInscValues", {
                "requestType": "FilterData",
                "filter": [{
                    "name": "NRESTABELEC",
                    "operator": "=",
                    "value": widget.currentRow.NRESTABELEC
                }]
            }).then(function (response) {
                var inscs = response.dataset.cnc00200_adquirentes_getInscValues.shift();
                widget.currentRow.CDINSCMUNI = inscs.CDINSCMUNI;
                widget.currentRow.CDINSCESTA = inscs.CDINSCESTA;
                widget.currentRow.CDEMPRESA = inscs.CDEMPRESA;

                _this11.completeSerieFilter(widget);
            });
        }
    };

    this.salvarCadastro02500 = function (widget) {
        var NRESTABELEC = widget.getField('NRESTABELEC').value();
        var DSSERIE = widget.getField('DSSERIE').value();
        widget.currentRow.__is_new = false;
        var promisesArr = [];

        if (widget.isValid()) {
            restEngine.post("cnc#/cnc02500_serie_estabelecimento_cad_check", {
                "requestType": "FilterData",
                "filter": [{
                    "name": "NRESTABELEC",
                    "operator": "=",
                    "value": NRESTABELEC
                }, {
                    "name": "DSSERIE",
                    "operator": "=",
                    "value": DSSERIE
                }]
            }).then(function (logDeResposta) {
                ScreenService.confirmMessage('Ao realizar a operação parametrizações existentes serão sobrepostas. Deseja continuar?', 'question', function () {
                    for (var i = 0; i < NRESTABELEC.length; i++) {
                        widget.currentRow.NRESTABELEC = NRESTABELEC[i];
                        promisesArr = [widget.saveRow()];
                    }
                    ZHPromise.all(promisesArr).then(function (response) {
                        widget.currentRow.changed = false;
                        ScreenService.closePopup();
                        widget.container.getWidget('estabelecGrid').reload();
                        widget.currentRow.CDEMPRESA = "";
                        widget.currentRow.CDSERIE = "";
                        widget.getField('NRESTABELEC').setValue('');
                        widget.getField('DSSERIE').setValue('');
                        widget.getField('DSSERIE').dataSource.data = [];
                        widget.getField('DSSERIE').reload();
                        var widtemp = templateManager.container.getWidget('estabelecGrid');
                        widtemp.activate();
                        UtilitarioGeral.showErrorMessages(widtemp, logDeResposta.dataset.log, "Log de Cadastro");
                        // widtemp.actions[0].events[0].code = "ScreenService.closePopup(); "
                        // widget.container.getWidget('estabelecGrid').activate();
                    });
                }, function () {});
            });
        }
    };

    this.checkSerieEstabelec = function (NRESTABELEC, DSSERIE) {
        restEngine.post("cnc#/cnc02500_serie_estabelecimento_cad_check", {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRESTABELEC",
                "operator": "=",
                "value": NRESTABELEC
            }, {
                "name": "DSSERIE",
                "operator": "=",
                "value": DSSERIE
            }]
        }).then(function (response) {
            return response;
        });
    };

    this.completeSerieFilter = function (widget) {
        var serie = widget.getField("DSSERIE");

        var filterMethod = function filterMethod(needle) {
            return function (obj) {
                return obj.name == needle;
            };
        };

        var mapMethod = function mapMethod(propName) {
            return function (obj) {
                return obj.value = widget.currentRow[propName];
            };
        };

        ['CDINSCESTA', 'CDINSCMUNI', 'CDEMPRESA'].map(function (filterName) {
            serie.dataSourceFilter.filter(filterMethod(filterName)).map(mapMethod(filterName));
        });

        serie.dataSource.load(serie.dataSourceFilter).then(function (response) {
            if (response.dataset.cnc02500_serie_estabelecimento.length == 0) ScreenService.showMessage("Não há Serie(s) cadastrada(s) no(s) Estabelecimento(s) Selecionado(s)");
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc02500_serie_estabelecimento_Controller', Cnc02500_serie_estabelecimento_Controller);
});

// FILE: ./assets/js/controllers/Cnc02600_contcorEstabelecimento_Controller.js
function Cnc02600_contcorEstabelecimento_Controller(ScreenService, templateManager, Filter, maskEngine, ZhUtil, restEngine, ZHPromise) {
    var _this12 = this;

    var self = this;

    this.getWidget = function (args) {
        return args.owner;
    };

    this.editEstabAdqProd = function (args) {
        var widget = this.getWidget(args).widget;
        var bancoField = widget.getField('NMBANCO');
        var agenciaField = widget.getField('NMAGENCIA');
        var contaField = widget.getField('DSCONTCORR');
        widget.edit();
        if (widget.currentRow.CDBANCO != null) {
            delete agenciaField.readOnly;
            delete contaField.readOnly;
        } else {
            agenciaField.readOnly = true;
            contaField.readOnly = true;
        }
        bancoField.value(widget.currentRow.NMBANCO);
        agenciaField.value(widget.currentRow.NMAGENCIA);
        contaField.value(widget.currentRow.DSCONTCORR);
        widget.backUpRow = Util.clone(widget.currentRow);
        if (widget.widgets[0]) widget.widgets[0].enabled = false;
    };

    this.cancelEditEstabAdqProd = function (args) {
        var widget = this.getWidget(args).widget;

        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backUpRow);
            widget.view();
        }
        if (widget.widgets[0]) widget.widgets[0].enabled = true;
    };

    this.saveEstabAdqProd = function (args) {
        var widget = this.getWidget(args).widget;
        var NRESTABELEC = widget.container.getWidget('estabelecGrid').currentRow.NRESTABELEC;
        var NRADQUIRENTE = widget.container.getWidget('adquirentes').currentRow.NRADQUIRENTE;

        if (widget.isValid()) {
            if (widget.name == "adquirentes") {
                widget.currentRow.NRESTABELEC = NRESTABELEC;
            }
            if (widget.name == "produtosGrid") {
                widget.currentRow.NRESTABELEC = NRESTABELEC;
                widget.currentRow.NRADQUIRENTE = NRADQUIRENTE;
            }
            widget.saveRow().then(function (response) {
                if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;

                widget.currentRow.changed = false;
                if (widget.widgets[0]) widget.widgets[0].enabled = true;
                widget.view();
                ScreenService.closeSwipe();
                widget.reload();
            }).catch(function (e) {
                ScreenService.showMessage(e.data.error, "ERROR");
            });
        }
    };

    this.removerEstabAdqProd = function (widget) {
        ScreenService.confirmMessage('Limpar dados da Conta Corrente?', 'question', function () {
            widget.dataSource.remove([widget.currentRow]).then(function (message) {
                ScreenService.successNotification('Removido com sucesso!');
                widget.view();
                ScreenService.closeSwipe();
                widget.reload();
            });
        }, function () {});
    };

    this.validateSetDataSourceFilterBank = function (args) {
        var widget = args.owner.field.widget;
        var datesEstabelec = widget.getField('NRESTABELEC').dataSource.data;
        var valuesChecked = datesEstabelec.filter(function (row) {
            return row.__isSelected == true;
        });
        var isEqual = true;
        if (valuesChecked.length == 0) {
            ScreenService.showMessage("Favor selecionar o(s) Estabelecimento(s).");
        } else {
            valuesChecked.forEach(function (row, index) {
                if (index != 0) {
                    if (valuesChecked[0].CDEMPRESA != row.CDEMPRESA) isEqual = false;
                }
            });
            if (isEqual == false) {
                ScreenService.showMessage("Não é possível pesquisar os bancos pois os estabelecimentos selecionados são de empresas diferentes.");
            } else {
                var CDEMPRESA = valuesChecked.shift().CDEMPRESA;
                widget.currentRow.CDEMPRESA = CDEMPRESA;
                widget.getField('NMBANCO').dataSourceFilter[1].value = CDEMPRESA;
                widget.getField('NMBANCO').dataSource.load(widget.getField('NMBANCO').dataSourceFilter).then(function (dateBank) {
                    if (dateBank.dataset.cnc02600_estabelecimento_banco == 0) {
                        ScreenService.showMessage("Não é possível pesquisar os bancos pois as empresas dos estabelecimentos selecionados não possuem contas correntes cadastradas.");
                    } else {
                        if (!widget.currentRow.CDBANCO) {
                            restEngine.post('cnc#/cnc02600_estabelecimento_agencia', {
                                'requestType': 'FilterData',
                                'filter': [{
                                    'name': 'CDBANCO',
                                    'operator': '=',
                                    'value': widget.currentRow.CDBANCO
                                }, {
                                    'name': 'CDEMPRESA',
                                    'operator': '=',
                                    'value': CDEMPRESA
                                }]
                            }).then(function (response) {
                                widget.getField('NMAGENCIA').dataSource.data = response.dataset.dataSet;
                                self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
                                    widget.getField('NMAGENCIA').dataSource.data.forEach(function (row, index) {
                                        row.CDAGENCIA = Util.applyMask(row.CDAGENCIA, { type: "fix", params: { mask: maskObject.CDPICTAGEN } }, maskEngine, ZhUtil);
                                    });
                                });
                            });
                            restEngine.post('cnc#/cnc02600_estabelecimento_conta', {
                                'requestType': 'FilterData',
                                'filter': [{
                                    'name': 'CDBANCO',
                                    'operator': '=',
                                    'value': widget.currentRow.CDBANCO
                                }, {
                                    'name': 'CDAGENCIA',
                                    'operator': '=',
                                    'value': widget.currentRow.CDAGENCIA
                                }, {
                                    'name': 'CDEMPRESA',
                                    'operator': '=',
                                    'value': widget.currentRow.CDEMPRESA
                                }]
                            }).then(function (response) {
                                widget.getField('DSCONTCORR').dataSource.data = response.dataset.dataSet;
                                self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
                                    widget.getField('DSCONTCORR').dataSource.data.forEach(function (row, index) {
                                        row.CDCONTCORR = Util.applyMask(row.CDCONTCORR, { type: "fix", params: { mask: maskObject.CDPICTCONT } }, maskEngine, ZhUtil);
                                    });
                                });
                            });
                        }
                    }
                });
            }
        }
    };

    this.enableAndFilterAgenAut = function (args) {
        var widget = args.owner.field.widget;
        var CDEMPRESA = widget.currentRow.CDEMPRESA;
        var bancoField = widget.getField('NMBANCO');
        var agenciaField = widget.getField('NMAGENCIA');
        var contaField = widget.getField('DSCONTCORR');
        var redu = widget.getField("CDREDUCONT");

        agenciaField.readOnly = true;
        agenciaField.value('');
        contaField.readOnly = true;
        contaField.value('');
        redu.value("");

        if (bancoField.value() != '') {
            delete agenciaField.readOnly;
            restEngine.post('cnc#/cnc02600_estabelecimento_agencia', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': widget.currentRow.CDBANCO
                }, {
                    'name': 'CDEMPRESA',
                    'operator': '=',
                    'value': CDEMPRESA
                }]
            }).then(function (response) {
                self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
                    agenciaField.fields[0].mask["params"] = { mask: maskObject.CDPICTAGEN };
                    agenciaField.dataSource.data = response.dataset.dataSet;
                    agenciaField.dataSource.data.forEach(function (row, index) {
                        row.CDAGENCIA = Util.applyMask(row.CDAGENCIA, { type: "fix", params: { mask: maskObject.CDPICTAGEN } }, maskEngine, ZhUtil);
                    });
                });
            });
        }
    };

    this.enableAndFilterContAut = function (args) {
        var widget = args.owner.field.widget;
        var CDEMPRESA = widget.currentRow.CDEMPRESA;
        var agenciaField = widget.getField('NMAGENCIA');
        var contaField = widget.getField('DSCONTCORR');
        var redu = widget.getField("CDREDUCONT");

        contaField.readOnly = true;
        contaField.value('');
        redu.value("");

        if (agenciaField.value() != '') {
            contaField.readOnly = false;
            restEngine.post('cnc#/cnc02600_estabelecimento_conta', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': widget.currentRow.CDBANCO
                }, {
                    'name': 'CDAGENCIA',
                    'operator': '=',
                    'value': widget.currentRow.CDAGENCIA.replace(/\D/g, '')
                }, {
                    'name': 'CDEMPRESA',
                    'operator': '=',
                    'value': CDEMPRESA
                }]
            }).then(function (response) {
                self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
                    contaField.dataSource.data = response.dataset.dataSet;
                    contaField.dataSource.data.map(function (row) {
                        var groups = maskObject.CDPICTCONT.split("-");
                        var mask = groups.reduce(function (acc, charSet) {
                            return acc + '(\\d{' + charSet.length + '})';
                        }, "^") + '.*';
                        var format = groups.length == 1 ? "$1" : "$1-$2";

                        row.CDCONTCORR = '' + row.CDCONTCORR.replace(new RegExp(mask), format);
                    });
                });
            });
        } else {
            contaField.readOnly = true;
            contaField.value('');
            redu.value("");
            self.enableAndFilterAgenAut(args);
        }
    };

    this.setFieldValue02600 = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NRESTABELEC':
                    field.setValue('');
                    field.widget.currentRow.CDEMPRESA = "";
                    field.widget.getField('NMBANCO').setValue('');
                    field.widget.getField('NMBANCO').dataSource.data = [];
                    field.widget.getField('NMBANCO').dataSourceFilter[1].value = "";
                    field.widget.getField('NMBANCO').reload();
                    field.widget.getField('NMAGENCIA').setValue('');
                    field.widget.getField('NMAGENCIA').dataSource.data = [];
                    field.widget.getField('NMAGENCIA').reload();
                    field.widget.getField('NMAGENCIA').readOnly = true;
                    field.widget.getField('DSCONTCORR').setValue('');
                    field.widget.getField('DSCONTCORR').dataSource.data = [];
                    field.widget.getField('DSCONTCORR').reload();
                    field.widget.getField('DSCONTCORR').readOnly = true;
                    field.widget.getField('CDREDUCONT').setValue('');
                    break;
                case 'NRADQUIRENTE':
                    field.setValue('');
                    break;
                case 'NRPRODUTOADQ':
                    field.setValue('');
                    break;
                default:
                    field.setValue('');
            }
        }
    };

    this.setDataSourceFilterBank = function (widget) {
        var CDEMPRESA = widget.container.getWidget('estabelecGrid').currentRow.CDEMPRESA;
        widget.getField('NMBANCO').dataSourceFilter[1].value = CDEMPRESA;
        widget.getField('NMBANCO').reload();
        if (widget.currentRow.CDAGENCIA == null) {
            widget.getAction('delete').isVisible = false;
        } else {
            widget.getAction('delete').isVisible = true;
        }
        if (widget.currentRow.CDBANCO != null) {
            restEngine.post('cnc#/cnc02600_estabelecimento_agencia', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': widget.currentRow.CDBANCO
                }, {
                    'name': 'CDEMPRESA',
                    'operator': '=',
                    'value': CDEMPRESA
                }]
            }).then(function (response) {
                widget.getField('NMAGENCIA').dataSource.data = response.dataset.dataSet;
                self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
                    widget.getField('NMAGENCIA').dataSource.data.forEach(function (row, index) {
                        row.CDAGENCIA = Util.applyMask(row.CDAGENCIA, { type: "fix", params: { mask: maskObject.CDPICTAGEN } }, maskEngine, ZhUtil);
                    });
                });
            });
            restEngine.post('cnc#/cnc02600_estabelecimento_conta', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': widget.currentRow.CDBANCO
                }, {
                    'name': 'CDAGENCIA',
                    'operator': '=',
                    'value': widget.currentRow.CDAGENCIA
                }, {
                    'name': 'CDEMPRESA',
                    'operator': '=',
                    'value': widget.currentRow.CDEMPRESA
                }]
            }).then(function (response) {
                widget.getField('DSCONTCORR').dataSource.data = response.dataset.dataSet;
                self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
                    widget.getField('DSCONTCORR').dataSource.data.forEach(function (row, index) {
                        row.CDCONTCORR = Util.applyMask(row.CDCONTCORR, { type: "fix", params: { mask: maskObject.CDPICTCONT } }, maskEngine, ZhUtil);
                    });
                });
            });
        }
    };

    this.getAgenContMask = function (cdbanco) {
        return restEngine.post('cnc#/cnc01300_movimentacao_financeira_masks', {
            'requestType': 'FilterData',
            'filter': [{
                'name': 'CDBANCO',
                'operator': '=',
                'value': cdbanco
            }]
        }).then(function (response) {
            return response.dataset.dataSet[0];
        });
    };

    this.setDataSourceAqcuire = function (widget) {
        var NRESTABELEC = widget.container.getWidget('estabelecGrid').currentRow.NRESTABELEC;

        widget.dataSourceFilter[0].value = NRESTABELEC;
        widget.reload();
    };

    this.setDataSourceProduct = function (widget) {
        var NRESTABELEC = widget.container.getWidget('estabelecGrid').currentRow.NRESTABELEC;
        var NRADQUIRENTE = widget.container.getWidget('adquirentes').currentRow.NRADQUIRENTE;

        widget.dataSourceFilter[0].value = NRESTABELEC;
        widget.dataSourceFilter[1].value = NRADQUIRENTE;
        widget.reload();
    };

    this.enableAndFilterAgen = function (args) {
        var widget = args.owner.field.widget;
        var CDEMPRESA = widget.container.getWidget('estabelecGrid').currentRow.CDEMPRESA;
        var bancoField = widget.getField('NMBANCO');
        var agenciaField = widget.getField('NMAGENCIA');
        var contaField = widget.getField('DSCONTCORR');
        var redu = widget.getField("CDREDUCONT");

        agenciaField.readOnly = true;
        agenciaField.value('');
        contaField.readOnly = true;
        contaField.value('');
        redu.value("");

        if (bancoField.value() != '') {
            delete agenciaField.readOnly;
            restEngine.post('cnc#/cnc02600_estabelecimento_agencia', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': widget.currentRow.CDBANCO
                }, {
                    'name': 'CDEMPRESA',
                    'operator': '=',
                    'value': CDEMPRESA
                }]
            }).then(function (response) {
                self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
                    agenciaField.fields[0].mask["params"] = { mask: maskObject.CDPICTAGEN };
                    agenciaField.dataSource.data = response.dataset.dataSet;
                    agenciaField.dataSource.data.forEach(function (row, index) {
                        row.CDAGENCIA = Util.applyMask(row.CDAGENCIA, { type: "fix", params: { mask: maskObject.CDPICTAGEN } }, maskEngine, ZhUtil);
                    });
                });
            });
        }
    };

    this.enableAndFilterCont = function (args) {
        var widget = args.owner.field.widget;
        var CDEMPRESA = widget.container.getWidget('estabelecGrid').currentRow.CDEMPRESA;
        var agenciaField = widget.getField('NMAGENCIA');
        var contaField = widget.getField('DSCONTCORR');
        var redu = widget.getField("CDREDUCONT");

        contaField.readOnly = true;
        contaField.value('');
        redu.value("");

        if (agenciaField.value() != '') {
            contaField.readOnly = false;
            restEngine.post('cnc#/cnc02600_estabelecimento_conta', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': widget.currentRow.CDBANCO
                }, {
                    'name': 'CDAGENCIA',
                    'operator': '=',
                    'value': widget.currentRow.CDAGENCIA.replace(/\D/g, '')
                }, {
                    'name': 'CDEMPRESA',
                    'operator': '=',
                    'value': CDEMPRESA
                }]
            }).then(function (response) {
                self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
                    contaField.dataSource.data = response.dataset.dataSet;
                    contaField.dataSource.data.map(function (row) {
                        var groups = maskObject.CDPICTCONT.split("-");
                        var mask = groups.reduce(function (acc, charSet) {
                            return acc + '(\\d{' + charSet.length + '})';
                        }, "^") + '.*';
                        var format = groups.length == 1 ? "$1" : "$1-$2";

                        row.CDCONTCORR = '' + row.CDCONTCORR.replace(new RegExp(mask), format);
                    });
                });
            });
        } else {
            contaField.readOnly = true;
            contaField.value('');
            redu.value("");
            self.enableAndFilterAgen(args);
        }
    };

    this.handleContDataSource = function (widget) {
        var contaField = widget.getField("DSCONTCORR");

        _this12.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
            contaField.dataSource.data.map(function (row) {
                var groups = maskObject.CDPICTCONT.split("-");
                var mask = groups.reduce(function (acc, charSet) {
                    return acc + '(\\d{' + charSet.length + '})';
                }, "^") + '.*';
                var format = groups.length == 1 ? "$1" : "$1-$2";

                row.CDCONTCORR = '' + row.CDCONTCORR.replace(new RegExp(mask), format);
            });
        });
    };

    this.filterCont = function (args) {
        var widget = args.owner.field.widget;
        var contaField = widget.getField('DSCONTCORR');

        if (contaField.value() === '') {
            self.enableAndFilterCont(args);
        }
    };

    this.handleReduContField = function (widget) {
        var cont = widget.getField("DSCONTCORR");
        var redu = widget.getField("CDREDUCONT");

        var getCdReduCont = function getCdReduCont(cdcont) {
            return cont.dataSource.data.filter(function (row) {
                return row.CDCONTCORR == cdcont;
            }).shift().CDREDUCONT;
        };

        redu.value(cont.value() ? getCdReduCont(widget.currentRow.CDCONTCORR) : "");
    };

    this.contCorrMaskOnChange = function (args) {
        var widget = args.owner.field.widget;

        self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
            widget.currentRow.CDCONTCORR = Util.applyMask(widget.currentRow.CDCONTCORR, { type: "fix", params: { mask: maskObject.CDPICTCONT } }, maskEngine, ZhUtil);
        });
    };

    this.agenciaMaskOnChange = function (args) {
        var widget = args.owner.field.widget;

        self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
            widget.currentRow.CDAGENCIA = Util.applyMask(widget.currentRow.CDAGENCIA, { type: "fix", params: { mask: maskObject.CDPICTAGEN } }, maskEngine, ZhUtil);
        });
    };

    this.salvarCadastro02600 = function (widget) {

        var NRESTABELEC = widget.getField('NRESTABELEC').value();
        var NRADQUIRENTE = widget.getField('NRADQUIRENTE').value();
        var NRPRODUTOADQ = widget.getField('NRPRODUTOADQ').value();

        if (widget.isValid()) {
            ScreenService.confirmMessage('Ao realizar a operação parametrizações existentes serão sobrepostas. Deseja continuar?', 'question', function () {
                var promisesArr = [];
                NRESTABELEC.forEach(function (rowNRESTABELEC, index) {
                    widget.currentRow.NRESTABELEC = rowNRESTABELEC;
                    NRADQUIRENTE.forEach(function (rowNRADQUIRENTE, index) {
                        widget.currentRow.NRADQUIRENTE = rowNRADQUIRENTE;
                        NRPRODUTOADQ.forEach(function (rowNRPRODUTOADQ, index) {
                            widget.currentRow.NRPRODUTOADQ = rowNRPRODUTOADQ;
                            promisesArr = [widget.saveRow()];
                        });
                    });
                });
                ZHPromise.all(promisesArr).then(function () {
                    widget.currentRow.changed = false;
                    ScreenService.closePopup();
                    widget.container.getWidget('estabelecGrid').reload();
                    widget.currentRow.CDEMPRESA = "";
                    widget.getField('NMBANCO').setValue('');
                    widget.getField('NMBANCO').dataSource.data = [];
                    widget.getField('NMBANCO').dataSourceFilter[1].value = "";
                    widget.getField('NMBANCO').reload();
                    widget.getField('NMAGENCIA').setValue('');
                    widget.getField('NMAGENCIA').dataSource.data = [];
                    widget.getField('NMAGENCIA').reload();
                    widget.getField('NMAGENCIA').readOnly = true;
                    widget.getField('DSCONTCORR').setValue('');
                    widget.getField('DSCONTCORR').dataSource.data = [];
                    widget.getField('DSCONTCORR').reload();
                    widget.getField('DSCONTCORR').readOnly = true;
                    widget.getField('CDREDUCONT').setValue('');
                    ScreenService.showMessage("Cadastro automático feito com Sucesso!");
                }).catch(function (e) {
                    ScreenService.showMessage(e.data.error, "ERROR");
                });
            }, function () {});
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc02600_contcorEstabelecimento_Controller', Cnc02600_contcorEstabelecimento_Controller);
});

// FILE: ./assets/js/controllers/Cnc02700_taxa_servico_estabelecimento_Controller.js
function Cnc02700_taxa_servico_estabelecimento_Controller(ScreenService, templateManager, Filter, maskEngine, ZhUtil, restEngine, ZHPromise) {

    var self = this;

    this.getWidget = function (args) {
        return args.owner;
    };

    this.onChangeReadOnlyFieldGroup = function (field) {
        var widget = field.widget;
        if (field.value() == 'S') {
            widget.getField('VRPERCTXADMSERV').isVisible = true;
            widget.getField('VRPERCTXADMSERVEDIT').isVisible = true;
            widget.getField('VRPERCTXADMSERVEDIT').value(widget.getField('VRPERCTXADMSERV').value());
            widget.getField('DSTPAJUCTTXADM').isVisible = true;
            widget.getField('DSTPAJUCTTXSRV').isVisible = true;
        } else {
            widget.getField('VRPERCTXADMSERV').isVisible = false;
            widget.getField('VRPERCTXADMSERV').value('');
            widget.getField('VRPERCTXADMSERVEDIT').isVisible = false;
            widget.getField('VRPERCTXADMSERVEDIT').value('');
            widget.getField('DSTPAJUCTTXADM').isVisible = false;
            widget.getField('DSTPAJUCTTXADM').value('');
            widget.getField('DSTPAJUCTTXSRV').isVisible = false;
            widget.getField('DSTPAJUCTTXSRV').value('');
        }
    };

    this.setValueTo2700 = function (args) {
        var widget = args.owner.widget;
        widget.container.getWidget('cadastroautomatico').getField('IDCTRLTXSERV').value('N');
        ScreenService.openPopup(widget.container.getWidget('cadastroautomatico'));
    };

    this.setFieldValue02700 = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NRESTABELEC':
                    field.setValue('');
                    break;
                default:
                    field.setValue('');
            }
        }
    };

    this.editTaxaServ = function (args) {
        var widget = this.getWidget(args).widget;
        var values = widget.currentRow;
        widget.edit();
        widget.backUpRow = Util.clone(values);
    };

    this.cancelEditTaxaServ = function (args) {
        var widget = this.getWidget(args).widget;

        Util.extend(widget.currentRow, widget.backUpRow);
        self.onChangeReadOnlyFieldGroup(widget.getField("IDCTRLTXSERV"));
        widget.view();
    };

    this.saveTaxaServ = function (args) {
        var widget = this.getWidget(args).widget;
        widget.currentRow.__is_new = false;
        if (widget.isValid()) {
            if (widget.currentRow.VRPERCTXADMSERVEDIT < 0) {
                ScreenService.showMessage('O valor da taxa administrativa deve ser maior que 0.', 'ERROR');
            } else if (widget.currentRow.VRPERCTXADMSERVEDIT > 100) {
                ScreenService.showMessage('A taxa administrativa máxima é de 100%.', 'ERROR');
            } else if (widget.currentRow.DSTPAJUCTTXADM == widget.currentRow.DSTPAJUCTTXSRV && widget.currentRow.DSTPAJUCTTXADM && widget.currentRow.DSTPAJUCTTXSRV) {
                ScreenService.showMessage('Os tipos de ajuste devem ser diferentes', 'ERROR');
            } else {
                widget.saveRow().then(function (response) {
                    widget.view();
                    widget.reload();
                    self.infoEstabelecToTaxaServ(widget);
                    ScreenService.successNotification('Salvo com sucesso!');
                }).catch(function (e) {
                    ScreenService.showMessage(e.data.error, "ERROR");
                });
            }
        }
    };

    this.infoEstabelecToTaxaServ = function (widget) {
        var estabelec = widget.getField("NRESTABELEC");

        if (estabelec.value()) {
            restEngine.post("cnc#/cnc02700_taxa_servico_estabelecimento_select_row", {
                "requestType": "FilterData",
                "filter": [{
                    "name": "NRESTABELEC",
                    "operator": "=",
                    "value": widget.currentRow.NRESTABELEC
                }]
            }).then(function (response) {
                var inscs = response.dataset.cnc02700_taxa_servico_estabelecimento_select_row.shift();
                widget.currentRow.IDCTRLTXSERV = inscs.IDCTRLTXSERV;
                widget.currentRow.VRPERCTXADMSERV = inscs.VRPERCTXADMSERV;
                widget.currentRow.DSTPAJUCTTXADM = inscs.DSTPAJUCTTXADM;
                widget.currentRow.DSTPAJUCTTXSRV = inscs.DSTPAJUCTTXSRV;

                self.onChangeReadOnlyFieldGroup(widget.getField("IDCTRLTXSERV"));
            });
        }
    };

    this.salvarCadastro02700 = function (widget) {

        var NRESTABELEC = widget.getField('NRESTABELEC').value();

        var row = widget.currentRow;

        if (widget.isValid()) {
            if (widget.currentRow.VRPERCTXADMSERVEDIT < 0) {
                ScreenService.showMessage('O valor da taxa administrativa deve ser maior que 0.', 'ERROR');
            } else if (widget.currentRow.VRPERCTXADMSERVEDIT > 100) {
                ScreenService.showMessage('A taxa administrativa máxima é de 100%.', 'ERROR');
            } else if (widget.currentRow.DSTPAJUCTTXADM == widget.currentRow.DSTPAJUCTTXSRV && widget.currentRow.DSTPAJUCTTXADM && widget.currentRow.DSTPAJUCTTXSRV) {
                ScreenService.showMessage('Os tipos de ajuste devem ser diferentes', 'ERROR');
            } else {
                ScreenService.confirmMessage('Ao realizar a operação parametrizações existentes serão sobrepostas. Deseja continuar?', 'question', function () {
                    var promisesArr = [];
                    NRESTABELEC.forEach(function (rowNRESTABELEC, index) {
                        widget.currentRow.NRESTABELEC = rowNRESTABELEC;
                        promisesArr = [widget.saveRow()];
                    });
                    ZHPromise.all(promisesArr).then(function () {
                        ScreenService.closePopup();
                        widget.getField('IDCTRLTXSERV').value('N');
                        self.onChangeReadOnlyFieldGroup(widget.getField('IDCTRLTXSERV'));
                        widget.container.getWidget('estabelecGrid').reload();
                        ScreenService.showMessage("Cadastro automático realizado com sucesso!");
                    }).catch(function (e) {
                        ScreenService.showMessage(e.data.error, "ERROR");
                    });
                }, function () {});
            }
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc02700_taxa_servico_estabelecimento_Controller', Cnc02700_taxa_servico_estabelecimento_Controller);
});

// FILE: ./assets/js/controllers/Cnc02900_RecebimentoDataCredito_Controller.js
function Cnc02900_RecebimentoDataCredito_Controller(ScreenService, templateManager, Filter, restEngine, maskEngine, ZhUtil) {

    var self = this;

    this.enableAndFilterAgen = function (args) {
        var bancoField = args.owner.field;
        var agenciaField = bancoField.widget.getField('CDAGENCIA');
        var contaField = bancoField.widget.getField('CDCONTCORR');

        agenciaField.readOnly = true;
        agenciaField.value('');
        contaField.readOnly = true;
        contaField.value('');

        if (bancoField.value() != '') {
            agenciaField.readOnly = false;
            restEngine.post('cnc#/cnc02900_recebimento_data_credito_getAgencias', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': bancoField.widget.currentRow.CDBANCO
                }]
            }).then(function (response) {
                self.getAgenContMask(bancoField.widget.currentRow.CDBANCO).then(function (maskObject) {
                    // agenciaField.fields[0].mask.params.mask = maskObject.CDPICTAGEN;
                    agenciaField.dataSource.data = response.dataset.dataSet;
                });
            });
        }
    };

    this.getAgenContMask = function (cdbanco) {
        return restEngine.post('cnc#/cnc02900_recebimento_data_credito_getBancoMask', {
            'requestType': 'FilterData',
            'filter': [{
                'name': 'CDBANCO',
                'operator': '=',
                'value': cdbanco
            }]
        }).then(function (response) {
            return response.dataset.dataSet[0];
        });
    };

    this.enableAndFilterCont = function (args) {
        var widget = templateManager.container.getWidget('recebimentoDtCredGrid');
        var agenciaField = args.owner.field;
        var contaField = agenciaField.widget.getField('CDCONTCORR');

        contaField.readOnly = true;
        contaField.value('');

        if (agenciaField.value() != '') {
            contaField.readOnly = false;
            restEngine.post('cnc#/cnc02900_recebimento_data_credito_getContas', {
                'requestType': 'FilterData',
                'filter': [{
                    'name': 'CDBANCO',
                    'operator': '=',
                    'value': agenciaField.widget.currentRow.CDBANCO
                }, {
                    'name': 'CDAGENCIA',
                    'operator': '=',
                    'value': agenciaField.widget.currentRow.CDAGENCIA
                }]
            }).then(function (response) {
                self.getAgenContMask(agenciaField.widget.currentRow.CDBANCO).then(function (maskObject) {
                    // contaField.fields[0].mask.params.mask = maskObject.CDPICTCONT;
                    contaField.dataSource.data = response.dataset.dataSet;
                });
            });
        } else {
            contaField.readOnly = true;
            contaField.value('');
        }
    };

    this.agenciaMaskOnChange = function (args) {
        var widget = templateManager.container.getWidget('recebimentoDtCredGrid');

        self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
            widget.currentRow.AGENCIAMASK = Util.applyMask(widget.currentRow.CDAGENCIA, { type: "fix", params: { mask: maskObject.CDPICTAGEN } }, maskEngine, ZhUtil);
        });
    };

    this.contCorrMaskOnChange = function (args) {
        var widget = templateManager.container.getWidget('recebimentoDtCredGrid');
        var contaField = args.owner.field;
        if (contaField.value() == '') {
            contaField.value('');
        } else {
            self.getAgenContMask(widget.currentRow.CDBANCO).then(function (maskObject) {
                widget.currentRow.CONTMASK = Util.applyMask(widget.currentRow.CDCONTCORR, { type: "fix", params: { mask: maskObject.CDPICTCONT } }, maskEngine, ZhUtil);
            });
        }
    };

    this.setFieldValueIn = function (field) {
        if (field.value().length == 0) {
            switch (field.name) {
                case 'NMADQUIRENTE':
                    field.setValue('');
                    break;
                case 'NMPRODUTOADQ':
                    field.setValue('');
                    break;
                case 'LABELESTABELEC':
                    field.setValue('');
                    break;
                default:
                    field.setValue('');
            }
        }
    };

    this.setStripeFooter = function (args) {
        var widgetGrid = templateManager.container.getWidget('stripe-footer');
        var widgetPrincipal = templateManager.container.getWidget('recebimentoDtCredGrid');

        var VRTOTAlBRUTO = 0;
        var VRTOTAlLIQ = 0;

        widgetPrincipal.dataSource.data.forEach(function (data) {
            VRTOTAlBRUTO += parseFloat(data.VRBRUTVENDA);
            VRTOTAlLIQ += parseFloat(data.VRLIQPGTO);
        });

        VRTOTAlBRUTO = "R$ " + VRTOTAlBRUTO.formatMoney(2, ",", ".");
        VRTOTAlLIQ = "R$ " + VRTOTAlLIQ.formatMoney(2, ",", ".");

        widgetGrid.currentRow.VRTOTAlBRUTO = VRTOTAlBRUTO;
        widgetGrid.currentRow.VRTOTAlLIQ = VRTOTAlLIQ;
    };

    this.getWidget = function (args) {
        return args.owner;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Cnc02900_RecebimentoDataCredito_Controller', Cnc02900_RecebimentoDataCredito_Controller);
});

// FILE: ./assets/js/controllers/cnc02800_data_inicio_integracao_Controller.js
function cnc02800_data_inicio_integracao_Controller(ScreenService, restEngine, ZHPromise, Filter, templateManager, localStorageEngine) {

    this.activateMainWidget = function (widget) {
        return widget.activate();
    };

    this.handleActionVisibility = function (widget, actionName) {
        var action = widget.actions.filter(function (action) {
            return action.name.includes(actionName);
        }).shift();
        action.isVisible = widget.dataSource.data.length > 0 ? true : false;
    };

    this.loadInfo = function (widget) {
        widget.view();
        widget.reload();
    };

    this.saveParams = function (widget) {

        var DTINICIOINT = widget.currentRow.DTINICIOINT;
        var CDOPERULTATU = widget.currentRow.CDOPERULTATU;

        if (widget.isValid()) {
            ScreenService.confirmMessage('Atenção! Ao alterar a data de inicio de integração, todas as vendas anteriores à data parametrizada não serão enviadas ao financeiro. Deseja continuar?', 'question', function () {
                widget.saveRow().then(function (response) {
                    if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;
                    widget.currentRow.changed = false;
                    widget.template = 'widget/form-view.html';
                    ScreenService.updateTemplate();
                    widget.view();
                    widget.reload();
                }).catch(function (e) {
                    ScreenService.showMessage(e.data.error);
                });
            }, function () {});
        }
    };

    this.cancel = function (widget) {
        widget.view();
        widget.template = 'widget/form-view.html';
        ScreenService.updateTemplate();
        widget.currentRow = widget.backupRow;
        widget.reload();
    };

    this.editParams = function (widget) {
        widget.edit();
        widget.template = 'widget/form.html';
        widget.backupRow = Util.clone(widget.currentRow);
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('cnc02800_data_inicio_integracao_Controller', cnc02800_data_inicio_integracao_Controller);
});

// FILE: ./assets/js/controllers/cnc03000_cadastro_token_Controller.js
function cnc03000_cadastro_token_Controller(ScreenService, Filter, templateManager, General) {

    this.newToken = function (args) {
        args.owner.widget.saveRow().then(function () {
            var currentRow = args.owner.widget.currentRow;
            currentRow.__is_new = false;
            currentRow.changed = false;
            args.owner.widget.reload();
            ScreenService.closeSwipe();
        }).catch(function (response) {
            if (response && response.error) {
                ScreenService.showMessage(ScreenService.i18n(response.error), 'E');
            }
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('cnc03000_cadastro_token_Controller', cnc03000_cadastro_token_Controller);
});
//# sourceMappingURL=app.js.map
